@@ -1,7 +1,7 @@
 /**
  * @file common.h common routines
  *
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2011 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,11 +27,10 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <glib.h>
-
-extern gboolean lifereaStarted;
+#include <string.h>
 
 /*
- * Standard gettext macros (as provided by glade-2).
+ * Standard gettext macros
  */
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -54,16 +53,6 @@ extern gboolean lifereaStarted;
 #  define N_(String) (String)
 #endif
 
-/** 
- * Conversion function which should be applied to all read XML strings, 
- * to ensure proper UTF8. doc points to the xml document and its encoding and
- * string is a xmlChar pointer to the read string. The result gchar
- * string is returned, the original XML string is freed. 
- *
- * @param string	possibly invalid UTF-8 string
- */
-gchar * common_utf8_fix(gchar * string);
-
 /**
  * Parses the given string as a number.
  *
@@ -72,60 +61,7 @@ gchar * common_utf8_fix(gchar * string);
  *
  * @returns result value
  */
-long common_parse_long(gchar *str, long def);
-
-/**
- * Returns a formatted date string for the given timestamp.
- *
- * @param t		the timestamp
- * @param date_format	a strptime format string (encoded in UTF-8)
- *
- * @returns a new formatted date string (encoded in UTF-8)
- */
-gchar * common_format_date(time_t date, const gchar *date_format);
-
-/**
- * Returns a formatted date string for the given timestamp.
- * In difference to common_format_date() it uses different
- * format string according to the time difference to today.
- *
- * @param t		the timestamp
- *
- * @returns a new formatted date string (encoded in UTF-8)
- */
-gchar * common_format_nice_date(time_t date);
-
-/**
- * Parses a ISO8601 date.
- *
- * @returns timestamp
- */
-time_t	parseISO8601Date(gchar *date);
-
-/**
- * Parses a RFC822 format date. This FAILS if a timezone string is
- * specified such as EDT or EST and that timezone is in daylight
- * savings time.
- *
- * @returns timestamp (GMT, no daylight savings time)
- */
-time_t	parseRFC822Date(gchar *date);
-
-/**
- * Creates a date string from the given timestamp.
- *
- * @param time	pointer to timestamp
- *
- * @returns time as string
- */
-gchar *createRFC822Date(const time_t *time);
-
-/**
- * FIXME: formatDate used by several functions not only
- * to format date column, don't use always date column format!!!
- * maybe gchar * formatDate(time_t, gchar *format) 
- */
-gchar * formatDate(time_t t);
+long common_parse_long (const gchar *str, long def);
 
 /**
  * Returns the cache file storage path.
@@ -147,22 +83,34 @@ const gchar *	common_get_cache_path(void);
 gchar * common_create_cache_filename(const gchar *folder, const gchar *filename, const gchar *extension);
 
 /**
- * Takes an URL and returns a new string containing the escaped URL.
+ * Takes an URI and returns a new string containing the escaped URI.
  *
- * @param url		the URL to escape
+ * @param uri		the URI to escape
  *
- * @returns new escaped URL string
+ * @returns new escaped URI string
  */
-xmlChar * common_uri_escape(const xmlChar *url);
+xmlChar * common_uri_escape(const xmlChar *uri);
 
 /**
- * Takes an URL and returns a new string containing the unescaped URL.
+ * Takes an URI and returns a new string containing the unescaped URI.
  *
- * @param url		the URL to unescape
+ * @param uri		the URI to unescape
  *
- * @returns new unescaped URL string
+ * @returns new unescaped URI string
  */
-xmlChar * common_uri_unescape(const xmlChar *url);
+xmlChar * common_uri_unescape(const xmlChar *uri);
+
+/**
+ * Takes an URI of uncertain safety (e.g. partially escaped) and
+ * returns if fully escaped safe for passing to a shell or browser. 
+ * This means the resulting URL is ensured to have no quotes, spaces 
+ * or &. Note: commata are not escaped!
+ *
+ * @param uri		the URI to escape
+ *
+ * @returns new escaped URI string
+ */
+xmlChar * common_uri_sanitize(const xmlChar *uri);
 
 /** 
  * To correctly escape and expand URLs, does not touch the
@@ -176,15 +124,21 @@ xmlChar * common_uri_unescape(const xmlChar *url);
 xmlChar * common_build_url(const gchar *url, const gchar *baseURL);
 
 /**
- * Analyzes the given string and returns the LTR/RTL
- * setting that should be used when displaying it in
- * a GTK tree store.
+ * Analyzes the string, returns a diection setting immediately
+ * useful for insertion into HTML
  *
  * @param text		string to analyze
  *
- * @returns UTF-8 string with direction mark
+ * @returns a constant "ltr" (default) or "rtl"
  */
-const gchar * common_get_direction_mark(gchar *text);
+const gchar * common_get_text_direction(const gchar *text);
+
+/**
+ * Returns the overall directionality of the application
+ *
+ * @returns a constant "ltr" (default) or "rtl"
+ */
+const gchar * common_get_app_direction(void);
 
 #ifndef HAVE_STRSEP
 char * common_strsep(char **stringp, const char *delim);
@@ -221,4 +175,25 @@ char * common_strcasestr(const char *phaystack, const char *pneedle);
  */
 time_t common_get_mod_time(const gchar *file);
 
+/**
+ * Create a localized filename.
+ *
+ * This function tries all applicable locale names and replaces
+ * the %s in str with the first one that points to an existing file.
+ * "en" is always among the searched locale names.
+ *
+ * @param str		full path containing one %s
+ *
+ * @returns		a string with %s replaced (to be freed by the caller), or NULL
+ */
+gchar *common_get_localized_filename (const gchar *str);
+
+/**
+ * Copy a source file to a target filename.
+ *
+ * @param src           absolute source file path
+ * @param dest          absolute destination file path
+ */
+void common_copy_file (const gchar *src, const gchar *dest);
+
 #endif
