@@ -1,7 +1,7 @@
 /**
  * @file db.h sqlite backend 
  * 
- * Copyright (C) 2007-2008  Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2011  Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include <glib.h>
 
 #include "item.h"
-#include "itemlist.h"
 #include "itemset.h"
 #include "subscription.h"
 #include "update.h"
@@ -32,13 +31,12 @@
 /**
  * Open and initialize the DB.
  */
-void db_init(void);
+void    db_init (void);
 
 /**
  * Clean up and close the DB.
  */
-void db_deinit(void);
-
+void    db_deinit (void);
 
 /* item set access (note: item sets are identified by the node id string) */
 
@@ -49,14 +47,14 @@ void db_deinit(void);
  *
  * @returns a newly allocated item set, must be freed using itemset_free()
  */
-itemSetPtr	db_itemset_load(const gchar *id);
+itemSetPtr	db_itemset_load (const gchar *id);
 
 /**
  * Removes all items of the given item set from the DB.
  *
  * @param id	the node id
  */
-void		db_itemset_remove_all(const gchar *id);
+void	db_itemset_remove_all (const gchar *id);
 
 /**
  * Mass items state changing methods. Mark all items of
@@ -64,8 +62,7 @@ void		db_itemset_remove_all(const gchar *id);
  *
  * @param id	the node id
  */
-void		db_itemset_mark_all_old(const gchar *id);
-void		db_itemset_mark_all_popup(const gchar *id);
+void	db_itemset_mark_all_popup (const gchar *id);
 
 /**
  * Returns the number of unread items for the given item set.
@@ -74,7 +71,7 @@ void		db_itemset_mark_all_popup(const gchar *id);
  *
  * @returns the number of unread items
  */
-guint		db_itemset_get_unread_count(const gchar *id);
+guint	db_itemset_get_unread_count (const gchar *id);
 
 /**
  * Returns the number of items for the given item set.
@@ -83,15 +80,22 @@ guint		db_itemset_get_unread_count(const gchar *id);
  *
  * @returns the number of items
  */
-guint		db_itemset_get_item_count(const gchar *id);
+guint   db_itemset_get_item_count (const gchar *id);
 
 /**
- * Calls the given callback for each item of the given node id.
+ * Returns a batch of items starting with the given
+ * id and no more than the given limit. If ids are not
+ * consecutive less ids than limit might be returned.
+ * 
+ * To be used for batched item loading (search folder loaders)
  *
- * @param id		the node id
- * @param callback	the callback
+ * @param itemSet       an itemset to add the items to
+ * @param id            the first id to fetch
+ * @param limit         maximum number of items to fetch
+ * 
+ * @returns FALSE if no more items to fetch
  */
-void		db_itemset_foreach (const gchar *id, itemActionFunc callback);
+gboolean        db_itemset_get (itemSetPtr itemSet, gulong id, guint limit);
 
 /* item access (note: items are identified by the numeric item id) */
 
@@ -144,126 +148,28 @@ GSList * db_item_get_duplicates(const gchar *guid);
 GSList * db_item_get_duplicate_nodes(const gchar *guid);
 
 /**
- * Explicitely start a transaction.
- */
-void	db_begin_transaction (void);
-
-/**
- * End previously started transaction.
- */
-void	db_end_transaction (void);
-
-/**
- * Commit current transaction.
- */
-void	db_commit_transaction (void);
-
-/**
- * Rollback current transaction.
- */
-void	db_rollback_transaction (void);
-
-/** Query table flags (to construct join expression for a query) */
-typedef enum {
-	QUERY_TABLE_ITEMS	= (1 << 0),
-	QUERY_TABLE_METADATA	= (1 << 1),
-	QUERY_TABLE_NODE	= (1 << 2)
-} queryTables;
-
-/** Query column flags (to construct column list) */
-typedef enum {
-	QUERY_COLUMN_ITEM_ID		= (1 << 0),
-	QUERY_COLUMN_ITEM_READ_STATUS	= (1 << 1)
-} queryColumns;
-
-/** Query info structure to be used with views and for dynamic item checks. */
-typedef struct query {
-	guint		tables;		/**< used tables, set of queryTable flags */
-	guint		columns;	/**< used columns, set of queryColumn flags */
-	
-	gchar 		*conditions;	/**< condition in SQL WHERE syntax */
-} *queryPtr;
-
-/**
- * Executes the passed matching query and checks if the
- * given item id matches the query.
+ * Returns an item set of all items for the given search folder id.
  *
- * @param id		the item id
- * @param query		query info for item check
- *
- * @returns TRUE if the item matches the query
- */
-gboolean db_item_check (guint id, const queryPtr query);
-
-/**
- * Creates a new temporary view (used for search folders)
- *
- * @param id		the view id
- * @param query		query info to construct view
- */
-void db_view_create (const gchar *id, const queryPtr query);
-
-/**
- * Removes a temparory view with the given id from the DB session
- *
- * @param id		the view id
- */
-void db_view_remove (const gchar *id);
-
-/**
- * Returns an item set of all items for the given view id.
- *
- * @param id		the view id
+ * @param id		the search folder id
  *
  * @returns a new item set (to be free'd using itemset_free())
  */
-itemSetPtr db_view_load (const gchar *id);
+itemSetPtr      db_search_folder_load (const gchar *id);
 
 /**
- * Checks wether the given item id belongs to the item list
- * of the given view id.
+ * Removes all items from the given search folder
  *
- * @param id		the view id
- * @param itemId	the item id
- *
- * @returns TRUE if the item belongs to the view
+ * @param id		the search folder id
  */
-gboolean db_view_contains_item (const gchar *id, gulong itemId);
-
-/** 
- * Returns the item count for the given view id.
- *
- * @param id		the view id
- *
- * @returns the number of items in the view
- */
-guint db_view_get_item_count (const gchar *id);
-
-/** 
- * Returns the unread item count for the given view id.
- *
- * @param id		the view id
- *
- * @returns the number of unread items in the view
- */
-guint db_view_get_unread_count (const gchar *id);
+void    db_search_folder_reset (const gchar *id);
 
 /**
- * Loads the feed state for the given feed from the DB
+ * Add a list of item ids to a search folder.
  *
- * @param id		the node id
- * @param updateState	update state structure to fill
- *
- * @returns TRUE if update state could be loaded
+ * @param id            the search folder id
+ * @param items         the list of items
  */
-gboolean db_update_state_load (const gchar *id, updateStatePtr updateState);
-
-/**
- * Updates all attributes and state of the feed in the DB
- *
- * @param id		the node id
- */
-void db_update_state_save (const gchar *id, updateStatePtr updateState);
+void    db_search_folder_add_items (const gchar *id, GSList *items);
 
 /**
  * Load the metadata and update state of the given subscription.
@@ -286,13 +192,6 @@ void db_subscription_update (subscriptionPtr subscription);
  */
 void db_subscription_remove (const gchar *id);
 
-/**
- * Loads the node with the given node id from the DB.
- *
- * @param id		the node id
- */
-void db_node_load (const gchar *id);
-
 /**
  * Updates the given nodes properties in the DB.
  *
@@ -300,11 +199,4 @@ void db_node_load (const gchar *id);
  */
 void db_node_update (nodePtr node);
 
-/**
- * Remove the given node.
- *
- * @param id		the node id
- */
-void db_node_remove (nodePtr node);
-
 #endif
