@@ -1,7 +1,7 @@
 /**
  * @file feed.c  feed node and subscription type
  * 
- * Copyright (C) 2003-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2011 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,9 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include "feed.h"
 
 #include <string.h>
 
@@ -30,18 +28,17 @@
 #include "db.h"
 #include "debug.h"
 #include "favicon.h"
-#include "feed.h"
 #include "feedlist.h"
 #include "itemlist.h"
 #include "metadata.h"
 #include "node.h"
 #include "render.h"
-#include "script.h"
 #include "update.h"
 #include "xml.h"
-#include "ui/liferea_shell.h"
 #include "ui/auth_dialog.h"
-#include "ui/ui_subscription.h"
+#include "ui/icons.h"
+#include "ui/liferea_shell.h"
+#include "ui/subscription_dialog.h"
 #include "ui/ui_node.h"
 #include "notification/notification.h"
 
@@ -82,23 +79,12 @@ feed_import (nodePtr node, nodePtr parent, xmlNodePtr xml, gboolean trusted)
 		feed->cacheLimit = common_parse_long (cacheLimitStr, CACHE_DEFAULT);
 	xmlFree (cacheLimitStr);
 	
-	tmp = xmlGetProp (xml, BAD_CAST"noIncremental");
-	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
-		feed->noIncremental = TRUE;
-	xmlFree (tmp);
-	
 	/* enclosure auto download flag */
 	tmp = xmlGetProp (xml, BAD_CAST"encAutoDownload");
 	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
 		feed->encAutoDownload = TRUE;
 	xmlFree (tmp);
 			
-	/* auto item link loading flag */
-	tmp = xmlGetProp (xml, BAD_CAST"loadItemLink");
-	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
-		feed->loadItemLink = TRUE;
-	xmlFree (tmp);
-
 	/* comment feed handling flag */
 	tmp = xmlGetProp (xml, BAD_CAST"ignoreComments");
 	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
@@ -156,15 +142,9 @@ feed_export (nodePtr node, xmlNodePtr xml, gboolean trusted)
 		if (cacheLimit)
 			xmlNewProp (xml, BAD_CAST"cacheLimit", BAD_CAST cacheLimit);
 
-		if (feed->noIncremental)
-			xmlNewProp (xml, BAD_CAST"noIncremental", BAD_CAST"true");
-			
 		if (feed->encAutoDownload)
 			xmlNewProp (xml, BAD_CAST"encAutoDownload", BAD_CAST"true");
 			
-		if (feed->loadItemLink)
-			xmlNewProp (xml, BAD_CAST"loadItemLink", BAD_CAST"true");
-			
 		if (feed->ignoreComments)
 			xmlNewProp (xml, BAD_CAST"ignoreComments", BAD_CAST"true");
 			
@@ -217,7 +197,6 @@ feed_to_xml (nodePtr node, xmlNodePtr feedNode)
 	
 	if (!feedNode) {
 		doc = xmlNewDoc ("1.0");
-		feedNode = xmlDocGetRootElement (doc);
 		feedNode = xmlNewDocNode (doc, NULL, "feed", NULL);
 		xmlDocSetRootElement (doc, feedNode);
 	}
@@ -229,11 +208,13 @@ feed_to_xml (nodePtr node, xmlNodePtr feedNode)
 guint
 feed_get_max_item_count (nodePtr node)
 {
+	gint	default_max_items;
 	feedPtr	feed = (feedPtr)node->data;
 	
 	switch (feed->cacheLimit) {
 		case CACHE_DEFAULT:
-			return conf_get_int_value (DEFAULT_MAX_ITEMS);
+			conf_get_int_value (DEFAULT_MAX_ITEMS, &default_max_items);
+			return default_max_items;
 			break;
 		case CACHE_DISABLE:
 		case CACHE_UNLIMITED:
@@ -301,20 +282,18 @@ feed_process_update_result (subscriptionPtr subscription, const struct updateRes
 				db_subscription_update (subscription);
 
 			liferea_shell_set_status_bar (_("\"%s\" updated..."), node_get_title (node));
-	
-			if (!feed->preventPopup)				
+
+			if (!feed->preventPopup)
 				notification_node_has_new_items (node, feed->enforcePopup);
 		}
-				
+
 		feed_free_parser_ctxt (ctxt);
 	} else {
 		node->available = FALSE;
-		
+
 		liferea_shell_set_status_bar (_("\"%s\" is not available"), node_get_title (node));
 	}
 
-	script_run_for_hook (SCRIPT_HOOK_FEED_UPDATED);
-
 	debug_exit ("feed_process_update_result");
 }
 
@@ -341,7 +320,7 @@ feed_save (nodePtr node)
 }
 
 static void
-feed_update_unread_count (nodePtr node)
+feed_update_counters (nodePtr node)
 {
 	node->itemCount = db_itemset_get_item_count (node->id);
 	node->unreadCount = db_itemset_get_unread_count (node->id);
@@ -350,23 +329,35 @@ feed_update_unread_count (nodePtr node)
 static void
 feed_remove (nodePtr node)
 {
-	notification_node_removed (node);
 	ui_node_remove_node (node);
 	
 	favicon_remove_from_cache (node->id);
 	db_subscription_remove (node->id);
 }
 
+static const gchar *
+feed_get_direction(nodePtr feed)
+{
+	if (node_get_title (feed))
+		return (common_get_text_direction (node_get_title (feed)));
+	else
+		return ("ltr");
+}
+
 static gchar *
 feed_render (nodePtr node)
 {
-	renderParamPtr	params;
 	gchar		*output = NULL;
 	xmlDocPtr	doc;
+	renderParamPtr	params;
+	const gchar     *text_direction = NULL;
+
+	text_direction = feed_get_direction (node);
+	params = render_parameter_new ();
+	render_parameter_add (params, "appDirection='%s'", common_get_app_direction ());
+	render_parameter_add (params, "txtDirection='%s'", text_direction);
 
 	doc = feed_to_xml (node, NULL);
-	params = render_parameter_new ();
-	render_parameter_add (params, "pixmapsDir='file://" PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "'");
 	output = render_xml (doc, "feed", params);
 	xmlFreeDoc (doc);
 
@@ -376,14 +367,14 @@ feed_render (nodePtr node)
 static gboolean
 feed_add (void)
 {
-	ui_subscription_dialog_new ();
+	subscription_dialog_new ();
 	return TRUE;
 }
 
 static void
 feed_properties (nodePtr node)
 {
-	ui_subscription_prop_dialog_new (node->subscription);
+	subscription_prop_dialog_new (node->subscription);
 }
 
 static void
@@ -393,7 +384,6 @@ feed_free (nodePtr node)
 
 	if (feed->parseErrors)
 		g_string_free (feed->parseErrors, TRUE);
-	g_free (feed->htmlUrl);
 	g_free (feed);
 }
 
@@ -413,21 +403,22 @@ feed_get_node_type (void)
 { 
 	static struct nodeType nti = {
 		NODE_CAPABILITY_SHOW_UNREAD_COUNT |
-		NODE_CAPABILITY_UPDATE,
+		NODE_CAPABILITY_UPDATE |
+		NODE_CAPABILITY_UPDATE_FAVICON,
 		"feed",		/* not used, feed format ids are used instead */
 		NULL,
 		feed_import,
 		feed_export,
 		feed_load,
 		feed_save,
-		feed_update_unread_count,
+		feed_update_counters,
 		feed_remove,
 		feed_render,
 		feed_add,
 		feed_properties,
 		feed_free
 	};
-	nti.icon = icons[ICON_DEFAULT];
+	nti.icon = icon_get (ICON_DEFAULT);
 	
 	return &nti; 
 }
