@@ -1,8 +1,10 @@
 /**
  * @file google_source.c  Google reader feed list source support
  * 
- * Copyright (C) 2007-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2011 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2008 Arnold Noronha <arnstein87@gmail.com>
+ * Copyright (C) 2011 Peter Oliver
+ * Copyright (C) 2011 Sergey Snitsaruk <narren96c@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,7 @@
 #include "debug.h"
 #include "feedlist.h"
 #include "item_state.h"
+#include "metadata.h"
 #include "node.h"
 #include "subscription.h"
 #include "update.h"
@@ -72,15 +75,6 @@ google_source_free (GoogleSourcePtr gsource)
 	g_free (gsource);
 }
 
-nodePtr
-google_source_get_root_from_node (nodePtr node)
-{ 
-	while (node->parent->source == node->source) 
-		node = node->parent;
-		
-	return node;
-}
-
 static void
 google_source_login_cb (const struct updateResult * const result, gpointer userdata, updateFlags flags)
 {
@@ -119,7 +113,7 @@ google_source_login_cb (const struct updateResult * const result, gpointer userd
 		subscription->updateError = g_strdup (_("Google Reader login failed!"));
 		gsource->loginState = GOOGLE_SOURCE_STATE_NONE;
 		
-		ui_auth_dialog_new (subscription, flags);
+		auth_dialog_new (subscription, flags);
 	}
 }
 
@@ -184,14 +178,13 @@ google_source_auto_update (nodePtr node)
 
 	g_get_current_time (&now);
 	
-	
 	/* do daily updates for the feed list and feed updates according to the default interval */
 	if (node->subscription->updateState->lastPoll.tv_sec + GOOGLE_SOURCE_UPDATE_INTERVAL <= now.tv_sec) {
 		google_source_update (node);
 		g_get_current_time (&gsource->lastQuickUpdate);
 	}
 	else if (gsource->lastQuickUpdate.tv_sec + GOOGLE_SOURCE_QUICK_UPDATE_INTERVAL <= now.tv_sec) {
-		google_source_quick_update (gsource);
+		google_source_opml_quick_update (gsource);
 		google_source_edit_process (gsource);
 		g_get_current_time (&gsource->lastQuickUpdate);
 	}
@@ -202,19 +195,29 @@ static void google_source_init (void) { }
 static void google_source_deinit (void) { }
 
 static void
-google_source_import (nodePtr node)
+google_source_import_node (nodePtr node)
 {
 	GSList *iter; 
+	for (iter = node->children; iter; iter = g_slist_next(iter)) {
+		nodePtr subnode = iter->data;
+		if (subnode->subscription)
+			subnode->subscription->type = &googleSourceFeedSubscriptionType; 
+		if (subnode->type->capabilities
+		    & NODE_CAPABILITY_SUBFOLDERS)
+			google_source_import_node (subnode);
+	}
+}
+
+static void
+google_source_import (nodePtr node)
+{
 	opml_source_import (node);
 	
 	node->subscription->type = &googleSourceOpmlSubscriptionType;
 	if (!node->data)
 		node->data = (gpointer) google_source_new (node);
 
-	for (iter = node->children; iter; iter = g_slist_next(iter))
-		((nodePtr) iter->data)->subscription->type = &googleSourceFeedSubscriptionType; 
-		
-	google_source_edit_import (node->data) ;
+	google_source_import_node (node);
 }
 
 static void
@@ -249,7 +252,7 @@ google_source_add_subscription (nodePtr node, subscriptionPtr subscription)
 	
 	node_set_title (child, _("New Subscription"));
 
-	google_source_edit_add_subscription (google_source_get_root_from_node (node)->data, subscription->source);
+	google_source_edit_add_subscription (node_source_root_from_node (node)->data, subscription->source);
 	
 	debug_exit ("google_source_add_subscription");
 	
@@ -272,7 +275,7 @@ google_source_remove_node (nodePtr node, nodePtr child)
 	feedlist_node_removed (child);
 
 	/* propagate the removal only if there aren't other copies */
-	if (!google_source_get_node_by_source (gsource, source)) 
+	if (!google_source_opml_get_node_by_source (gsource, source)) 
 		google_source_edit_remove_subscription (gsource, source);
 	
 	g_free (source);
@@ -298,7 +301,6 @@ on_google_source_selected (GtkDialog *dialog,
 		node_source_new (node, google_source_get_type ());
 		node_set_subscription (node, subscription);
 		node->data = google_source_new (node);
-		node->icon = ui_common_create_pixbuf ("fl_google.png");
 		feedlist_node_added (node);
 		google_source_update (node);
 	}
@@ -312,7 +314,7 @@ ui_google_source_get_account_info (void)
 	GtkWidget	*dialog;
 
 	
-	dialog = liferea_dialog_new ("google_source.glade", "google_source_dialog");
+	dialog = liferea_dialog_new ("google_source.ui", "google_source_dialog");
 	
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (on_google_source_selected), 
@@ -323,20 +325,42 @@ static void
 google_source_cleanup (nodePtr node)
 {
 	GoogleSourcePtr reader = (GoogleSourcePtr) node->data;
-	google_source_edit_export(reader);
 	google_source_free(reader);
 	node->data = NULL ;
 }
 
+static void 
+google_source_item_set_flag (nodePtr node, itemPtr item, gboolean newStatus)
+{
+	const gchar* sourceUrl = metadata_list_get (item->metadata, "GoogleBroadcastOrigFeed");
+	if (!sourceUrl)
+		sourceUrl = node->subscription->source;
+	nodePtr root = node_source_root_from_node (node);
+	google_source_edit_mark_starred ((GoogleSourcePtr)root->data, item->sourceId, sourceUrl, newStatus);
+	item_flag_state_changed (item, newStatus);
+}
+
+static void
+google_source_item_mark_read (nodePtr node, itemPtr item, gboolean newStatus)
+{
+	const gchar* sourceUrl = metadata_list_get(item->metadata, "GoogleBroadcastOrigFeed");
+	if (!sourceUrl)
+		sourceUrl = node->subscription->source;
+	nodePtr root = node_source_root_from_node (node);
+	google_source_edit_mark_read ((GoogleSourcePtr)root->data, item->sourceId, sourceUrl, newStatus);
+	item_read_state_changed (item, newStatus);
+}
+
 /* node source type definition */
 
 static struct nodeSourceType nst = {
-	.api_version         = NODE_SOURCE_TYPE_API_VERSION,
 	.id                  = "fl_google",
 	.name                = N_("Google Reader"),
 	.description         = N_("Integrate the feed list of your Google Reader account. Liferea will "
 	   "present your Google Reader subscriptions, and will synchronize your feed list and reading lists."),
-	.capabilities        = NODE_SOURCE_CAPABILITY_DYNAMIC_CREATION | NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST,
+	.capabilities        = NODE_SOURCE_CAPABILITY_DYNAMIC_CREATION | 
+	                       NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST |
+	                       NODE_SOURCE_CAPABILITY_ITEM_STATE_SYNC,
 	.source_type_init    = google_source_init,
 	.source_type_deinit  = google_source_deinit,
 	.source_new          = ui_google_source_get_account_info,
