@@ -1,7 +1,7 @@
 /**
  * @file opml_source.c  OPML Planet/Blogroll feed list source
  * 
- * Copyright (C) 2006-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2010 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,10 +31,8 @@
 #include "folder.h"
 #include "node.h"
 #include "xml.h"
+#include "ui/icons.h"
 #include "ui/liferea_dialog.h"
-#include "ui/ui_common.h"
-#include "ui/ui_feedlist.h"
-#include "ui/ui_node.h"
 
 /** default OPML update interval = once a day */
 #define OPML_SOURCE_UPDATE_INTERVAL 60*60*24
@@ -162,7 +160,7 @@ opml_subscription_process_update_result (subscriptionPtr subscription, const str
 	node->available = FALSE;
 
 	if (result->data) {
-		doc = xml_parse (result->data, result->size, FALSE, NULL);
+		doc = xml_parse (result->data, result->size, NULL);
 		if (doc) {
 			gchar *filename;
 			
@@ -190,7 +188,7 @@ opml_subscription_process_update_result (subscriptionPtr subscription, const str
 			if (g_str_equal (node_get_title (node), OPML_SOURCE_DEFAULT_TITLE)) {
 				title = xpath_find (root, "/opml/head/title"); 
 				if (title) {
-					xmlChar *titleStr = common_utf8_fix (xmlNodeListGetString(title->doc, title->xmlChildrenNode, 1));
+					xmlChar *titleStr = xmlNodeListGetString(title->doc, title->xmlChildrenNode, 1);
 					if (titleStr) {
 						node_set_title (node, titleStr);
 						xmlFree (titleStr);
@@ -240,10 +238,10 @@ opml_source_import (nodePtr node)
 
 	debug_enter ("opml_source_import");
 
-	filename = g_strdup_printf ("%s.png", NODE_SOURCE_TYPE (node)->id);
-	node->icon = ui_common_create_pixbuf (filename);
-	g_free (filename);
-	
+	/* We only ship an icon for opml, not for other sources */
+	if (g_str_equal (NODE_SOURCE_TYPE (node)->id, "fl_opml"))
+		node->icon = icon_create_from_file ("fl_opml.png");
+
 	debug1 (DEBUG_CACHE, "starting import of opml source instance (id=%s)", node->id);
 	filename = opml_source_get_feedlist (node);
 	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
@@ -323,7 +321,6 @@ static void opml_source_deinit(void) { }
 /* node source type definition */
 
 static struct nodeSourceType nst = {
-	NODE_SOURCE_TYPE_API_VERSION,
 	"fl_opml",
 	N_("Planet, BlogRoll, OPML"),
 	N_("Integrate blogrolls or Planets in your feed list. Liferea will "
@@ -387,7 +384,7 @@ on_opml_file_selected (const gchar *filename, gpointer user_data)
 static void
 on_opml_file_choose_clicked (GtkButton *button, gpointer user_data)
 {
-	ui_choose_file (_("Choose OPML File"), GTK_STOCK_OPEN, FALSE, on_opml_file_selected, NULL, NULL, user_data);
+	ui_choose_file (_("Choose OPML File"), GTK_STOCK_OPEN, FALSE, on_opml_file_selected, NULL, NULL, "*.opml", _("OPML Files"), user_data);
 }
 
 static void
@@ -395,7 +392,7 @@ ui_opml_source_get_source_url (void)
 {
 	GtkWidget	*dialog, *button;
 
-	dialog = liferea_dialog_new ("opml_source.glade", "opml_source_dialog");
+	dialog = liferea_dialog_new ("opml_source.ui", "opml_source_dialog");
 	button = liferea_dialog_lookup (dialog, "select_button");
 
 	g_signal_connect (G_OBJECT (dialog), "response",
