@@ -1,7 +1,7 @@
 /**
  * @file htmlview.c  item view interface for HTML rendering
  * 
- * Copyright (C) 2006-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2010 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,12 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
- 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
 
 #include <string.h>
+#include <libxml/uri.h>
 
 #include "common.h"
 #include "debug.h"
@@ -31,7 +28,6 @@
 #include "htmlview.h"
 #include "item.h"
 #include "itemlist.h"
-#include "itemset.h"
 #include "render.h"
 #include "vfolder.h"
 #include "ui/liferea_htmlview.h"
@@ -171,30 +167,17 @@ htmlview_update_all_items (void)
 	}
 }
 
-// FIXME: bad naming -> unclear concept!
-static xmlDocPtr
-itemset_to_xml (nodePtr node) 
+static const gchar *
+htmlview_get_item_direction(itemPtr item)
 {
-	xmlDocPtr 	doc;
-	xmlNodePtr 	itemSetNode;
-	
-	doc = xmlNewDoc ("1.0");
-	itemSetNode = xmlNewDocNode (doc, NULL, "itemset", NULL);
-	
-	xmlDocSetRootElement (doc, itemSetNode);
-	
-	xmlNewTextChild (itemSetNode, NULL, "favicon", node_get_favicon_file (node));
-	xmlNewTextChild (itemSetNode, NULL, "title", node_get_title (node));
+	if (item_get_title (item))
+		return (common_get_text_direction (item_get_title (item)));
 
-	if (node->subscription) {
-		if (subscription_get_source (node->subscription))
-			xmlNewTextChild (itemSetNode, NULL, "source", subscription_get_source (node->subscription));
-			
-		if (subscription_get_homepage (node->subscription))
-			xmlNewTextChild (itemSetNode, NULL, "link", subscription_get_homepage (node->subscription));
-	}
+	if (item_get_description (item))
+		return (common_get_text_direction (item_get_description (item)));
 
-	return doc;
+	/* what can we do? */
+	return ("ltr");
 }
 
 static gchar *
@@ -203,34 +186,45 @@ htmlview_render_item (itemPtr item,
                       gboolean summaryMode) 
 {
 	renderParamPtr	params;
-	gchar		*output = NULL, *baseUrl;
+	gchar		*output = NULL, *baseUrl = NULL;
 	nodePtr		node;
-	xmlDocPtr	doc;
+	xmlDocPtr 	doc;
+	xmlNodePtr 	xmlNode;
+	const gchar     *text_direction = NULL;
 
 	debug_enter ("htmlview_render_item");
 
 	/* don't use node from htmlView_priv as this would be
 	   wrong for folders and other merged item sets */
 	node = node_from_id (item->nodeId);
-	baseUrl = common_uri_escape (node_get_base_url (node));
 
 	/* do the XML serialization */
-	doc = itemset_to_xml (node);
-			
+	doc = xmlNewDoc ("1.0");
+	xmlNode = xmlNewDocNode (doc, NULL, "itemset", NULL);
+	xmlDocSetRootElement (doc, xmlNode);
+				
 	item_to_xml(item, xmlDocGetRootElement (doc));
+
+	text_direction = htmlview_get_item_direction (item);
 			
 	if (IS_FEED (node)) {
 		xmlNodePtr feed;
-		feed = xmlNewChild(xmlDocGetRootElement(doc), NULL, "feed", NULL);
-		feed_to_xml(node, feed);
+		feed = xmlNewChild (xmlDocGetRootElement (doc), NULL, "feed", NULL);
+		feed_to_xml (node, feed);
 	}
 	
 	/* do the XSLT rendering */
 	params = render_parameter_new ();
-	render_parameter_add (params, "pixmapsDir='file://" PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "'");
-	render_parameter_add (params, "baseUrl='%s'", baseUrl);
+	
+	if (NULL != node_get_base_url (node)) {
+		baseUrl = common_uri_escape (node_get_base_url (node));
+		render_parameter_add (params, "baseUrl='%s'", baseUrl);
+	}
+	
 	render_parameter_add (params, "summary='%d'", summaryMode?1:0);
 	render_parameter_add (params, "single='%d'", (viewMode == ITEMVIEW_SINGLE_ITEM)?1:0);
+	render_parameter_add (params, "txtDirection='%s'", text_direction);
+	render_parameter_add (params, "appDirection='%s'", common_get_app_direction ());
 	output = render_xml (doc, "item", params);
 	
 	/* For debugging use: xmlSaveFormatFile("/tmp/test.xml", doc, 1); */
@@ -242,8 +236,6 @@ htmlview_render_item (itemPtr item,
 	return output;
 }
 
-extern htmlviewImplPtr htmlview_get_impl();
-
 void 
 htmlview_start_output (GString *buffer,
                        const gchar *base,
