@@ -1,7 +1,7 @@
 /**
  * @file item.c common item handling
  *
- * Copyright (C) 2003-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2010 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *	      
  * This program is free software; you can redistribute it and/or modify
@@ -22,16 +22,15 @@
 #include "item.h"
 
 #include <glib.h>
-#include <time.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "comments.h"
 #include "common.h"
+#include "date.h"
 #include "db.h"
+#include "debug.h"
 #include "metadata.h"
 #include "xml.h"
-#include "ui/itemview.h"	// FIXME: evil
 
 itemPtr
 item_new (void)
@@ -123,10 +122,49 @@ const gchar *	item_get_title(itemPtr item) {return item->title; }
 const gchar *	item_get_description(itemPtr item) { return item->description; }
 const gchar *	item_get_source(itemPtr item) { return item->source; }
 
+gchar *
+item_make_link (itemPtr item)
+{
+	const gchar	*src;
+	gchar		*link;
+
+	src = item_get_source (item);
+	if (!src)
+		return NULL;
+
+	/* check for relative link */
+	if (*src == '/') {
+		const gchar * base = item_get_base_url (item);
+		gchar * pos = (gchar *)base;
+		int host_url_size, i;
+
+		/* Find the third /, start of link on
+		 * site. */
+		for (i = 0; pos && i < 3; i++) {
+			pos = strstr(pos + 1, "/");
+		}
+
+		if (!pos) {
+			debug0 (DEBUG_PARSING, "Feed contains relative link and invalid base URL");
+			return NULL;
+		}
+
+		host_url_size = pos - base + 1;
+
+		link = g_malloc (host_url_size + strlen(src));
+		strncpy (link, base, host_url_size - 1);
+		pos = link + host_url_size - 1;
+		strcpy (pos, src);
+	} else {
+		link = g_strdup (src);
+	}
+	
+	return link;
+}
+
 void
 item_unload (itemPtr item) 
 {
-
 	g_free (item->title);
 	g_free (item->source);
 	g_free (item->sourceId);
@@ -155,6 +193,7 @@ item_to_xml (itemPtr item, xmlNodePtr parentNode)
 	xmlNodePtr	duplicatesNode;		
 	xmlNodePtr	itemNode;
 	gchar		*tmp;
+	gchar		*tmp2;
 	
 	itemNode = xmlNewChild (parentNode, NULL, "item", NULL);
 	g_return_if_fail (itemNode);
@@ -163,8 +202,10 @@ item_to_xml (itemPtr item, xmlNodePtr parentNode)
 
 	if (item_get_description (item)) {
 		tmp = xhtml_strip_dhtml (item_get_description (item));
-		xmlNewTextChild (itemNode, NULL, "description", tmp);
+		tmp2 = xhtml_strip_unsupported_tags (tmp);
+		xmlNewTextChild (itemNode, NULL, "description", tmp2);
 		g_free (tmp);
+		g_free (tmp2);
 	}
 	
 	if (item_get_source (item))
@@ -190,7 +231,7 @@ item_to_xml (itemPtr item, xmlNodePtr parentNode)
 	xmlNewTextChild (itemNode, NULL, "time", tmp);
 	g_free (tmp);
 
-	tmp = itemview_format_date (item->time);
+	tmp = date_format (item->time, NULL);
 	xmlNewTextChild (itemNode, NULL, "timestr", tmp);
 	g_free (tmp);
 
