@@ -1,7 +1,7 @@
 /**
  * @file itemlist.c  itemlist handling
  *
- * Copyright (C) 2004-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2011 Lars Lindner <lars.lindner@gmail.com>
  *	      
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
  */
 
 #include <string.h>
+#include <glib.h>
 
 #include "comments.h"
 #include "common.h"
@@ -34,28 +35,35 @@
 #include "metadata.h"
 #include "node.h"
 #include "rule.h"
-#include "script.h"
 #include "vfolder.h"
+#include "ui/item_list_view.h"
 #include "ui/itemview.h"
 #include "ui/liferea_shell.h"
-#include "ui/ui_feedlist.h"
-#include "ui/ui_itemlist.h"
+#include "ui/feed_list_view.h"
 #include "ui/liferea_htmlview.h"
 #include "ui/ui_node.h"
 
-/* This is a simple controller implementation for itemlist handling. 
-   It manages the currently displayed itemset, realizes filtering,
-   duplicate elimination and provides synchronisation for backend 
-   and GUI access to this itemset.  
+/* The 'item list' is a controller for 'item view' and database backend. 
+   It manages the currently displayed 'node', realizes filtering
+   by node and 'item set' rules, also duplicate elimination and provides 
+   synchronisation for backend and GUI access to the current itemset.
+
+   The 'item list' provides methods to add/remove items from the 'item
+   view' synchronously and allows registering 'item loaders' for
+   asynchronous adding of item batches.
+
+   The 'item list' does not handle item list rendering variants, which
+   is done by the 'item view'.
  */
- 
-static struct itemlist_priv 
+
+#define ITEMLIST_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), ITEMLIST_TYPE, ItemListPrivate))
+
+struct ItemListPrivate
 {
 	GHashTable	*guids;			/**< list of GUID to avoid having duplicates in currently loaded list */
-	GSList	 	*filter;		/**< currently active filter rule */
+	itemSetPtr	filter;			/**< currently active filter rules */
 	nodePtr		currentNode;		/**< the node whose own or its child items are currently displayed */
 	gulong		selectedId;		/**< the currently selected (and displayed) item id */
-	gboolean	hasEnclosures;		/**< TRUE if at least one item of the current itemset has an enclosure */
 	
 	nodeViewType	viewMode;		/**< current viewing mode */
 	guint 		loading;		/**< if >0 prevents selection effects when loading the item list */
@@ -63,22 +71,20 @@ static struct itemlist_priv
 
 	gboolean 	deferredRemove;		/**< TRUE if selected item needs to be removed from cache on unselecting */
 	gboolean 	deferredFilter;		/**< TRUE if selected item needs to be filtered on unselecting */
-	
-	gboolean	isSearchResult;		/**< TRUE if a search result is displayed */
-	gboolean	searchResultComplete;	/**< TRUE if search result merging is complete */
-	
-} itemlist_priv;
+};
+
+static GObjectClass *parent_class = NULL;
+static ItemList *itemlist = NULL;
+
+G_DEFINE_TYPE (ItemList, itemlist, G_TYPE_OBJECT);
 
 static void
-itemlist_filter_free (void)
+itemlist_init (ItemList *il)
 {
-	GSList *iter = itemlist_priv.filter;
-	while (iter) {
-		rule_free (iter->data);
-		iter = g_slist_next (iter);
-	}
-	g_slist_free (itemlist_priv.filter);
-	itemlist_priv.filter = NULL;
+	/* 1. Prepare globally accessible singleton */
+	g_assert (NULL == itemlist);
+	itemlist = il;
+	itemlist->priv = ITEMLIST_GET_PRIVATE (il);
 }
 
 static void
@@ -86,9 +92,9 @@ itemlist_duplicate_list_remove_item (itemPtr item)
 {
 	if (!item->validGuid)
 		return;
-	if (!itemlist_priv.guids)
+	if (!itemlist->priv->guids)
 		return;
-	g_hash_table_remove (itemlist_priv.guids, item->sourceId);
+	g_hash_table_remove (itemlist->priv->guids, item->sourceId);
 }
 
 static void
@@ -96,66 +102,74 @@ itemlist_duplicate_list_add_item (itemPtr item)
 {
 	if (!item->validGuid)
 		return;
-	if (!itemlist_priv.guids)
-		itemlist_priv.guids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	g_hash_table_insert (itemlist_priv.guids, g_strdup (item->sourceId), GUINT_TO_POINTER (item->id));
+	if (!itemlist->priv->guids)
+		itemlist->priv->guids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	g_hash_table_insert (itemlist->priv->guids, g_strdup (item->sourceId), GUINT_TO_POINTER (item->id));
 }
 
 static gboolean
 itemlist_duplicate_list_check_item (itemPtr item)
 {
-	if (!itemlist_priv.guids || !item->validGuid)
+	if (!itemlist->priv->guids || !item->validGuid)
 		return TRUE;
 
-	return (NULL == g_hash_table_lookup (itemlist_priv.guids, item->sourceId));
+	return (NULL == g_hash_table_lookup (itemlist->priv->guids, item->sourceId));
 }
 
 static void
 itemlist_duplicate_list_free (void)
 {
-	if (itemlist_priv.guids) {
-		g_hash_table_destroy (itemlist_priv.guids);
-		itemlist_priv.guids = NULL;
+	if (itemlist->priv->guids) {
+		g_hash_table_destroy (itemlist->priv->guids);
+		itemlist->priv->guids = NULL;
 	}
 }
 
-void
-itemlist_free (void)
+static void
+itemlist_finalize (GObject *object)
 {
-	ui_itemlist_destroy (); 
-	
-	itemlist_filter_free ();
+	itemset_free (itemlist->priv->filter);
 	itemlist_duplicate_list_free ();
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+itemlist_class_init (ItemListClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = itemlist_finalize;
+
+	g_type_class_add_private (object_class, sizeof(ItemListPrivate));
+}
+
+/* member wrappers */
+
 itemPtr
 itemlist_get_selected (void)
 {
-	return item_load(itemlist_priv.selectedId);
+	return item_load(itemlist->priv->selectedId);
 }
 
 gulong
 itemlist_get_selected_id (void)
 {
-	return itemlist_priv.selectedId;
+	return itemlist->priv->selectedId;
 }
 
 static void
 itemlist_set_selected (itemPtr item)
 {
-	if(itemlist_priv.selectedId)
-		script_run_for_hook(SCRIPT_HOOK_ITEM_UNSELECT);
-
-	itemlist_priv.selectedId = item?item->id:0;
-		
-	if(itemlist_priv.selectedId)
-		script_run_for_hook(SCRIPT_HOOK_ITEM_SELECTED);
+	itemlist->priv->selectedId = item?item->id:0;
 }
 
 nodePtr
 itemlist_get_displayed_node (void)
 {
-	return itemlist_priv.currentNode; 
+	return itemlist->priv->currentNode; 
 }
 
 /* called when unselecting the item or unloading the item list */
@@ -164,21 +178,21 @@ itemlist_check_for_deferred_action (void)
 {
 	itemPtr	item;
 	
-	if(itemlist_priv.selectedId) {
-		gulong id = itemlist_priv.selectedId;
+	if(itemlist->priv->selectedId) {
+		gulong id = itemlist->priv->selectedId;
 		itemlist_set_selected(NULL);
 
 		/* check for removals caused by itemlist filter rule */
-		if(itemlist_priv.deferredFilter) {
-			itemlist_priv.deferredFilter = FALSE;
+		if(itemlist->priv->deferredFilter) {
+			itemlist->priv->deferredFilter = FALSE;
 			item = item_load(id);
 			itemview_remove_item(item);
 			ui_node_update(item->nodeId);
 		}
 
 		/* check for removals caused by vfolder rules */
-		if(itemlist_priv.deferredRemove) {
-			itemlist_priv.deferredRemove = FALSE;
+		if(itemlist->priv->deferredRemove) {
+			itemlist->priv->deferredRemove = FALSE;
 			item = item_load(id);
 			itemlist_remove_item(item);
 		}
@@ -190,14 +204,14 @@ static gboolean
 itemlist_filter_check_item (itemPtr item)
 {
 	/* use search folder rule list in case of a search folder */
-	if (itemlist_priv.currentNode && IS_VFOLDER (itemlist_priv.currentNode)) {
-		vfolderPtr vfolder = (vfolderPtr)itemlist_priv.currentNode->data;
-		return rules_check_item (vfolder->rules, vfolder->anyMatch, item);
+	if (itemlist->priv->currentNode && IS_VFOLDER (itemlist->priv->currentNode)) {
+		vfolderPtr vfolder = (vfolderPtr)itemlist->priv->currentNode->data;
+		return itemset_check_item (vfolder->itemset, item);
 	}
 
 	/* apply the item list filter if available */
-	if (itemlist_priv.filter)
-		return rules_check_item (itemlist_priv.filter, TRUE /* OR logic */, item);
+	if (itemlist->priv->filter)
+		return itemset_check_item (itemlist->priv->filter, item);
 	
 	/* otherwise keep the item */
 	return TRUE;
@@ -212,74 +226,56 @@ itemlist_merge_item (itemPtr item)
 	if (!itemlist_filter_check_item (item))
 		return;
 		
-	itemlist_priv.hasEnclosures |= item->hasEnclosure;
 	itemlist_duplicate_list_add_item (item);
 	itemview_add_item (item);
 }
 
+/* Helper method checking if the passed item set is relevant
+   for the currently item list content. */
+static gboolean
+itemlist_itemset_is_valid (itemSetPtr itemSet)
+{
+	gint	folder_display_mode;
+	nodePtr node;
+
+	node = node_from_id (itemSet->nodeId);
+
+	if (!itemlist->priv->currentNode)
+		return FALSE; /* Nothing to do if nothing is displayed */
+		
+	if (!IS_VFOLDER (itemlist->priv->currentNode) &&
+	    (itemlist->priv->currentNode != node) && 
+	    !node_is_ancestor (itemlist->priv->currentNode, node))
+		return FALSE; /* Nothing to do if the item set does not belong to this node, or this is a search folder */
+
+	conf_get_int_value (FOLDER_DISPLAY_MODE, &folder_display_mode);
+	if (IS_FOLDER (itemlist->priv->currentNode) && !folder_display_mode)
+		return FALSE; /* Bail out if it is a folder without the recursive display preference set */
+		
+	debug1 (DEBUG_GUI, "reloading item list with node \"%s\"", node_get_title (node));
+
+	return TRUE;
+}
+
 /**
  * To be called whenever an itemset was updated. If it is the
- * displayed itemset it will be merged against the item list
- * tree view.
+ * displayed itemset it will be merged against the item view.
  */
 void
 itemlist_merge_itemset (itemSetPtr itemSet) 
 {
 	debug_enter ("itemlist_merge_itemset");
 	
-	debug_start_measurement (DEBUG_GUI);
-	
-	/* No node check when loading search results directly */
-	if (!itemlist_priv.isSearchResult) {
-		nodePtr node = node_from_id (itemSet->nodeId);
-
-		if (!itemlist_priv.currentNode)
-			return; /* Nothing to do if nothing is displayed */
-		
-		if (!IS_VFOLDER (itemlist_priv.currentNode) &&
-		    (itemlist_priv.currentNode != node) && 
-		    !node_is_ancestor (itemlist_priv.currentNode, node))
-			return; /* Nothing to do if the item set does not belong to this node, or this is a search folder */
-
-		if (IS_FOLDER (itemlist_priv.currentNode) && 
-		    (0 == conf_get_int_value (FOLDER_DISPLAY_MODE)))
-			return; /* Bail out if it is a folder without the recursive display preference set */
-			
-		debug1 (DEBUG_GUI, "reloading item list with node \"%s\"", node_get_title (node));
-	} else {
-		/* If we are loading a search result we must never merge 
-		   anything besides the search items. In fact if we already
-		   have items we just return. */
-		if (itemlist_priv.searchResultComplete)
-			return;
-			
-		itemlist_priv.searchResultComplete = TRUE;
+	if (itemlist_itemset_is_valid (itemSet)) {
+		debug_start_measurement (DEBUG_GUI);
+		itemset_foreach (itemSet, itemlist_merge_item);
+		itemview_update ();
+		debug_end_measurement (DEBUG_GUI, "itemlist merge");
 	}
 
-	/* merge items into item view */
-	itemset_foreach (itemSet, itemlist_merge_item);
-	
-	if (itemlist_priv.hasEnclosures)
-		ui_itemlist_enable_encicon_column (TRUE);
-
-	itemview_update ();
-	
-	debug_end_measurement (DEBUG_GUI, "itemlist merge");
-
 	debug_exit ("itemlist_merge_itemset");
 }
 
-void
-itemlist_load_search_result (itemSetPtr itemSet)
-{
-	itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
-	
-	itemlist_priv.isSearchResult = TRUE;
-	itemlist_priv.searchResultComplete = FALSE;	/* enable result merging */
-	itemlist_merge_itemset (itemSet);	
-	itemlist_priv.searchResultComplete = TRUE;	/* disable result merging */
-}
-
 /** 
  * To be called whenever a node was selected and should
  * replace the current itemlist.
@@ -288,6 +284,8 @@ void
 itemlist_load (nodePtr node) 
 {
 	itemSetPtr	itemSet;
+	gint		folder_display_mode;
+	gboolean	folder_display_hide_read;
 
 	debug_enter ("itemlist_load");
 
@@ -295,10 +293,8 @@ itemlist_load (nodePtr node)
 	
 	debug1 (DEBUG_GUI, "loading item list with node \"%s\"", node_get_title (node));
 
-	g_assert (!itemlist_priv.guids);
-	g_assert (!itemlist_priv.filter);
-	itemlist_priv.hasEnclosures = FALSE;
-	itemlist_priv.isSearchResult = FALSE;
+	g_assert (!itemlist->priv->guids);
+	g_assert (!itemlist->priv->filter);
 
 	/* 1. Filter check. Don't continue if folder is selected and 
 	   no folder viewing is configured. If folder viewing is enabled
@@ -307,34 +303,40 @@ itemlist_load (nodePtr node)
 	/* for folders and other heirarchic nodes do filtering */
 	if (IS_FOLDER (node) || node->children) {
 		liferea_shell_update_allitems_actions (FALSE, 0 != node->unreadCount);
-		
-		if (0 == conf_get_int_value (FOLDER_DISPLAY_MODE))
+
+		conf_get_int_value (FOLDER_DISPLAY_MODE, &folder_display_mode);
+		if (!folder_display_mode)
 			return;
 	
-		if (conf_get_bool_value (FOLDER_DISPLAY_HIDE_READ))
-			itemlist_priv.filter = g_slist_append (NULL, rule_new (NULL, "unread", "", TRUE));
+		conf_get_bool_value (FOLDER_DISPLAY_HIDE_READ, &folder_display_hide_read);
+		if (folder_display_hide_read) {
+			itemlist->priv->filter = g_new0(struct itemSet, 1);
+			itemlist->priv->filter->anyMatch = TRUE;
+			itemset_add_rule (itemlist->priv->filter, "unread", "", TRUE);
+		}
 	} else {
 		liferea_shell_update_allitems_actions (0 != node->itemCount, 0 != node->unreadCount);
 	}
 
-	itemlist_priv.loading++;
-	itemlist_priv.viewMode = node_get_view_mode (node);
-	itemview_set_layout (itemlist_priv.viewMode);
+	itemlist->priv->loading++;
+	itemlist->priv->viewMode = node_get_view_mode (node);
+	itemview_set_layout (itemlist->priv->viewMode);
 
 	/* Set the new displayed node... */
-	itemlist_priv.currentNode = node;
-	itemview_set_displayed_node (itemlist_priv.currentNode);
+	itemlist->priv->currentNode = node;
+	itemview_set_displayed_node (itemlist->priv->currentNode);
 
 	if (NODE_VIEW_MODE_COMBINED != node_get_view_mode (node))
 		itemview_set_mode (ITEMVIEW_NODE_INFO);
 	else
 		itemview_set_mode (ITEMVIEW_ALL_ITEMS);
 	
-	itemSet = node_get_itemset (itemlist_priv.currentNode);
+	itemSet = node_get_itemset (itemlist->priv->currentNode);
 	itemlist_merge_itemset (itemSet);
-	itemset_free (itemSet);
+	if (!IS_VFOLDER (node))			/* FIXME: this is ugly! */
+		itemset_free (itemSet);
 
-	itemlist_priv.loading--;
+	itemlist->priv->loading--;
 
 	debug_exit("itemlist_load");
 }
@@ -342,51 +344,27 @@ itemlist_load (nodePtr node)
 void
 itemlist_unload (gboolean markRead) 
 {
+	/* Always clear to ensure clearing on search */
 	itemview_clear ();
 
-	if (itemlist_priv.currentNode) {
+	if (itemlist->priv->currentNode) {
 		itemview_set_displayed_node (NULL);
 		
 		/* 1. Postprocessing for previously selected node, this is necessary
 		   to realize reliable read marking when using condensed mode. It's
 		   important to do this only when the selection really changed. */
-		if (markRead && (2 == node_get_view_mode (itemlist_priv.currentNode)))
-			feedlist_mark_all_read (itemlist_priv.currentNode);
+		if (markRead && (2 == node_get_view_mode (itemlist->priv->currentNode)))
+			feedlist_mark_all_read (itemlist->priv->currentNode);
 
 		itemlist_check_for_deferred_action ();
 	}
 
 	itemlist_set_selected (NULL);
-	itemlist_filter_free ();
 	itemlist_duplicate_list_free ();
-	itemlist_priv.currentNode = NULL;
-}
-
-/* next unread selection logic */
-
-static itemPtr
-itemlist_find_unread_item (void) 
-{
-	itemPtr	result = NULL;
+	itemlist->priv->currentNode = NULL;
 	
-	if (!itemlist_priv.currentNode)
-		return NULL;
-
-	if (itemlist_priv.currentNode->children) {
-		feedlist_find_unread_feed (itemlist_priv.currentNode);
-		return NULL;
-	}
-
-	/* Note: to select in sorting order we need to do it in the GUI code
-	   otherwise we would have to sort the item list here... */
-	
-	if (itemlist_priv.selectedId)
-		result = ui_itemlist_find_unread_item (itemlist_priv.selectedId);
-	
-	if (!result)
-		result = ui_itemlist_find_unread_item (0);
-
-	return result;
+	itemset_free (itemlist->priv->filter);
+	itemlist->priv->filter = NULL;
 }
 
 void
@@ -397,16 +375,16 @@ itemlist_select_next_unread (void)
 	/* If we are in combined mode we have to mark everything
 	   read or else we would never jump to the next feed,
 	   because no item will be selected and marked read... */
-	if (itemlist_priv.currentNode) {
-		if (NODE_VIEW_MODE_COMBINED == node_get_view_mode (itemlist_priv.currentNode))
-			node_mark_all_read (itemlist_priv.currentNode);
+	if (itemlist->priv->currentNode) {
+		if (NODE_VIEW_MODE_COMBINED == node_get_view_mode (itemlist->priv->currentNode))
+			node_mark_all_read (itemlist->priv->currentNode);
 	}
 
-	itemlist_priv.loading++;	/* prevent unwanted selections */
+	itemlist->priv->loading++;	/* prevent unwanted selections */
 
 	/* before scanning the feed list, we test if there is a unread 
 	   item in the currently selected feed! */
-	result = itemlist_find_unread_item ();
+	result = itemview_find_unread_item (itemlist->priv->selectedId);
 	
 	/* If none is found we continue searching in the feed list */
 	if (!result) {
@@ -416,17 +394,17 @@ itemlist_select_next_unread (void)
 		node = feedlist_find_unread_feed (feedlist_get_root ());
 		if (node) {
 			/* load found feed */
-			ui_feedlist_select (node);
+			feed_list_view_select (node);
 
 			if (NODE_VIEW_MODE_COMBINED != node_get_view_mode (node))
-				result = itemlist_find_unread_item ();	/* find first unread item */
+				result = itemview_find_unread_item (0);	/* find first unread item */
 		} else {
 			/* if we don't find a feed with unread items do nothing */
-			liferea_shell_set_status_bar (_("There are no unread items "));
+			liferea_shell_set_status_bar (_("There are no unread items"));
 		}
 	}
 
-	itemlist_priv.loading--;
+	itemlist->priv->loading--;
 	
 	if (result)
 		itemview_select_item (result);
@@ -454,11 +432,11 @@ itemlist_hide_item (itemPtr item)
 {
 	/* if the currently selected item should be removed we
 	   don't do it and set a flag to do it when unselecting */
-	if (itemlist_priv.selectedId != item->id) {
+	if (itemlist->priv->selectedId != item->id) {
 		itemview_remove_item (item);
 		ui_node_update (item->nodeId);
 	} else {
-		itemlist_priv.deferredFilter = TRUE;
+		itemlist->priv->deferredFilter = TRUE;
 		/* update the item to show new state that forces
 		   later removal */
 		itemview_update_item (item);
@@ -469,21 +447,22 @@ itemlist_hide_item (itemPtr item)
 static void
 itemlist_unhide_item (itemPtr item)
 {
-	itemlist_priv.deferredFilter = FALSE;
+	itemlist->priv->deferredFilter = FALSE;
 }
 
 /* functions to remove items on remove requests */
+
+/* hard unconditional item remove */
 void
 itemlist_remove_item (itemPtr item) 
 {
 	/* update search folder counters */
-	vfolder_foreach (vfolder_update_counters);
-
-	if (itemlist_priv.selectedId == item->id) {
+	vfolder_foreach_data (vfolder_remove_item, item);
+	
+	if (itemlist->priv->selectedId == item->id) {
 		itemlist_set_selected (NULL);
-		itemlist_priv.deferredFilter = FALSE;
-		itemlist_priv.deferredRemove = FALSE;
-		itemview_select_item (NULL);
+		itemlist->priv->deferredFilter = FALSE;
+		itemlist->priv->deferredRemove = FALSE;
 	}
 
 	itemlist_duplicate_list_remove_item (item);
@@ -499,15 +478,16 @@ itemlist_remove_item (itemPtr item)
 	item_unload (item);
 }
 
+/* soft possibly delayed item remove */
 void
 itemlist_request_remove_item (itemPtr item) 
 {	
 	/* if the currently selected item should be removed we
 	   don't do it and set a flag to do it when unselecting */
-	if (itemlist_priv.selectedId != item->id) {
+	if (itemlist->priv->selectedId != item->id) {
 		itemlist_remove_item (item);
 	} else {
-		itemlist_priv.deferredRemove = TRUE;
+		itemlist->priv->deferredRemove = TRUE;
 		/* update the item to show new state that forces
 		   later removal */
 		itemview_update_item (item);
@@ -522,7 +502,9 @@ itemlist_remove_items (itemSetPtr itemSet, GList *items)
 	while (iter) {
 		itemPtr item = (itemPtr) iter->data;
 
-		if (itemlist_priv.selectedId != item->id) {
+		vfolder_foreach_data (vfolder_remove_item, item);
+
+		if (itemlist->priv->selectedId != item->id) {
 			/* don't call itemlist_remove_item() here, because it's to slow */
 			itemview_remove_item (item);
 			db_item_remove (item->id);
@@ -536,24 +518,34 @@ itemlist_remove_items (itemSetPtr itemSet, GList *items)
 
 	itemview_update ();
 	node_update_counters (node_from_id (itemSet->nodeId));
-	vfolder_foreach (vfolder_update_counters);
 }
 
 void
 itemlist_remove_all_items (nodePtr node)
 {	
-	if (node == itemlist_priv.currentNode)
+	GList		*iter;
+	itemSetPtr	itemset;
+	
+	if (node == itemlist->priv->currentNode)
 		itemview_clear ();
+
+	itemset = db_itemset_load (node->id);
+	iter = itemset->ids;
+	while (iter) {
+		itemPtr item = item_load ((gulong)iter->data);
+		vfolder_foreach_data (vfolder_remove_item, item);
+		item_unload (item);
+		iter = g_list_next (iter);
+	}
+	itemset_free (itemset);
 		
 	db_itemset_remove_all (node->id);
 	
-	if (node == itemlist_priv.currentNode) {
+	if (node == itemlist->priv->currentNode) {
 		itemview_update ();
 		itemlist_duplicate_list_free ();
 	}
-	
-	vfolder_foreach (vfolder_update_counters);
-	
+
 	node_update_counters (node);
 }
 
@@ -566,7 +558,15 @@ itemlist_update_item (itemPtr item)
 	} else {
 		itemlist_unhide_item (item);
 	}
-	
+
+	/* FIXME: this is tricky. It's possible that the item
+	 * selected, but the itemview contains a webpage. In
+	 * that case, we don't want to reload the item, but if
+	 * the itemview contains the item, we want, as the
+	 * background will turn yellow and the the title will
+	 * turn red. So how to know whether the itemview contains
+	 * the item?
+	 */
 	itemview_update_item (item);
 }
 
@@ -577,7 +577,7 @@ itemlist_selection_changed (itemPtr item)
 	debug_enter ("itemlist_selection_changed");
 	debug_start_measurement (DEBUG_GUI);
 
-	if (0 == itemlist_priv.loading)	{
+	if (0 == itemlist->priv->loading)	{
 		/* folder&vfolder postprocessing to remove/filter unselected items no
 		   more matching the display rules because they have changed state */
 		itemlist_check_for_deferred_action ();
@@ -595,15 +595,17 @@ itemlist_selection_changed (itemPtr item)
 
 			item_set_read_state (item, TRUE);
 
-			if (node_load_link_preferred (node)) {
-				itemview_launch_URL (item_get_source (item), TRUE /* force internal */);
+			if (node->loadItemLink) {
+				gchar* link = item_make_link (item);
+
+				itemview_launch_URL (link, TRUE /* force internal */);
+				g_free (link);
 			} else {
 				itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
 				itemview_select_item (item);
 				itemview_update ();
 			}
 			ui_node_update (item->nodeId);
-			
 		}
 
 		feedlist_reset_new_item_count ();
@@ -621,24 +623,36 @@ itemlist_selection_changed (itemPtr item)
 guint
 itemlist_get_view_mode (void)
 {
-	return itemlist_priv.viewMode; 
+	return itemlist->priv->viewMode; 
 }
 
 void
-itemlist_set_view_mode (guint newMode) 
-{ 
+itemlist_set_view_mode (guint newMode)
+{
 	nodePtr		node;
+	itemPtr		item;
+
+	itemlist->priv->viewMode = newMode;
 
-	itemlist_priv.viewMode = newMode;
-	
 	node = itemlist_get_displayed_node ();
+	item = itemlist_get_selected ();
+
 	if (node) {
 		itemlist_unload (FALSE);
-		
-		node_set_view_mode (node, itemlist_priv.viewMode);
-		itemview_set_layout (itemlist_priv.viewMode);
+
+		node_set_view_mode (node, itemlist->priv->viewMode);
+		itemview_set_layout (itemlist->priv->viewMode);
 		itemlist_load (node);
+
+		/* If there was an item selected, select it again since
+		 * itemlist_unload() unselects it.
+		 */
+		if (item && itemlist->priv->viewMode != NODE_VIEW_MODE_COMBINED)
+			itemview_select_item (item);
 	}
+
+	if (item)
+		item_unload (item);
 }
 
 void
@@ -648,3 +662,66 @@ on_view_activate (GtkRadioAction *action, GtkRadioAction *current, gpointer user
 	itemlist_set_view_mode (val);
 }
 
+/* item loader methods */
+
+static void
+itemlist_item_batch_fetched_cb (ItemLoader *il, GSList *items, gpointer user_data)
+{
+	GSList		*iter;
+
+	if (item_loader_get_node (il) != itemlist->priv->currentNode)
+		return;	/* Bail on loader not matching selection */
+
+	debug0 (DEBUG_CACHE, "itemlist_item_batch_fetched_cb()");
+
+	iter = items;
+	while (iter) {
+		itemPtr item = (itemPtr)iter->data;
+
+		itemlist_merge_item (item);
+		item_unload (item);
+
+		iter= g_slist_next (iter);
+	}
+
+	itemview_update();
+	g_slist_free (items);
+}
+
+void
+itemlist_add_loader (ItemLoader *loader)
+{
+	g_signal_connect (G_OBJECT (loader), "item-batch-fetched", G_CALLBACK (itemlist_item_batch_fetched_cb), NULL);
+
+	item_loader_start (loader);
+}
+
+void
+itemlist_add_search_result (ItemLoader *loader)
+{
+	nodeViewType viewMode;
+
+	/* Ensure that we are in a useful viewing mode (3 paned) */
+	itemlist_unload (FALSE);
+
+	viewMode = itemlist_get_view_mode ();
+	if ((NODE_VIEW_MODE_NORMAL != viewMode) &&
+	    (NODE_VIEW_MODE_WIDE != viewMode))
+		itemview_set_layout (NODE_VIEW_MODE_NORMAL);
+
+	itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
+
+	/* Set current node to search result dummy node so that
+	   we except only items from the respective loader for
+	   the item view. */
+	itemlist->priv->currentNode = item_loader_get_node (loader);
+
+	itemlist_add_loader (loader);
+}
+
+ItemList *
+itemlist_create (void)
+{
+	return ITEMLIST (g_object_new (ITEMLIST_TYPE, NULL));
+}
+
