@@ -1,7 +1,7 @@
 /**
  * @file main.c Liferea main program
  *
- * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2012 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *  
  * Some code like the command line handling was inspired by 
@@ -29,7 +29,7 @@
 #endif
 
 #include <gtk/gtk.h>
-#include <locale.h> /* For setlocale */
+#include <unique/unique.h>
 
 #include <string.h>
 #include <stdlib.h>
@@ -41,20 +41,13 @@
 #include "db.h"
 #include "dbus.h"
 #include "debug.h"
-#include "plugin.h"
-#include "script.h"
+#include "feedlist.h"
 #include "social.h"
 #include "update.h"
-#include "vfolder.h"		// FIXME: should not be necessary
 #include "xml.h"
-#include "ui/ui_feedlist.h"	// FIXME: should not be necessary
 #include "ui/ui_session.h"
 #include "ui/liferea_shell.h"
-#include "sync/avahi_publisher.h"
-
-#include "bacon-message-connection.h"
-
-static BaconMessageConnection *bacon_connection = NULL;
+#include "notification/notification.h"
 
 static enum {
 	STATE_STARTING,
@@ -62,42 +55,61 @@ static enum {
 	STATE_SHUTDOWN
 } runState = STATE_STARTING;
 
-gboolean on_quit(GtkWidget *widget, GdkEvent *event, gpointer user_data);
+enum {
+	COMMAND_0 = 0, /* 0 is not a valid command */
+	COMMAND_ADD_FEED
+};
 
-
-/** bacon message callback */
-static void
-on_bacon_message_received (const char *message, gpointer data)
+static UniqueResponse
+message_received_cb (UniqueApp         *app,
+		     UniqueCommand      command,
+		     UniqueMessageData *message,
+		     guint              time_,
+		     gpointer           user_data)
 {
-	debug1(DEBUG_GUI, "bacon message received >>>%s<<<", message);
-	
-	/* Currently we only know a single simple command "raise"
-	   which tells the program to raise the window because
-	   another instance was requested which is not supported */
-	   
-	if (g_str_equal (message, "raise")) {
-		debug0 (DEBUG_GUI, "-> raise window requested");
-		liferea_shell_present ();
-	} else {
-		g_warning ("Received unknown bacon command: >>>%s<<<", message);
+	UniqueResponse res;
+	gchar *feed;
+
+	debug1(DEBUG_GUI, "libunique command received >>>%d<<<", command);
+
+	switch (command)
+	{
+		case UNIQUE_ACTIVATE:
+			/* Raise the liferea window */
+			debug0 (DEBUG_GUI, "-> raise window requested");
+			liferea_shell_present ();
+			res = UNIQUE_RESPONSE_OK;
+			break;
+		case COMMAND_ADD_FEED:
+			feed = unique_message_data_get_text (message);
+			feedlist_add_subscription (feed, NULL, NULL, 0);
+
+			res = UNIQUE_RESPONSE_OK;
+			break;
+		default:
+			g_warning ("Received unknown libunique command: >>>%d<<<", command);
+			res = UNIQUE_RESPONSE_OK;
+			break;
 	}
+
+	return res;
 }
 
-static void fatal_signal_handler(int sig) {
+static void G_GNUC_NORETURN
+fatal_signal_handler (int sig)
+{
 	sigset_t sigset;
 
-	sigemptyset(&sigset);
-	sigprocmask(SIG_SETMASK, &sigset, NULL);
+	sigemptyset (&sigset);
+	sigprocmask (SIG_SETMASK, &sigset, NULL);
 
-	g_print("\nLiferea did receive signal %d (%s).\n", sig, g_strsignal(sig));
+	g_print ("\nLiferea did receive signal %d (%s).\n", sig, g_strsignal (sig));
 
-	if(debug_level) {
-		g_print("You have probably triggered a program bug. I will now try to \n");
-		g_print("create a backtrace which you can attach to any support requests.\n\n");
-		g_on_error_stack_trace(PACKAGE);
-	}
+	g_print ("You have probably triggered a program bug. I will now try to \n");
+	g_print ("create a backtrace which you can attach to any support requests.\n\n");
+	g_on_error_stack_trace (PACKAGE);
 
-	_exit(1);
+	_exit (1);
 }
 
 static void
@@ -132,12 +144,12 @@ debug_entries_parse_callback (const gchar *option_name,
 		*debug_flags |= DEBUG_PARSING;
 	} else if (g_str_equal (option_name, "--debug-performance")) {
 		*debug_flags |= DEBUG_PERF;
-	} else if (g_str_equal (option_name, "--debug-plugins")) {
-		*debug_flags |= DEBUG_PLUGINS;
 	} else if (g_str_equal (option_name, "--debug-trace")) {
 		*debug_flags |= DEBUG_TRACE;
 	} else if (g_str_equal (option_name, "--debug-update")) {
 		*debug_flags |= DEBUG_UPDATE;
+	} else if (g_str_equal (option_name, "--debug-vfolder")) {
+		*debug_flags |= DEBUG_VFOLDER;
 	} else if (g_str_equal (option_name, "--debug-verbose")) {
 		*debug_flags |= DEBUG_VERBOSE;
 	} else {
@@ -147,7 +159,7 @@ debug_entries_parse_callback (const gchar *option_name,
 	return TRUE;
 }
 
-static gboolean
+static gboolean G_GNUC_NORETURN
 show_version (const gchar *option_name,
 	      const gchar *value,
 	      gpointer data,
@@ -160,13 +172,17 @@ show_version (const gchar *option_name,
 int
 main (int argc, char *argv[])
 {
+	UniqueApp	*app;
+	UniqueMessageData	*msg;
 	GError		*error = NULL;
 	GOptionContext	*context;
 	GOptionGroup	*debug;
 	gulong		debug_flags = 0;
 	LifereaDBus	*dbus = NULL;
-	gchar		*initial_state = "shown";
+	const gchar	*initial_state = "shown";
+	gchar		*feed = NULL;
 	int		initialState;
+	gboolean	show_tray_icon, start_in_tray;
 
 #ifdef USE_SM
 	gchar *opt_session_arg = NULL;
@@ -178,22 +194,23 @@ main (int argc, char *argv[])
 		{ "session", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &opt_session_arg, NULL, NULL },
 #endif
 		{ "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version, N_("Show version information and exit"), NULL },
+		{ "add-feed", 'a', 0, G_OPTION_ARG_STRING, &feed, N_("Add a new subscription"), N_("uri") },
 		{ NULL }
 	};
 
 	GOptionEntry debug_entries[] = {
 		{ "debug-all", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of all types"), NULL },
 		{ "debug-cache", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages for the cache handling"), NULL },
-		{ "debug-conf", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of the configuration handling"), NULL },
+		{ "debug-conf", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages for the configuration handling"), NULL },
 		{ "debug-db", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of the database handling"), NULL },
 		{ "debug-gui", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of all GUI functions"), NULL },
-		{ "debug-html", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Enables HTML rendering debugging. Each time Liferea renders HTML output it will also dump the generated HTML into ~/.liferea_1.6/output.xhtml"), NULL },
+		{ "debug-html", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Enables HTML rendering debugging. Each time Liferea renders HTML output it will also dump the generated HTML into ~/.liferea_1.8/output.xhtml"), NULL },
 		{ "debug-net", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of all network activity"), NULL },
 		{ "debug-parsing", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of all parsing functions"), NULL },
 		{ "debug-performance", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages when a function takes too long to process"), NULL },
-		{ "debug-plugins", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages for the plugin loading"), NULL },
 		{ "debug-trace", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages when entering/leaving functions"), NULL },
 		{ "debug-update", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of the feed update processing"), NULL },
+		{ "debug-vfolder", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of the search folder matching"), NULL },
 		{ "debug-verbose", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print verbose debugging messages"), NULL },
 		{ NULL }
 	};
@@ -207,11 +224,11 @@ main (int argc, char *argv[])
 #endif
 
 	debug = g_option_group_new ("debug",
-				    N_("Print debugging messages for the given topic"),
-				    N_("Print debugging messages for the given topic"),
+				    _("Print debugging messages for the given topic"),
+				    _("Print debugging messages for the given topic"),
 				    &debug_flags,
 				    NULL);
-	g_option_group_set_translation_domain (debug, GETTEXT_PACKAGE);
+	g_option_group_set_translation_domain(debug, GETTEXT_PACKAGE);
 	g_option_group_add_entries (debug, debug_entries);
 
 	context = g_option_context_new (NULL);
@@ -234,79 +251,52 @@ main (int argc, char *argv[])
 	   has to be initialized before update_init() */
 	conf_init ();
 
-#ifdef USE_DBUS
-	dbus_g_thread_init ();
-#endif
-
 	/* We need to do the network initialization here to allow
 	   network-manager to be setup before gtk_init() */
 	update_init ();
 
 	gtk_init (&argc, &argv);
 
+	/* Single instance checks */
+	app = unique_app_new_with_commands ("net.sourceforge.liferea", NULL,
+					    "add_feed", COMMAND_ADD_FEED,
+					    NULL);
+	if (unique_app_is_running (app)) {
+		g_print ("Liferea is already running\n");
+		unique_app_send_message (app, UNIQUE_ACTIVATE, NULL);
+		if (feed) {
+			msg = unique_message_data_new ();
+			unique_message_data_set_text (msg, feed, -1);
+			unique_app_send_message (app, COMMAND_ADD_FEED, msg);
+		}
+		return 1;
+	} else {
+		g_signal_connect (app, "message-received", G_CALLBACK (message_received_cb), NULL);
+	}
+
 	/* GTK theme support */
 	g_set_application_name (_("Liferea"));
 	gtk_window_set_default_icon_name ("liferea");
 
-	/* Note: bacon connection check needs to be done after the
-	   command line parameter checking to allow help and version
-	   switches to be used when we are already running */
-	bacon_connection = bacon_message_connection_new ("liferea");
-	if (bacon_connection)	{
-		if (!bacon_message_connection_get_is_server (bacon_connection)) {
-			g_warning(_("Liferea seems to be running already!"));
-			
-		  	debug0(DEBUG_VERBOSE, "Startup as bacon client...");
-			bacon_message_connection_send (bacon_connection, "raise");
-			bacon_message_connection_free (bacon_connection);
-			
-			gdk_notify_startup_complete ();
-			exit (0);
-		} else {
-		  	debug0 (DEBUG_VERBOSE, "Startup as bacon server...");
-			bacon_message_connection_set_callback (bacon_connection,
-							       on_bacon_message_received,
-							       NULL);
-		}
-	} else {
-		g_warning ("Cannot create IPC connection for Liferea!");
-	}
-	
 	debug_start_measurement (DEBUG_DB);
 
 	/* order is important! */
 	db_init ();			/* initialize sqlite */
 	xml_init ();			/* initialize libxml2 */
-	plugin_mgmt_init ();		/* get list of plugins and initialize them */
-	conf_load ();			/* load global feed settings */
-	script_init ();			/* setup scripting if supported */
-	social_init ();			/* initialized social bookmarking */
-#ifdef USE_DBUS	
-	dbus = liferea_dbus_new ();	
-#else
-	debug0 (DEBUG_GUI, "Compiled without DBUS support.");
+#ifdef HAVE_LIBNOTIFY
+	notification_plugin_register (&libnotify_plugin);
 #endif
+	social_init ();			/* initialize social bookmarking */
 
-#ifdef USE_AVAHI
-	if (conf_get_bool_value (SYNC_AVAHI_ENABLED)) {
-		LifereaAvahiPublisher	*avahiPublisher = NULL;
-
-		debug0 (DEBUG_CACHE, "Registering with AVAHI");
-		avahiPublisher = liferea_avahi_publisher_new ();
-		liferea_avahi_publisher_publish (avahiPublisher, conf_get_str_value (SYNC_AVAHI_SERVICE_NAME), 23632);
-	} else {
-		debug0 (DEBUG_CACHE, "Avahi support available, but disabled by preferences.");
-	}
-#else
-	debug0 (DEBUG_CACHE, "Compiled without AVAHI support");
-#endif
+	dbus = liferea_dbus_new ();
 
 	/* how to start liferea, command line takes precedence over preferences */
+	conf_get_bool_value (SHOW_TRAY_ICON, &show_tray_icon);
+	conf_get_bool_value (START_IN_TRAY, &start_in_tray);
 	if (g_str_equal(initial_state, "iconified")) {
 		initialState = MAINWINDOW_ICONIFIED;
 	} else if (g_str_equal(initial_state, "hidden") ||
-	    (conf_get_bool_value (SHOW_TRAY_ICON) &&
-	     conf_get_bool_value (START_IN_TRAY))) {
+	    (show_tray_icon && start_in_tray)) {
 		initialState = MAINWINDOW_HIDDEN;
 	} else {
 		initialState = MAINWINDOW_SHOWN;
@@ -315,8 +305,6 @@ main (int argc, char *argv[])
 	liferea_shell_create (initialState);
 	g_set_prgname ("liferea");
 	
-	script_run_for_hook (SCRIPT_HOOK_STARTUP);
-	
 #ifdef USE_SM
 	/* This must be after feedlist reading because some session
 	   managers will tell Liferea to exit if Liferea does not
@@ -341,11 +329,13 @@ main (int argc, char *argv[])
 	runState = STATE_STARTING;
 	
 	debug_end_measurement (DEBUG_DB, "startup");
-	
+
+	if (feed)
+		feedlist_add_subscription (feed, NULL, NULL, 0);
+
 	gtk_main ();
 	
 	g_object_unref (G_OBJECT (dbus));
-	bacon_message_connection_free (bacon_connection);
 	return 0;
 }
 
@@ -361,13 +351,8 @@ on_shutdown (gpointer user_data)
 	runState = STATE_SHUTDOWN;
 
 	/* order is important ! */
-		
-	script_run_for_hook (SCRIPT_HOOK_SHUTDOWN);
-	
-	itemlist_free ();
 	update_deinit ();
 	db_deinit ();
-	script_deinit ();
 	social_free ();
 
 	liferea_shell_destroy ();
