@@ -1,7 +1,7 @@
 /**
  * @file migrate.c migration between different cache versions
  * 
- * Copyright (C) 2007-2008  Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2011  Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 #include "xml.h"
 #include "ui/ui_common.h"
 
-#define LIFEREA_CURRENT_DIR ".liferea_1.6"
+#define LIFEREA_CURRENT_DIR ".liferea_1.8"
 
 static void 
 migrate_copy_dir (const gchar *from,
@@ -51,15 +51,11 @@ migrate_copy_dir (const gchar *from,
 	
 	dir = g_dir_open (fromDirname, 0, NULL);
 	while (NULL != (srcfile = (gchar *)g_dir_read_name (dir))) {
-		gchar	*content;
-		gsize	length;
 		destfile = g_build_filename (toDirname, srcfile, NULL);
 		srcfile = g_build_filename (fromDirname, srcfile, NULL);
 		if (g_file_test (srcfile, G_FILE_TEST_IS_REGULAR)) {
 			g_print ("copying %s\n     to %s\n", srcfile, destfile);
-			if (g_file_get_contents (srcfile, &content, &length, NULL))
-				g_file_set_contents (destfile, content, length, NULL);
-			g_free (content);
+			common_copy_file (srcfile, destfile);
 		} else {
 			g_print("skipping %s\n", srcfile);
 		}
@@ -87,7 +83,7 @@ migrate_item_parse_cache (xmlNodePtr cur,
 	cur = cur->xmlChildrenNode;
 	while(cur) {
 		if(cur->type != XML_ELEMENT_NODE ||
-		   !(tmp = common_utf8_fix(xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1)))) {
+		   !(tmp = xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1))) {
 			cur = cur->next;
 			continue;
 		}
@@ -191,7 +187,7 @@ migrate_load_from_cache (const gchar *sourceDir, const gchar *id)
 		while (cur && xmlIsBlankNode (cur))
 			cur = cur->next;
 
-		if (!xmlStrcmp (cur->name, BAD_CAST"feed")) {
+		if (cur && !xmlStrcmp (cur->name, BAD_CAST"feed")) {
 			xmlChar *version;			
 			if ((version = xmlGetProp (cur, BAD_CAST"version"))) {
 				migrateFrom10 = xmlStrcmp (BAD_CAST FEED_CACHE_VERSION, version);
@@ -291,7 +287,7 @@ migrate_from_12 (void)
 }
 
 static void
-migrate_from_13_14 (const gchar *olddir)
+migrate_from_14plus (const gchar *olddir)
 {
 	g_print("Performing %s -> %s cache migration...\n", olddir, LIFEREA_CURRENT_DIR);	
 	
@@ -322,14 +318,19 @@ migration_execute (migrationMode mode)
 			olddir = ".liferea_1.2";
 			migrate_from_12 ();
 			break;
-		case MIGRATION_FROM_13:
-			olddir = ".liferea_1.3";
-			migrate_from_13_14 (olddir);
-			break;
 		case MIGRATION_FROM_14:
 			olddir = ".liferea_1.4";
-			migrate_from_13_14 (olddir);
+			migrate_from_14plus (olddir);
 			break;
+		case MIGRATION_FROM_16:
+			olddir = ".liferea_1.6";
+			migrate_from_14plus (olddir);
+			break;
+		case MIGRATION_FROM_17:
+			olddir = ".liferea_1.7";
+			migrate_from_14plus (olddir);
+			break;
+		case MIGRATION_MODE_INVALID:
 		default:
 			g_error ("Invalid migration mode!");
 			return;
