@@ -1,7 +1,7 @@
 /**
- * @file node.h common feed list node handling interface
+ * @file node.h  hierarchic feed list node interface
  * 
- * Copyright (C) 2003-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2010 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,22 +23,20 @@
 #define _NODE_H
 
 #include "itemset.h"
-#include "update.h"
+#include "node_view.h"
 
 /* Liferea's GUI consists of three parts. Feed list, item list
-   and HTML view. The feed list is a view of all available
-   nodes. The feed list allows nodes of different types.
+   and HTML view. The feed list is a view of some or all available
+   nodes. The feed list allows nodes of different types defining
+   different UI behaviours.
 
-   According to the node's type the interface propagates
+   According to the node's type this interface propagates
    user interaction to the feed list node type implementation
-   and allows the implementation to change the nodes state. */
-
-/** node view mode types */
-enum {
-	NODE_VIEW_MODE_NORMAL	= 0,
-	NODE_VIEW_MODE_WIDE	= 1,
-	NODE_VIEW_MODE_COMBINED	= 2
-};
+   and allows the implementation to change the nodes state. 
+ 
+   This interface is to hide the node type and node source type
+   complexity for the GUI, scripting and updating functionality.
+ */
 
 /** generic feed list node structure */
 typedef struct node {
@@ -64,71 +62,73 @@ typedef struct node {
 	gboolean		expanded;	/**< expansion state (for nodes with childs) */
 
 	/* item list state properties of this node */
-	guint		viewMode;	/**< Viewing mode for this node (one of NODE_VIEW_MODE_*) */
-	gint		sortColumn;	/**< Sorting column. Set to either IS_TITLE, IS_FAVICON, IS_ENCICON or IS_TIME */
-	gboolean	sortReversed;	/**< Sort in the reverse order? */
+	nodeViewType		viewMode;	/**< Viewing mode for this node (one of NODE_VIEW_MODE_*) */
+	nodeViewSortType	sortColumn;	/**< Node specific item view sort attribute. */
+	gboolean		sortReversed;	/**< Sort in the reverse order? */
 	
+	/* rendering behaviour of this node */
+	gboolean	loadItemLink;	/**< if TRUE do automatically load the item link into the HTML pane */
+	
+	/* current state of this node */	
 	gboolean	needsUpdate;	/**< if TRUE: the item list has changed and the nodes feed list representation needs to be updated */
 	gboolean	needsRecount;	/**< if TRUE: the number of unread/total items is currently unknown and needs recounting */
 
 } *nodePtr;
 
-enum {
-	NODE_CAPABILITY_ADD_CHILDS		= (1<<0),	/**< allows adding new childs */
-	NODE_CAPABILITY_REMOVE_CHILDS		= (1<<1),	/**< allows removing it's childs */
-	NODE_CAPABILITY_SUBFOLDERS		= (1<<2),	/**< allows creating/removing sub folders */
-	NODE_CAPABILITY_REMOVE_ITEMS		= (1<<3),	/**< allows removing of single items */
-	NODE_CAPABILITY_RECEIVE_ITEMS		= (1<<4),	/**< is a DnD target for item copies */
-	NODE_CAPABILITY_REORDER			= (1<<5),	/**< allows DnD to reorder childs */
-	NODE_CAPABILITY_SHOW_UNREAD_COUNT	= (1<<6),	/**< display the unread item count in the feed list */
-	NODE_CAPABILITY_SHOW_ITEM_COUNT		= (1<<7),	/**< display the absolute item count in the feed list */
-	NODE_CAPABILITY_UPDATE			= (1<<8),	/**< node type always has a subscription and can be updated */
-	NODE_CAPABILITY_UPDATE_CHILDS		= (1<<9)	/**< childs of this node type can be updated */
-};
-
 /**
  * Creates a new node structure.
  *
  * @returns the new node
  */
-nodePtr node_new(void);
+nodePtr node_new (struct nodeType *type);
 
 /**
- * Node lookup by node id.
+ * Can be used to check wether an id is used or not.
+ *
+ * @param id	the node id to check
  *
  * @returns the node with the given id (or NULL)
  */
-nodePtr node_from_id(const gchar *id);
+nodePtr node_is_used_id (const gchar *id);
 
 /**
- * Sets a nodes parent and updates the feed list. If no
- * parent node is given the parent node of the currently
- * selected feed or the selected folder will be used.
+ * Node lookup by node id. Will report an error if the queried
+ * id does not exist.
+ *
+ * @param id	the node id to look up
+ *
+ * @returns the node with the given id (or NULL)
+ */
+nodePtr node_from_id (const gchar *id);
+
+/**
+ * Sets a nodes parent. If no parent node is given the 
+ * parent node of the currently selected feed or the 
+ * selected folder will be used.
+ *
+ * To be used before calling feedlist_node_added()
  *
- * @param parent	the parent node (optional can be NULL)
  * @param node		the node
+ * @param parent	the parent node (optional can be NULL)
  * @param position	insert position (optional can be 0)
  */
-void node_add_child(nodePtr parent, nodePtr node, gint position);
+void node_set_parent (nodePtr node, nodePtr parent, gint position);
 
 /**
- * Automatic subscription adding (e.g. URL DnD), creates a new feed
- * node and creates a new feed without any user interaction.
- *
- * @param source	the subscriptions source URL
- * @param title		NULL or the node title
- * @param filter	NULL or the filter for the subscription
- * @param options	NULL or the update options
- * @param flags		download request flags
- */
-void node_request_automatic_add(const gchar *source, const gchar *title, const gchar *filter, updateOptionsPtr options, gint flags);
-	
+ * Set a node's new parent and update UI. If a node already has a parent, 
+ * it will be removed from its parent children list. 
+ * 
+ * @param node			the node
+ * @param new_parent	nodes new parent
+ */ 
+void node_reparent (nodePtr node, nodePtr new_parent);
+
 /**
- * Removes the given node from the feed list.
+ * Removes all data associated with the given node.
  *
- * @param parent	the node
+ * @param node		the node
  */
-void node_request_remove(nodePtr node);
+void node_remove (nodePtr node);
 
 /**
  * Attaches a data structure to the given node.
@@ -242,11 +242,11 @@ gpointer node_get_icon(nodePtr node);
 const gchar * node_get_favicon_file(nodePtr node);
 
 /**
- * Returns a new unique node id.
+ * Returns a new unused unique node id.
  *
- * @returns new id
+ * @returns new id (to be free'd using g_free)
  */
-gchar * node_new_id(void);
+gchar * node_new_id (void);
 
 /**
  * Query the unique id string of the node.
@@ -255,7 +255,7 @@ gchar * node_new_id(void);
  *
  * @returns id string
  */
-const gchar *node_get_id(nodePtr node);
+const gchar *node_get_id (nodePtr node);
 
 /** 
  * Set the unique id string of the node.
@@ -272,13 +272,6 @@ void node_set_id(nodePtr node, const gchar *id);
  */
 void node_free(nodePtr node);
 
-/**
- * Refreshes all counters of the given node.
- *
- * @param node	node to update
- */
-void node_update_counters(nodePtr node);
-
 /**
  * Helper function for generic node rendering. Performs
  * a generic node serialization to XML and passes the
@@ -298,24 +291,6 @@ gchar * node_default_render(nodePtr node);
  */
 void node_save(nodePtr node);
 
-/**
- * Passes the result of a completed update request to the node
- * type specific processing callback.
- *
- * @param node		the node
- * @param result	the update request result
- * @param flags		update result processing flags
- */
-void node_process_update_result (nodePtr node, const struct updateResult * const result, updateFlags flags);
-
-/**
- * Resets the update interval for a given node.
- *
- * @param node	the node
- * @param now	current time
- */
-void node_reset_update_counter(nodePtr node, GTimeVal *now);
-
 /**
  * Loads all items of the given node into memory.
  * The caller needs to free the item set using itemset_free()
@@ -339,9 +314,8 @@ gchar * node_render(nodePtr node);
  * Called when updating favicons is requested.
  *
  * @param node		the node
- * @param now		current time
  */
-void node_update_favicon (nodePtr node, GTimeVal *now);
+void node_update_favicon (nodePtr node);
 
 /**
  * Change/Set the sort column of a given node.
@@ -349,35 +323,27 @@ void node_update_favicon (nodePtr node, GTimeVal *now);
  * @param node		the node
  * @param sortColumn	sort column id
  * @param reversed	TRUE if order should be reversed
+ *
+ * @returns TRUE if the passed settings were different from the previous ones
  */
-void node_set_sort_column(nodePtr node, gint sortColumn, gboolean reversed);
+gboolean node_set_sort_column (nodePtr node, nodeViewSortType sortColumn, gboolean reversed);
 
 /**
  * Change/Set the viewing mode of a given node.
  *
  * @param node		the node
- * @param newMode	viewing mode (0 = normal, 1 = wide, 2 = combined)
+ * @param newMode	viewing mode (NODE_VIEW_MODE_*)
  */
-void node_set_view_mode(nodePtr node, guint newMode);
+void node_set_view_mode(nodePtr node, nodeViewType newMode);
 
 /**
  * Query the viewing mode setting of a given mode.
  *
  * @param node 	the node
  *
- * @returns viewing mode (0 = normal, 1 = wide, 2 = combined)
+ * @returns viewing mode (NODE_VIEW_MODE_*)
  */
-gboolean node_get_view_mode(nodePtr node);
-
-/**
- * Allows to check wether an node requires to load
- * the item link or the content after selecting an item.
- *
- * @param node	the node
- *
- * @returns TRUE if the item link is to be loaded
- */
-gboolean node_load_link_preferred(nodePtr node);
+nodeViewType node_get_view_mode(nodePtr node);
 
 /**
  * Returns the base URL for the given node.
