@@ -2,7 +2,7 @@
  * @file atom10.c  Atom 1.0 Parser
  * 
  * Copyright (C) 2005-2006 Nathan Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2003-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2010 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,10 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+ 
+#include "atom10.h"
 
 #include <string.h>
 
 #include "common.h"
+#include "date.h"
 #include "debug.h"
 #include "enclosure.h"
 #include "feed_parser.h"
@@ -43,7 +46,6 @@
 #include "metadata.h"
 #include "subscription.h"
 #include "xml.h"
-#include "atom10.h"
 
 #define ATOM10_NS BAD_CAST"http://www.w3.org/2005/Atom"
 
@@ -106,21 +108,16 @@ atom10_parse_content_construct (xmlNodePtr cur, feedParserCtxtPtr ctxt)
 	gchar *ret = NULL;
 	
 	if (xmlHasNsProp (cur, BAD_CAST"src", NULL )) {
-		gchar *src = xml_get_ns_attribute (cur, "src", NULL);
-		
-		if (!src) {
-			ret = g_strdup (_("Liferea is unable to display this item's content."));
-		} else {
-			gchar *baseURL = common_utf8_fix ((gchar *)xmlNodeGetBase (cur->doc, cur));
-			xmlChar *url;
-			
-			url = common_build_url (src, baseURL);
-			ret = g_markup_printf_escaped (_("<p><a href=\"%s\">View this item's content.</a></p>"), url);
-			
-			g_free (url);
-			xmlFree (baseURL);
-			xmlFree (src);
-		}
+		/*
+		   RFC 4287 says a feed must have a summary when there's
+		   a src attribute in the content (and the content therefore
+		   empty). We are already parsing the summary separately.
+
+		   RFC 4287 also says an entry must contain one link element
+		   with rel="alternate", so there's no point in parsing
+		   src and setting it as link.
+		*/
+		ret = NULL;
 	} else {
 		gchar *type;
 
@@ -129,11 +126,11 @@ atom10_parse_content_construct (xmlNodePtr cur, feedParserCtxtPtr ctxt)
 		
 		/* Contents need to be de-encoded and should not contain sub-tags.*/
 		if (type && (g_str_equal (type,"html") || !g_ascii_strcasecmp (type, "text/html"))) {
-			ret = common_utf8_fix (xhtml_extract (cur, 0, NULL));
+			ret = xhtml_extract (cur, 0, NULL);
 		} else if (!type || !strcmp (type, "text") || !strncasecmp (type, "text/",5)) {
 			gchar *tmp;
 			/* Assume that "text/ *" files can be directly displayed.. kinda stated in the RFC */
-			ret = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
+			ret = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
 			
 			g_strchug (g_strchomp (ret));
 			
@@ -145,7 +142,7 @@ atom10_parse_content_construct (xmlNodePtr cur, feedParserCtxtPtr ctxt)
 			ret = tmp;
 		} else if (!strcmp(type,"xhtml") || !g_ascii_strcasecmp (type, "application/xhtml+xml")) {
 			/* The spec says to only show the contents of the div tag that MUST be present */
-			ret = common_utf8_fix(xhtml_extract (cur, 2, NULL));
+			ret = xhtml_extract (cur, 2, NULL);
 		} else {
 			/* Do nothing on unsupported content types. This allows summaries to be used. */
 			ret = NULL;
@@ -177,7 +174,7 @@ atom10_parse_text_construct (xmlNodePtr cur, gboolean htmlified)
 	
 	/* This that need to be de-encoded and should not contain sub-tags.*/
 	if (!type || !strcmp(type, "text")) {
-		ret = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
+		ret = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
 		if (ret) {
 			g_strchug (g_strchomp (ret));
 
@@ -188,12 +185,12 @@ atom10_parse_text_construct (xmlNodePtr cur, gboolean htmlified)
 			}
 		}
 	} else if (!strcmp(type, "html")) {
-		ret = common_utf8_fix (xhtml_extract (cur, 0, NULL));
+		ret = xhtml_extract (cur, 0, NULL);
 		if (!htmlified)
 			ret = unhtmlize (unxmlize (ret));
 	} else if (!strcmp (type, "xhtml")) {
 		/* The spec says to show the contents of the div tag that MUST be present */
-		ret = common_utf8_fix (xhtml_extract (cur, 2, NULL));
+		ret = xhtml_extract (cur, 2, NULL);
 		
 		if (!htmlified)
 			ret = unhtmlize (ret);
@@ -224,14 +221,14 @@ atom10_parse_person_construct (xmlNodePtr cur)
 		if (xmlStrEqual (cur->ns->href, ATOM10_NS)) {
 			if (xmlStrEqual (cur->name, BAD_CAST"name")) {
 				g_free (name);
-				name = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
+				name = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
 			}
 			
 			if (xmlStrEqual (cur->name, BAD_CAST"email")) {
 				if (email)
 					invalid = TRUE;
 				g_free(email);
-				tmp = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+				tmp = (gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1);
 				email = g_markup_printf_escaped (" - <a href=\"mailto:%s\">%s</a>", tmp, tmp);
 				g_free(tmp);
 			}
@@ -240,8 +237,8 @@ atom10_parse_person_construct (xmlNodePtr cur)
 				if (!uri)
 					invalid = TRUE;
 				g_free (uri);
-				tmp = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
-				uri = g_markup_printf_escaped (" (<a href=\"%s\">Website</a>)", tmp);
+				tmp = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
+				uri = g_markup_printf_escaped (" (<a href=\"%s\">%s</a>)", tmp, _("Website"));
 				g_free (tmp);
 			}
 		} else {
@@ -253,7 +250,7 @@ atom10_parse_person_construct (xmlNodePtr cur)
 		invalid = TRUE;
 		name = g_strdup (_("Invalid Atom feed: unknown author"));
 	}
-	/* FIXME: so somthing with "invalid" flag */
+	/* FIXME: so something with "invalid" flag */
 	tmp = g_strdup_printf ("%s%s%s", name, uri?uri:"", email?email:"");
 	g_free (uri);
 	g_free (email);
@@ -271,11 +268,10 @@ atom10_parse_link (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserSt
 	if (href) {
 		xmlChar *baseURL = xmlNodeGetBase (cur->doc, cur);
 		gchar *url, *relation, *type, *escTitle = NULL, *title;
+		const gchar *feedURL = subscription_get_homepage (ctxt->subscription);
 		
-		if (!baseURL && ctxt->feed->htmlUrl && 
-		    ctxt->feed->htmlUrl[0] != '|' &&
-		    strstr (ctxt->feed->htmlUrl, "://"))
-			baseURL = xmlStrdup (BAD_CAST (ctxt->feed->htmlUrl));
+		if (!baseURL && feedURL && feedURL[0] != '|' && strstr (feedURL, "://"))
+			baseURL = xmlStrdup (BAD_CAST (feedURL));
 		url = (gchar *)common_build_url (href, (gchar *)baseURL);
 
 		type = xml_get_ns_attribute (cur, "type", NULL);
@@ -384,7 +380,7 @@ atom10_parse_entry_id (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10Pars
 {
 	gchar *id;
 	
-	id = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
+	id = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
 	if (id) {
 		if (strlen (id) > 0) {
 			item_set_id (ctxt->item, id);
@@ -411,9 +407,9 @@ atom10_parse_entry_published (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct ato
 {
 	gchar *datestr;
 	
-	datestr = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
+	datestr = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
 	if (datestr) {
-		ctxt->item->time = parseISO8601Date (datestr);
+		ctxt->item->time = date_parse_ISO8601 (datestr);
 		ctxt->item->metadata = metadata_list_append (ctxt->item->metadata, "pubDate", datestr);
 		g_free (datestr);
 	}
@@ -451,7 +447,7 @@ atom10_parse_entry_title (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10P
 {
 	gchar *title;
 	
-	title = common_utf8_fix (atom10_parse_text_construct(cur, FALSE));
+	title = atom10_parse_text_construct(cur, FALSE);
 	if (title) {
 		item_set_title (ctxt->item, title);
 		g_free (title);
@@ -463,10 +459,10 @@ atom10_parse_entry_updated (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom1
 {
 	gchar *datestr;
 	
-	datestr = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
+	datestr = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
 	/* if pubDate is already set, don't overwrite it */
 	if (datestr && !metadata_list_get(ctxt->item->metadata, "pubDate")) {
-		ctxt->item->time = parseISO8601Date (datestr);
+		ctxt->item->time = date_parse_ISO8601 (datestr);
 		ctxt->item->metadata = metadata_list_append (ctxt->item->metadata, "contentUpdateDate", datestr);
 	}
 
@@ -619,7 +615,15 @@ atom10_parse_feed_generator (xmlNodePtr cur, feedParserCtxtPtr ctxt, itemPtr ip,
 static void
 atom10_parse_feed_icon (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state)
 {
-	/* FIXME: Parse icon and use as a favicon? */
+	gchar *icon_uri;
+
+	icon_uri = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
+
+	if (icon_uri) {
+		debug1 (DEBUG_PARSING, "icon URI found in atom feed: %s", icon_uri);
+		ctxt->subscription->metadata = metadata_list_append (ctxt->subscription->metadata,
+								     "icon", icon_uri);
+	}
 }
 
 static void
@@ -695,15 +699,37 @@ atom10_parse_feed_title (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10Pa
 	}
 }
 
+/* Sort items in descending date order (newer items first). */
+static gint
+atom10_item_sort_by_date (gconstpointer a, gconstpointer b)
+{
+	itemPtr item1 = (itemPtr)a;
+	itemPtr item2 = (itemPtr)b;
+
+	g_assert (item1 && item2);
+
+	if (item1->time == item2->time) {
+		/* Items identical.. can we distinguish further? */
+		return 0;
+	}
+	
+	if (item1->time < item2->time)
+		return 1;
+	if (item1->time > item2->time)
+		return -1;
+
+	return 0;
+}
+
 static void
 atom10_parse_feed_updated (xmlNodePtr cur, feedParserCtxtPtr ctxt, struct atom10ParserState *state)
 {
 	gchar *timestamp;
 	
-	timestamp = common_utf8_fix ((gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1));
+	timestamp = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
 	if (timestamp) {
 		ctxt->subscription->metadata = metadata_list_append (ctxt->subscription->metadata, "contentUpdateDate", timestamp);
-		ctxt->feed->time = parseISO8601Date (timestamp);
+		ctxt->feed->time = date_parse_ISO8601 (timestamp);
 		g_free (timestamp);
 	}
 }
@@ -789,7 +815,7 @@ atom10_parse_feed (feedParserCtxtPtr ctxt, xmlNodePtr cur)
 			} else if (xmlStrEqual (cur->name, BAD_CAST"entry")) {
 				ctxt->item = atom10_parse_entry (ctxt, cur);
 				if (ctxt->item)
-					ctxt->items = g_list_append (ctxt->items, ctxt->item);
+					ctxt->items = g_list_insert_sorted (ctxt->items, ctxt->item, atom10_item_sort_by_date);
 			}
 			cur = cur->next;
 		}
@@ -846,11 +872,9 @@ atom10_init_feed_handler (void)
 		atom10_add_ns_handler (ns_georss_get_handler ());
 	}	
 	/* prepare feed handler structure */
-	fhp->typeStr = "pie";
-	fhp->icon = ICON_AVAILABLE;
+	fhp->typeStr = "atom";
 	fhp->feedParser	= atom10_parse_feed;
 	fhp->checkFormat = atom10_format_check;
-	fhp->merge = TRUE;
 
 	return fhp;
 }
