@@ -1,7 +1,7 @@
 /**
  * @file pie_feed.c Atom 0.3 channel parsing
  * 
- * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2010 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,18 +18,17 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include "pie_feed.h"
 
 #include <sys/time.h>
 #include <string.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
+
 #include "common.h"
+#include "date.h"
 #include "feed_parser.h"
 #include "feedlist.h"
-#include "pie_feed.h"
 #include "pie_entry.h"
 #include "ns_dc.h"
 #include "metadata.h"
@@ -64,7 +63,7 @@ gchar* pie_parse_content_construct(xmlNodePtr cur) {
 		if(!strcmp(mode, "escaped")) {
 			gchar	*tmp;
 
-			tmp = common_utf8_fix(xhtml_extract (cur, 0, NULL));
+			tmp = xhtml_extract (cur, 0, NULL);
 			if(NULL != tmp)
 				ret = tmp;
 			
@@ -89,7 +88,7 @@ gchar* pie_parse_content_construct(xmlNodePtr cur) {
 			!g_ascii_strcasecmp(type, "TEXT") ||
 			!strcmp(type, "text/plain")) {
 			gchar *tmp;
-			tmp = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+			tmp = (gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1);
 			ret = g_markup_printf_escaped("<div xmlns=\"http://www.w3.org/1999/xhtml\"><pre>%s</pre></div>", tmp);
 			g_free(tmp);
 			/* Next are things that contain subttags */
@@ -127,10 +126,10 @@ gchar * parseAuthor(xmlNodePtr cur) {
 		}
 		
 		if (!xmlStrcmp(cur->name, BAD_CAST"name"))
-			tmp = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+			tmp = (gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1);
 
 		if (!xmlStrcmp(cur->name, BAD_CAST"email")) {
-			tmp2 = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
+			tmp2 = (gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1);
 			tmp3 = g_strdup_printf("%s <a href=\"mailto:%s\">%s</a>", tmp, tmp2, tmp2);
 			g_free(tmp);
 			g_free(tmp2);
@@ -138,8 +137,8 @@ gchar * parseAuthor(xmlNodePtr cur) {
 		}
 					
 		if (!xmlStrcmp(cur->name, BAD_CAST"url")) {
-			tmp2 = common_utf8_fix((gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1));
-			tmp3 = g_strdup_printf("%s (<a href=\"%s\">Website</a>)", tmp, tmp2);
+			tmp2 = (gchar *)xmlNodeListGetString(cur->doc, cur->xmlChildrenNode, 1);
+			tmp3 = g_strdup_printf("%s (<a href=\"%s\">%s</a>)", tmp, tmp2, _("Website"));
 			g_free(tmp);
 			g_free(tmp2);
 			tmp = tmp3;
@@ -205,7 +204,7 @@ static void pie_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 					g_free(tmp);
 				} else {
 					/* 0.2 link : element content is the link, or non-alternate link in 0.3 */
-					tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+					tmp = (gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1);
 					if(tmp) {
 						subscription_set_homepage (ctxt->subscription, tmp);
 						g_free(tmp);
@@ -221,7 +220,7 @@ static void pie_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 					g_free(tmp);
 				}
 			} else if (!xmlStrcmp (cur->name, BAD_CAST"tagline")) {
-				tmp = common_utf8_fix (pie_parse_content_construct (cur));
+				tmp = pie_parse_content_construct (cur);
 				if (tmp) {
 					metadata_list_set (&ctxt->subscription->metadata, "description", tmp);
 					g_free (tmp);				
@@ -247,25 +246,25 @@ static void pie_parse(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 				}
 				g_free(tmp);
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"copyright")) {
-				tmp = common_utf8_fix(pie_parse_content_construct(cur));
+				tmp = pie_parse_content_construct(cur);
 				if(tmp) {
 					ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "copyright", tmp);
 					g_free(tmp);
 				}				
 				
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"modified")) { /* Modified was last used in IETF draft 02) */
-				tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+				tmp = (gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1);
 				if(tmp) {
 					ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "pubDate", tmp);
-					ctxt->feed->time = parseISO8601Date(tmp);
+					ctxt->feed->time = date_parse_ISO8601 (tmp);
 					g_free(tmp);
 				}
 
 			} else if(!xmlStrcmp(cur->name, BAD_CAST"updated")) { /* Updated was added in IETF draft 03 */
-				tmp = common_utf8_fix((gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1));
+				tmp = (gchar *)xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, 1);
 				if(tmp) {
 					ctxt->subscription->metadata = metadata_list_append(ctxt->subscription->metadata, "pubDate", tmp);
-					ctxt->feed->time = parseISO8601Date(tmp);
+					ctxt->feed->time = date_parse_ISO8601 (tmp);
 					g_free(tmp);
 				}
 
@@ -326,11 +325,9 @@ pie_init_feed_handler (void)
 
 
 	/* prepare feed handler structure */
-	fhp->typeStr = "pie";
-	fhp->icon = ICON_AVAILABLE;
+	fhp->typeStr = "atom";
 	fhp->feedParser	= pie_parse;
 	fhp->checkFormat = pie_format_check;
-	fhp->merge = TRUE;
 
 	return fhp;
 }
