@@ -1,7 +1,7 @@
 /**
  * @file render.c  generic XSLT rendering handling
  * 
- * Copyright (C) 2006-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2011 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,22 +58,26 @@
    and performs CSS adaptions to the current GTK theme. */
 
 static renderParamPtr	langParams = NULL;	/* the current locale settings (for localization stylesheet) */
-static gchar		*defaultParams = NULL;	/* some default parameters (for rendering stylesheets) */
 
 static GHashTable	*stylesheets = NULL;	/* XSLT stylesheet cache */
 
 static void
 render_init (void)
 {
-	gchar   **shortlang = NULL;	/* e.g. "de" */
-	gchar	**lang = NULL;		/* e.g. "de_AT" */
+	gchar   	**shortlang = NULL;	/* e.g. "de" */
+	gchar		**lang = NULL;		/* e.g. "de_AT" */
+	gchar		*filename;
 
 	if (langParams)
 		render_parameter_free (langParams);
-	if (defaultParams)
-		g_free (defaultParams);
 
-	/* prepare localization parameters */
+	/* Install default stylesheet if it does not yet exist */
+	filename = common_create_cache_filename ("", "liferea", "css");
+	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+		common_copy_file (PACKAGE_DATA_DIR "/" PACKAGE "/css/user.css", filename);
+	g_free(filename);
+
+	/* Prepare localization parameters */
 	debug1 (DEBUG_HTML, "XSLT localisation: setlocale(LC_MESSAGES, NULL) reports '%s'", setlocale(LC_MESSAGES, NULL));
 	lang = g_strsplit (setlocale (LC_MESSAGES, NULL), "@", 0);
 	shortlang = g_strsplit (setlocale (LC_MESSAGES, NULL), "_", 0);
@@ -86,9 +90,6 @@ render_init (void)
 	g_strfreev (shortlang);
 	g_strfreev (lang);
 
-	/* prepare rendering default parameters */
-	defaultParams = g_strdup_printf("search_link_enable='%s'", conf_get_bool_value (SOCIAL_LINK_SEARCH_HIDE)?"false":"true");
-
 	if (!stylesheets)
 		stylesheets = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
 }
@@ -148,7 +149,6 @@ render_load_stylesheet (const gchar *xsltName)
 
 /** cached CSS definitions */
 static GString	*css = NULL;
-static time_t lastCssModification = 0;
 
 /** widget background theme colors as 8bit HTML RGB code */
 typedef struct themeColor {
@@ -225,102 +225,41 @@ render_set_theme_colors (gchar *css)
 	return css;
 }
 
-static void
-render_remove_css_comments (void)
-{
-	gssize i, len = css->len;
-	gchar *str = css->str;
-	GString *newcss;
-
-	newcss = g_string_new(NULL);
-	i = 0;
-	while (i < len) {
-		if (i+1 < len && str[i] == '/' && str[i+1] == '*') {
-			i += 4;
-			while (i-1 < len && (str[i-2] != '*' || str[i-1] != '/'))
-				i++;
-			continue;
-		}
-		if (i+1 < len && str[i] == '/' && str[i+1] == '/') {
-			i += 2;
-			while (i < len && str[i] != '\n')
-				i++;
-			continue;
-		}
-		newcss = g_string_append_c(newcss, str[i]);
-		i++;
-	}
-	g_string_free(css, TRUE);
-	css = newcss;
-}
-
 const gchar *
 render_get_css (gboolean externalCss)
 {
-	gchar *styleSheetFile;
-	time_t newLastModification;
-
-	styleSheetFile      = g_build_filename (common_get_cache_path (), "liferea.css", NULL);
-	newLastModification = common_get_mod_time ((char *)styleSheetFile);
-
-	if (!css || lastCssModification != newLastModification) {
-		gchar	*defaultStyleSheetFile, *adblockStyleSheetFile;
-		gchar	*font = NULL;
-		gchar	*fontsize = NULL;
+	if (!css) {
+		gchar	*defaultStyleSheetFile;
+		gchar	*userStyleSheetFile;
+		gchar	*adblockStyleSheetFile;
 		gchar	*tmp;
 
-		// Update last modification timestamp
-		lastCssModification = newLastModification;
-
-		if (themeColors == NULL) {
+		if (!themeColors)
 			render_get_theme_colors();
-		}
 
 		css = g_string_new(NULL);
 
-		/* font configuration support */
-		font = conf_get_str_value (USER_FONT);
-		if(0 == strlen(font)) {
-			g_free(font);
-			font = conf_get_str_value (DEFAULT_FONT);
-		}
-
-		if(font) {
-			fontsize = font;
-			/* the GTK2/GNOME font name format is <font name>,<font size in point>
-			 Or it can also be "Font Name size*/
-			strsep(&fontsize, ",");
-			if(fontsize == NULL) {
-				if(NULL != (fontsize = strrchr(font, ' '))) {
-					*fontsize = '\0';
-					fontsize++;
-				}
-			}
-			g_string_append(css, "body, table, div {");
-			g_string_append_printf(css, "font-family: %s;\n", font);
-
-			if(fontsize)
-				g_string_append_printf(css, "font-size: %spt;\n", fontsize);
-
-			g_free(font);
-			g_string_append(css, "}\n");
-		}
-
 		defaultStyleSheetFile = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "css", "liferea.css", NULL);
 
 		if (g_file_get_contents(defaultStyleSheetFile, &tmp, NULL, NULL)) {
 			tmp = render_set_theme_colors(tmp);
 			g_string_append(css, tmp);
 			g_free(tmp);
+		} else {
+			g_error ("Loading %s failed.", defaultStyleSheetFile);
 		}
 
-		if (g_file_get_contents(styleSheetFile, &tmp, NULL, NULL)) {
+		g_free(defaultStyleSheetFile);
+
+		userStyleSheetFile = g_build_filename (common_get_cache_path (), "liferea.css", NULL);
+
+		if (g_file_get_contents(userStyleSheetFile, &tmp, NULL, NULL)) {
 			tmp = render_set_theme_colors(tmp);
 			g_string_append(css, tmp);
 			g_free(tmp);
 		}
 
-		g_free(defaultStyleSheetFile);
+		g_free(userStyleSheetFile);
 
 		adblockStyleSheetFile = g_build_filename(PACKAGE_DATA_DIR, PACKAGE, "css", "adblock.css", NULL);
 
@@ -331,9 +270,6 @@ render_get_css (gboolean externalCss)
 
 		g_free(adblockStyleSheetFile);
 
-		/* remove comments from the CSS to circumvent libgtkhtml2 freezing bug */
-		render_remove_css_comments();
-
 		if (externalCss) {
 			/* dump CSS to cache file and create a <style> tag to use the it */
 			gchar *filename = common_create_cache_filename("cache", "style", "css");
@@ -354,8 +290,6 @@ render_get_css (gboolean externalCss)
 		}
 	}
 
-	g_free(styleSheetFile);
-
 	return css->str;
 }
 
@@ -373,9 +307,9 @@ render_xml (xmlDocPtr doc, const gchar *xsltName, renderParamPtr paramSet)
 
 	if (!paramSet)
 		paramSet = render_parameter_new ();
-	render_parameter_add (paramSet, "%s", defaultParams);
+	render_parameter_add (paramSet, "pixmapsDir='file://" PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "'");
 
-	resDoc = xsltApplyStylesheet(xslt, doc, (const gchar **)paramSet->params);
+	resDoc = xsltApplyStylesheet (xslt, doc, (const gchar **)paramSet->params);
 	if (!resDoc) {
 		g_warning ("fatal: applying rendering stylesheet (%s) failed!", xsltName);
 		return NULL;
@@ -415,24 +349,6 @@ render_xml (xmlDocPtr doc, const gchar *xsltName, renderParamPtr paramSet)
 	return output;
 }
 
-gchar *
-render_file (const gchar *filename, const gchar *xsltName, renderParamPtr paramSet)
-{
-	xmlDocPtr	srcDoc;
-	gchar		*output;
-
-	srcDoc = xmlParseFile(filename);
-	if (!srcDoc) {
-		g_warning ("fatal: loading source XML (%s) failed", filename);
-		return NULL;
-	}
-
-	output = render_xml (srcDoc, xsltName, paramSet);
-	xmlFreeDoc (srcDoc);
-
-	return output;
-}
-
 /* parameter handling */
 
 renderParamPtr
