@@ -1,7 +1,7 @@
 /**
  * @file enclosure-list-view.c enclosures/podcast handling GUI
  *
- * Copyright (C) 2005-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2005-2011 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include "item.h"
 #include "metadata.h"
 #include "ui/liferea_dialog.h"
-#include "ui/ui_popup.h"
+#include "ui/popup_menu.h"
 #include "ui/ui_prefs.h"
 
 /* enclosure list view implementation */
@@ -44,9 +44,6 @@ enum {
 	ES_LEN
 };
 
-static void enclosure_list_view_class_init	(EnclosureListViewClass *klass);
-static void enclosure_list_view_init		(EnclosureListView *ld);
-
 #define ENCLOSURE_LIST_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), ENCLOSURE_LIST_VIEW_TYPE, EnclosureListViewPrivate))
 
 struct EnclosureListViewPrivate {
@@ -59,34 +56,7 @@ struct EnclosureListViewPrivate {
 
 static GObjectClass *parent_class = NULL;
 
-GType
-enclosure_list_view_get_type (void) 
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) 
-	{
-		static const GTypeInfo our_info = 
-		{
-			sizeof (EnclosureListViewClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) enclosure_list_view_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (EnclosureListView),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) enclosure_list_view_init,
-			NULL /* value_table */
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EnclosureListView",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EnclosureListView, enclosure_list_view, G_TYPE_OBJECT);
 
 static void
 enclosure_list_view_finalize (GObject *object)
@@ -146,6 +116,23 @@ on_enclosure_list_button_press (GtkWidget *treeview, GdkEventButton *event, gpoi
 	return TRUE;
 }
 
+static gboolean
+on_enclosure_list_popup_menu (GtkWidget *widget, gpointer user_data)
+{
+	GtkTreeView		*treeview = GTK_TREE_VIEW (widget);
+	GtkTreeModel		*model;
+	GtkTreeIter		iter;
+
+	if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (treeview), &model, &iter)) {
+		enclosurePtr enclosure;	
+		gtk_tree_model_get (model, &iter, ES_PTR, &enclosure, -1);
+		ui_popup_enclosure_menu (enclosure, 3, 0);
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
 static gboolean
 on_enclosure_list_activate (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data)
 {
@@ -216,11 +203,9 @@ enclosure_list_view_new ()
 	
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (elv->priv->treeview), FALSE);
 
-	g_signal_connect ((gpointer)elv->priv->treeview, "button_press_event",
-	                  G_CALLBACK (on_enclosure_list_button_press), (gpointer)elv);
-			  
-	g_signal_connect ((gpointer)elv->priv->treeview, "row-activated",
-	                  G_CALLBACK (on_enclosure_list_activate), (gpointer)elv);
+	g_signal_connect (G_OBJECT (elv->priv->treeview), "button_press_event", G_CALLBACK (on_enclosure_list_button_press), (gpointer)elv);
+	g_signal_connect (G_OBJECT (elv->priv->treeview), "row-activated", G_CALLBACK (on_enclosure_list_activate), (gpointer)elv);
+	g_signal_connect (G_OBJECT (elv->priv->treeview), "popup_menu", G_CALLBACK (on_enclosure_list_popup_menu), (gpointer)elv);
 
 	g_signal_connect_object (elv->priv->container, "destroy", G_CALLBACK (enclosure_list_view_destroy_cb), elv, 0);
 
@@ -248,7 +233,7 @@ enclosure_list_view_load (EnclosureListView *elv, itemPtr item)
 	}
 	g_slist_free (elv->priv->enclosures);
 	elv->priv->enclosures = NULL;	
-
+	
 	/* decide visibility of the list */
 	list = metadata_list_get_values (item->metadata, "enclosure");
 	len = g_slist_length (list);
@@ -350,7 +335,7 @@ on_selectcmd_pressed (GtkButton *button, gpointer user_data)
 	utfname = gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (dialog,"enc_cmd_entry")));
 	name = g_filename_from_utf8 (utfname, -1, NULL, NULL, NULL);
 	if (name) {
-		ui_choose_file (_("Choose File"), GTK_STOCK_OPEN, FALSE, on_selectcmdok_clicked, name, NULL, dialog);
+		ui_choose_file (_("Choose File"), GTK_STOCK_OPEN, FALSE, on_selectcmdok_clicked, name, NULL, NULL, NULL, dialog);
 		g_free (name);
 	}
 }
@@ -384,6 +369,8 @@ on_adddialog_response (GtkDialog *dialog, gint response_id, gpointer user_data)
 		etp->remote = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (liferea_dialog_lookup (GTK_WIDGET (dialog), "enc_remote_open_btn")));
 		if (new)
 			enclosure_mime_type_add (etp);
+		else
+			enclosure_mime_types_save ();
 
 		/* now we have ensured an existing type configuration and
 		   can launch the URL for which we configured the type */
@@ -403,7 +390,6 @@ ui_enclosure_type_setup (encTypePtr type, enclosurePtr enclosure, gchar *typestr
 	gchar		*tmp;
 
 	dialog = liferea_dialog_new (NULL, "enchandlerdialog");
-
 	if (type) {
 		typestr = type->mime?type->mime:type->extension;
 		gtk_entry_set_text (GTK_ENTRY (liferea_dialog_lookup (dialog, "enc_cmd_entry")), type->cmd);
@@ -412,10 +398,10 @@ ui_enclosure_type_setup (encTypePtr type, enclosurePtr enclosure, gchar *typestr
 	}
 
 	if (!strchr(typestr, '/')) 
-		tmp = g_strdup_printf (_("<b>File Extension .%s</b>"), typestr);
+		tmp = g_strdup_printf (_("File Extension .%s"), typestr);
 	else
-		tmp = g_strdup_printf ("<b>%s</b>", typestr);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (liferea_dialog_lookup (dialog, "enc_type_label")), tmp);
+		tmp = g_strdup_printf ("%s", typestr);
+	gtk_label_set_text (GTK_LABEL (liferea_dialog_lookup (dialog, "enc_type_label")), tmp);
 	g_free (tmp);
 
 	g_object_set_data (G_OBJECT(dialog), "typestr", g_strdup (typestr));
@@ -438,14 +424,25 @@ on_popup_open_enclosure (gpointer callback_data)
 
 	/* 1.) Always try to determine the file extension... */
 	
-	/* FIXME: improve this to match only '.' not followed by '/' chars */
+	/* find extension by looking for last '.' */
 	typestr = strrchr (enclosure->url, '.');
 	if (typestr)
 		typestr = g_strdup (typestr + 1);
-			
+
+	/* handle case where there is a slash after the '.' */
+	if (typestr && strrchr (typestr, '/'))
+		typestr = strrchr (typestr, '/');
+		
+	/* handle case where there is no '.' at all */
+	if (!typestr && strrchr (enclosure->url, '/'))
+		typestr = strrchr (enclosure->url, '/');
+	
 	/* if we found no extension we map to dummy type "data" */
 	if (!typestr)
 		typestr = g_strdup ("data");
+		
+	/* strip GET parameters from typestr */	
+	g_strdelimit (typestr, "?", 0);
 
 	debug2 (DEBUG_CACHE, "url:%s, mime:%s", enclosure->url, enclosure->mime);
 	
@@ -511,7 +508,12 @@ on_popup_save_enclosure (gpointer callback_data)
 	else
 		filename = enclosure->url;
 		
-	ui_choose_file (_("Choose File"), GTK_STOCK_SAVE_AS, TRUE, on_encsave_clicked, NULL, filename, enclosure->url);
+	filename = g_strdup (filename);
+	g_strdelimit (filename, "?", 0);	/* strip GET parameters */
+		
+	ui_choose_file (_("Choose File"), GTK_STOCK_SAVE_AS, TRUE, on_encsave_clicked, NULL, filename, NULL, NULL, enclosure->url);
+
+	g_free (filename);
 }
 
 void
