@@ -1,7 +1,7 @@
 /**
  * @file itemview.c  viewing feed content in different presentation modes
  * 
- * Copyright (C) 2006-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2011 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,11 +31,16 @@
 #include "vfolder.h"
 #include "ui/ui_common.h"
 #include "ui/liferea_shell.h"
-#include "ui/ui_itemlist.h"
+#include "ui/item_list_view.h"
 #include "ui/liferea_htmlview.h"
 
-static void itemview_class_init	(ItemViewClass *klass);
-static void itemview_init	(ItemView *fl);
+/* The item view is the layer that switches item list presentations:
+   a HTML single item or list and GtkTreeView list presentation. 
+   It hides the item loading behaviour of GtkTreeView and HTML view.
+
+   The item view does not handle item filtering, which is done by
+   the item list implementation.
+ */
 
 #define ITEMVIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), ITEMVIEW_TYPE, ItemViewPrivate))
 
@@ -43,18 +48,18 @@ struct ItemViewPrivate {
 	gboolean	htmlOnly;		/**< TRUE if HTML only mode */
 	guint		mode;			/**< current item view mode */
 	nodePtr		node;			/**< the node whose item are displayed */
-	gchar 		*userDefinedDateFmt;	/**< user defined date formatting string */
 	gboolean	needsHTMLViewUpdate;	/**< flag to be set when HTML rendering is to be 
 						     updated, used to delay HTML updates */
+	gboolean	hasEnclosures;		/**< TRUE if at least one item of the current itemset has an enclosure */
 						     
 	nodeViewType	viewMode;		/**< current viewing mode */
 	guint		currentLayoutMode;	/**< layout mode (3 pane, 2 pane, wide view) */
 								     
-	GtkWidget	*itemlistContainer;	/**< scrolled window holding item list tree view */
-	GtkTreeView	*itemlist;		
+	GtkWidget	*itemListViewContainer;	/**< scrolled window holding item list tree view */
+	ItemListView	*itemListView;		/**< widget instance used to present items in list mode */
 
 	EnclosureListView	*enclosureView;	/**< Enclosure list widget */
-	LifereaHtmlView		*htmlview;	/**< HTML rendering widget used for item display */
+	LifereaHtmlView		*htmlview;	/**< HTML rendering widget instance used to render single items and summaries mode */
 
 	gfloat			zoom;		/**< HTML rendering widget zoom level */
 };
@@ -62,34 +67,7 @@ struct ItemViewPrivate {
 static GObjectClass *parent_class = NULL;
 static ItemView *itemview = NULL;
 
-GType
-itemview_get_type (void) 
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) 
-	{
-		static const GTypeInfo our_info = 
-		{
-			sizeof (ItemViewClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) itemview_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (ItemView),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) itemview_init,
-			NULL /* value_table */
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "ItemView",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (ItemView, itemview, G_TYPE_OBJECT);
 
 static void
 itemview_finalize (GObject *object)
@@ -105,8 +83,8 @@ itemview_finalize (GObject *object)
 	
 	if (priv->enclosureView)
 		g_object_unref (priv->enclosureView);
-		
-	g_free (priv->userDefinedDateFmt);
+	if (priv->itemListView)
+		g_object_unref (priv->itemListView);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -126,11 +104,10 @@ itemview_class_init (ItemViewClass *klass)
 void
 itemview_clear (void) 
 {
-
-	ui_itemlist_clear ();
+	item_list_view_clear (itemview->priv->itemListView);
 	htmlview_clear ();
 	enclosure_list_view_hide (itemview->priv->enclosureView);
-	
+	itemview->priv->hasEnclosures = FALSE;
 	itemview->priv->needsHTMLViewUpdate = TRUE;
 }
 
@@ -151,32 +128,21 @@ itemview_set_displayed_node (nodePtr node)
 		
 	itemview->priv->node = node;
 
-	/* 1. Perform UI item list preparations ... */
-
-	/* Free the old itemstore and create a new one; this is the only way to disable sorting */
-	ui_itemlist_reset_tree_store ();	 /* this also clears the itemlist. */
-
-	/* Disable attachment icon column (will be enabled when loading first item with an enclosure) */
-	ui_itemlist_enable_encicon_column (FALSE);
-
-	if (node) {
-		ui_itemlist_enable_favicon_column (NODE_TYPE (node)->capabilities & NODE_CAPABILITY_SHOW_ITEM_FAVICONS);
-		ui_itemlist_set_sort_column (node->sortColumn, node->sortReversed);
-	}
-
-	/* 2. Reset view state */
+	/* 1. Reset view state */
 	itemview_clear ();
 
-	/* 3. And prepare HTML view */
+	/* 2. And prepare HTML view */
 	htmlview_set_displayed_node (node);
 }
 
 void
 itemview_add_item (itemPtr item)
 {
+	itemview->priv->hasEnclosures |= item->hasEnclosure;
+
 	if (ITEMVIEW_ALL_ITEMS != itemview->priv->mode)
 		/* add item in 3 pane mode */
-		ui_itemlist_add_item (item);
+		item_list_view_add_item (itemview->priv->itemListView, item);
 	else
 		/* force HTML update in 2 pane mode */
 		itemview->priv->needsHTMLViewUpdate = TRUE;
@@ -187,12 +153,12 @@ itemview_add_item (itemPtr item)
 void
 itemview_remove_item (itemPtr item)
 {
-	if (!ui_itemlist_contains_item (item->id))
+	if (!item_list_view_contains_id (itemview->priv->itemListView, item->id))
 		return;
 
 	if (ITEMVIEW_ALL_ITEMS != itemview->priv->mode)
 		/* remove item in 3 pane mode */
-		ui_itemlist_remove_item (item);
+		item_list_view_remove_item (itemview->priv->itemListView, item);
 	else
 		/* force HTML update in 2 pane mode */
 		itemview->priv->needsHTMLViewUpdate = TRUE;
@@ -207,7 +173,7 @@ itemview_select_item (itemPtr item)
 		
 	ivp->needsHTMLViewUpdate = TRUE;
 	
-	ui_itemlist_select (item);
+	item_list_view_select (ivp->itemListView, item);
 	htmlview_select_item (item);
 
 	if (item)
@@ -221,13 +187,13 @@ itemview_update_item (itemPtr item)
 {
 	/* Always update the GtkTreeView (bail-out done in ui_itemlist_update_item() */
 	if (ITEMVIEW_ALL_ITEMS != itemview->priv->mode)
-		ui_itemlist_update_item (item);
+		item_list_view_update_item (itemview->priv->itemListView, item);
 
 	/* Bail out if no HTML update necessary */
 	switch (itemview->priv->mode) {
 		case ITEMVIEW_ALL_ITEMS:
 			/* No HTML update needed if 2 pane mode and item not in item set */
-			if (!ui_itemlist_contains_item (item->id))
+			if (!item_list_view_contains_id (itemview->priv->itemListView, item->id))
 				return;
 			break;
 		case ITEMVIEW_SINGLE_ITEM:		
@@ -250,7 +216,7 @@ itemview_update_all_items (void)
 {
 	/* Always update the GtkTreeView (bail-out done in ui_itemlist_update_item() */
 	if (ITEMVIEW_ALL_ITEMS != itemview->priv->mode)
-		ui_itemlist_update_all_items ();
+		item_list_view_update_all_items (itemview->priv->itemListView);
 		
 	itemview->priv->needsHTMLViewUpdate = TRUE;
 	htmlview_update_all_items ();
@@ -275,6 +241,13 @@ itemview_update_node_info (nodePtr node)
 void
 itemview_update (void)
 {
+	item_list_view_update (itemview->priv->itemListView, itemview->priv->hasEnclosures);
+	
+	if (itemview->priv->node) {
+		item_list_view_enable_favicon_column (itemview->priv->itemListView, NODE_TYPE (itemview->priv->node)->capabilities & NODE_CAPABILITY_SHOW_ITEM_FAVICONS);
+		item_list_view_set_sort_column (itemview->priv->itemListView, itemview->priv->node->sortColumn, itemview->priv->node->sortReversed);
+	}
+	
 	if (itemview->priv->needsHTMLViewUpdate) {
 		itemview->priv->needsHTMLViewUpdate = FALSE;
 		htmlview_update (itemview->priv->htmlview, itemview->priv->mode);
@@ -289,19 +262,26 @@ itemview_display_info (const gchar *html)
 	liferea_htmlview_write (itemview->priv->htmlview, html, NULL);
 }
 
-/* date format handling (not sure if this is the right place) */
+/* next unread selection logic */
 
-gchar *
-itemview_format_date (time_t date)
+itemPtr
+itemview_find_unread_item (gulong startId) 
 {
-	gchar		*timestr;
-
-	if (itemview->priv->userDefinedDateFmt)
-		timestr = common_format_date (date, itemview->priv->userDefinedDateFmt);
-	else
-		timestr = common_format_nice_date (date);
+	itemPtr	result = NULL;
 	
-	return timestr;
+	/* Note: to select in sorting order we need to do it in the ItemListView
+	   otherwise we would have to sort the item list here... */
+
+	/* First do a scan from the start position (usually the selected 
+	   item to the end of the sorted item list) if one is given. */	
+	if (startId)
+		result = item_list_view_find_unread_item (itemview->priv->itemListView, startId);
+	
+	/* Now perform a wrap around by searching again from the top */
+	if (!result)
+		result = item_list_view_find_unread_item (itemview->priv->itemListView, 0);
+
+	return result;
 }
 
 void
@@ -319,13 +299,13 @@ itemview_scroll (void)
 void
 itemview_move_cursor (int step)
 {
-	ui_common_treeview_move_cursor (itemview->priv->itemlist, step);
+	ui_common_treeview_move_cursor (item_list_view_get_widget (itemview->priv->itemListView), step);
 }
 
 void
 itemview_move_cursor_to_first (void)
 {
-	ui_common_treeview_move_cursor_to_first (itemview->priv->itemlist);
+	ui_common_treeview_move_cursor_to_first (item_list_view_get_widget (itemview->priv->itemListView));
 }
 
 static void
@@ -396,6 +376,7 @@ itemview_set_layout (nodeViewType newMode)
 			ilWidgetName = NULL;
 			encViewVBoxName = NULL;
 			break;
+		case NODE_VIEW_MODE_INVALID:
 		default:
 			g_warning("fatal: illegal viewing mode!");
 			return;
@@ -406,7 +387,7 @@ itemview_set_layout (nodeViewType newMode)
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (liferea_shell_lookup ("itemtabs")), newMode);
 	gtk_widget_reparent (liferea_htmlview_get_widget (ivp->htmlview), liferea_shell_lookup (htmlWidgetName));
 	if (ilWidgetName)
-		gtk_widget_reparent (GTK_WIDGET (ivp->itemlistContainer), liferea_shell_lookup (ilWidgetName));
+		gtk_widget_reparent (GTK_WIDGET (ivp->itemListViewContainer), liferea_shell_lookup (ilWidgetName));
 	
 	/* Destroy previous enclosure list. */
 	if (ivp->enclosureView) {
@@ -425,43 +406,30 @@ itemview_set_layout (nodeViewType newMode)
 }
 
 ItemView *
-itemview_create (GtkWidget *ilc)
+itemview_create (GtkWidget *window)
 {
+	gint zoom;
+
 	/* 1. Create widgets, load preferences */
 	
 	g_object_new (ITEMVIEW_TYPE, NULL);
 	
 	itemview->priv->currentLayoutMode = NODE_VIEW_MODE_INVALID;
-	itemview->priv->itemlistContainer = ilc;
-	itemview->priv->itemlist = GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (itemview->priv->itemlistContainer)));
-	itemview->priv->zoom = conf_get_int_value (LAST_ZOOMLEVEL);
-	itemview->priv->userDefinedDateFmt = conf_get_str_value (DATE_FORMAT);
-
+	itemview->priv->itemListView = item_list_view_create (window);
+	itemview->priv->itemListViewContainer = gtk_widget_get_parent (GTK_WIDGET (item_list_view_get_widget (itemview->priv->itemListView)));
+	conf_get_int_value (LAST_ZOOMLEVEL, &zoom);
+	
+	if (zoom == 0) {
+		zoom = 100;
+		conf_set_int_value (LAST_ZOOMLEVEL, zoom);
+	}
+	itemview->priv->zoom = zoom;
+	
 	/* initially we pack the item list in the normal view pane,
 	   which is later changed in itemview_set_layout() */
-	gtk_container_add (GTK_CONTAINER (liferea_shell_lookup ("normalViewItems")), itemview->priv->itemlistContainer);
+	gtk_container_add (GTK_CONTAINER (liferea_shell_lookup ("normalViewItems")), itemview->priv->itemListViewContainer);
 
-	/* 2. Sanity checks on the settings... */
-	
-	/* We now have an empty string or a format string... */
-	if (itemview->priv->userDefinedDateFmt && !strlen (itemview->priv->userDefinedDateFmt)) {
-		/* It's empty and useless... */
-		g_free (itemview->priv->userDefinedDateFmt);
-		itemview->priv->userDefinedDateFmt = NULL;
-	}
-	
-	if (itemview->priv->userDefinedDateFmt) {
-		debug1 (DEBUG_GUI, "user defined date format is: >>>%s<<<", itemview->priv->userDefinedDateFmt);
-	} else {
-		debug0 (DEBUG_GUI, "using default date format");
-	}
-
-	if (0 == itemview->priv->zoom) {	/* workaround for scheme problem with the last releases */
-		itemview->priv->zoom = 100;
-		conf_set_int_value (LAST_ZOOMLEVEL, 100);
-	}
-	
-	/* 3. Prepare HTML rendering */
+	/* 2. Prepare HTML rendering */
 	htmlview_init ();
 	
 	return itemview;
@@ -493,6 +461,6 @@ itemview_launch_URL (const gchar *url, gboolean forceInternal)
 void
 itemview_do_zoom (gboolean in)
 {
-g_assert(itemview->priv->htmlview != NULL);
+	g_assert(itemview->priv->htmlview != NULL);
 	liferea_htmlview_do_zoom (itemview->priv->htmlview, in);
 }
