@@ -2,7 +2,7 @@
  * @file ui_shell.c  UI layout handling
  *
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2007-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2011 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,10 +26,7 @@
 #include "ui/liferea_shell.h"
 
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
-#include <string.h>		/* strcmp() and strsep() */
 
 #include "browser.h"
 #include "common.h"
@@ -39,32 +36,24 @@
 #include "feedlist.h"
 #include "htmlview.h"
 #include "itemlist.h"
-#include "net.h"
-#include "notification/notification.h"
+#include "net_monitor.h"
 #include "ui/browser_tabs.h"
+#include "ui/feed_list_view.h"
+#include "ui/icons.h"
 #include "ui/itemview.h"
+#include "ui/item_list_view.h"
 #include "ui/liferea_dialog.h"
-#include "ui/ui_common.h"
-#include "ui/ui_feedlist.h"
-#include "ui/ui_itemlist.h"
-#include "ui/ui_prefs.h"
-#include "ui/ui_script.h"
-#include "ui/ui_search.h"
 #include "ui/search_dialog.h"
+#include "ui/ui_common.h"
+#include "ui/ui_prefs.h"
 #include "ui/ui_session.h"
 #include "ui/ui_tray.h"
 #include "ui/ui_update.h"
 
-/* all used icons (FIXME: evil) */
-GdkPixbuf *icons[MAX_ICONS];
-
-static void liferea_shell_class_init	(LifereaShellClass *klass);
-static void liferea_shell_init		(LifereaShell *ls);
-
 #define LIFEREA_SHELL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), LIFEREA_SHELL_TYPE, LifereaShellPrivate))
 
 struct LifereaShellPrivate {
-	GladeXML	*xml;
+	GtkBuilder	*xml;
 
 	GtkWindow	*window;		/**< Liferea main window */
 	GtkWidget	*menubar;
@@ -81,7 +70,8 @@ struct LifereaShellPrivate {
 	GtkActionGroup	*feedActions;		/**< update and mark read */
 	GtkActionGroup	*readWriteActions;	/**< node remove and properties, node itemset items remove */
 	GtkActionGroup	*itemActions;		/**< item state toggline, single item remove */
-	
+
+	ItemList	*itemlist;	
 	FeedList	*feedlist;
 	ItemView	*itemview;
 	BrowserTabs	*tabs;
@@ -90,33 +80,7 @@ struct LifereaShellPrivate {
 static GObjectClass *parent_class = NULL;
 static LifereaShell *shell = NULL;
 
-GType
-liferea_shell_get_type (void) 
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) 
-	{
-		static const GTypeInfo our_info = 
-		{
-			sizeof (LifereaShellClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) liferea_shell_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (LifereaShell),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) liferea_shell_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "LifereaShell",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (LifereaShell, liferea_shell, G_TYPE_OBJECT);
 
 static void
 liferea_shell_finalize (GObject *object)
@@ -146,19 +110,28 @@ liferea_shell_lookup (const gchar *name)
 	g_return_val_if_fail (shell != NULL, NULL);
 	g_return_val_if_fail (shell->priv != NULL, NULL);
 
-	return glade_xml_get_widget (shell->priv->xml, name);
+	return GTK_WIDGET (gtk_builder_get_object (shell->priv->xml, name));
 }
 
 static void
 liferea_shell_init (LifereaShell *ls)
 {
+	/* FIXME: we should only need to load mainwindow, but GtkBuilder won't
+	 * load the other required objects, so we need to do it manually...
+	 * See fixme in liferea_dialog_new()
+	 * http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=508585
+	 */
+	gchar *objs[] = { "adjustment6", "mainwindow", NULL };
 	/* globally accessible singleton */
 	g_assert (NULL == shell);
 	shell = ls;
 	
 	shell->priv = LIFEREA_SHELL_GET_PRIVATE (ls);
-	shell->priv->xml = glade_xml_new (PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "liferea.glade", "mainwindow", GETTEXT_PACKAGE);
-	glade_xml_signal_autoconnect (shell->priv->xml);
+	shell->priv->xml = gtk_builder_new ();
+	if (!gtk_builder_add_objects_from_file (shell->priv->xml, PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "liferea.ui", objs, NULL))
+		g_error ("Loading " PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S  "liferea.ui failed");
+
+	gtk_builder_connect_signals (shell->priv->xml, NULL);
 }
 
 /**
@@ -170,12 +143,13 @@ liferea_shell_restore_position (void)
 {
 	/* load window position */
 	int x, y, w, h;
+	gboolean last_window_maximized;
 
-	x = conf_get_int_value (LAST_WINDOW_X);
-	y = conf_get_int_value (LAST_WINDOW_Y);
+	conf_get_int_value (LAST_WINDOW_X, &x);
+	conf_get_int_value (LAST_WINDOW_Y, &y);
 
-	w = conf_get_int_value (LAST_WINDOW_WIDTH);
-	h = conf_get_int_value (LAST_WINDOW_HEIGHT);
+	conf_get_int_value (LAST_WINDOW_WIDTH, &w);
+	conf_get_int_value (LAST_WINDOW_HEIGHT, &h);
 	
 	debug4 (DEBUG_GUI, "Retrieved saved setting: size %dx%d position %d:%d", w, h, x, y);
 	
@@ -200,7 +174,9 @@ liferea_shell_restore_position (void)
 		gtk_window_resize (GTK_WINDOW (shell->priv->window), w, h);
 	}
 
-	if (conf_get_bool_value (LAST_WINDOW_MAXIMIZED))
+	conf_get_bool_value (LAST_WINDOW_MAXIMIZED, &last_window_maximized);
+
+	if (last_window_maximized)
 		gtk_window_maximize (GTK_WINDOW (shell->priv->window));
 	else
 		gtk_window_unmaximize (GTK_WINDOW (shell->priv->window));
@@ -212,6 +188,7 @@ liferea_shell_save_position (void)
 {
 	GtkWidget	*pane;
 	gint		x, y, w, h;
+	gboolean	last_window_maximized;
 
 	/* save pane proportions */
 	pane = liferea_shell_lookup ("leftpane");
@@ -233,10 +210,12 @@ liferea_shell_save_position (void)
 	}
 	
 	/* The following needs to be skipped when the window is not visible */
-	if (!GTK_WIDGET_VISIBLE (shell->priv->window))
+	if (!gtk_widget_get_visible (GTK_WIDGET (shell->priv->window)))
 		return;
 
-	if (conf_get_bool_value (LAST_WINDOW_MAXIMIZED))
+	conf_get_bool_value (LAST_WINDOW_MAXIMIZED, &last_window_maximized);
+
+	if (last_window_maximized)
 		return;
 
 	gtk_window_get_position (shell->priv->window, &x, &y);
@@ -263,11 +242,11 @@ liferea_shell_set_toolbar_style (const gchar *toolbar_style)
 {	
 	if (!toolbar_style) /* default to icons */
 		gtk_toolbar_set_style (GTK_TOOLBAR (shell->priv->toolbar), GTK_TOOLBAR_ICONS);
-	else if (!strcmp (toolbar_style, "text"))
+	else if (g_str_equal (toolbar_style, "text"))
 		gtk_toolbar_set_style (GTK_TOOLBAR (shell->priv->toolbar), GTK_TOOLBAR_TEXT);
-	else if (!strcmp (toolbar_style, "both"))
+	else if (g_str_equal (toolbar_style, "both"))
 		gtk_toolbar_set_style (GTK_TOOLBAR (shell->priv->toolbar), GTK_TOOLBAR_BOTH);
-	else if (!strcmp (toolbar_style, "both_horiz") || !strcmp (toolbar_style, "both-horiz") )
+	else if (g_str_equal (toolbar_style, "both_horiz") || g_str_equal (toolbar_style, "both-horiz") )
 		gtk_toolbar_set_style (GTK_TOOLBAR (shell->priv->toolbar), GTK_TOOLBAR_BOTH_HORIZ);
 	else /* default to icons */
 		gtk_toolbar_set_style (GTK_TOOLBAR (shell->priv->toolbar), GTK_TOOLBAR_ICONS);
@@ -276,7 +255,11 @@ liferea_shell_set_toolbar_style (const gchar *toolbar_style)
 void
 liferea_shell_update_toolbar (void)
 {
-	if (conf_get_bool_value (DISABLE_TOOLBAR))
+	gboolean disable_toolbar;
+
+	conf_get_bool_value (DISABLE_TOOLBAR, &disable_toolbar);
+
+	if (disable_toolbar)
 		gtk_widget_hide (shell->priv->toolbar);
 	else
 		gtk_widget_show (shell->priv->toolbar);
@@ -473,7 +456,7 @@ on_notebook_scroll_event_null_cb (GtkWidget *widget, GdkEventScroll *event)
 	GtkWidget* child;
 	GtkWidget* originator;
 
-	if (!notebook->cur_page)
+	if (!gtk_notebook_get_current_page (notebook))
 		return FALSE;
 
 	child = gtk_notebook_get_nth_page (notebook, gtk_notebook_get_current_page (notebook));
@@ -489,7 +472,11 @@ on_notebook_scroll_event_null_cb (GtkWidget *widget, GdkEventScroll *event)
 static gboolean
 on_close (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
-	if ((ui_tray_get_count() == 0) || (conf_get_bool_value (DONT_MINIMIZE_TO_TRAY))) {
+	gboolean dont_maximize_to_tray;
+
+	conf_get_bool_value (DONT_MINIMIZE_TO_TRAY, &dont_maximize_to_tray);
+
+	if ((ui_tray_get_count() == 0) || dont_maximize_to_tray) {
 		liferea_shutdown ();
 		return TRUE;
 	}
@@ -529,6 +516,7 @@ on_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	guint		default_modifiers;
 	const gchar	*type;
 	GtkWidget	*focusw;
+	gint		browse_key_setting;
 
 	if (event->type == GDK_KEY_PRESS) {
 		default_modifiers = gtk_accelerator_get_default_mod_mask ();
@@ -536,7 +524,8 @@ on_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		/* handle [<modifier>+]<Space> headline skimming hotkey */
 		switch (event->keyval) {
 			case GDK_space:
-				switch (conf_get_int_value (BROWSE_KEY_SETTING)) {
+				conf_get_int_value (BROWSE_KEY_SETTING, &browse_key_setting);
+				switch (browse_key_setting) {
 					default:
 					case 0:
 						modifier_matches = ((event->state & default_modifiers) == 0);
@@ -586,18 +575,17 @@ on_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 			return FALSE;
 
 		/* prevent usage of navigation keys in HTML view */
-		type = g_type_name (GTK_WIDGET_TYPE (focusw));
-		if (type && (!strcmp (type, "WebKitWebView")))
+		type = g_type_name (G_OBJECT_TYPE (focusw));
+		if (type && (g_str_equal (type, "WebKitWebView")))
 			return FALSE;
 		
 		/* check for treeview navigation */
 		if (0 == (event->state & default_modifiers)) {
 			switch (event->keyval) {
 				case GDK_KP_Delete:
-
 				case GDK_Delete:
 					if (focusw == GTK_WIDGET (shell->priv->feedlistView))
-						return FALSE;	/* to be handled in ui_feedlist_key_press_cb() */
+						return FALSE;	/* to be handled in feed_list_view_key_press_cb() */
 						
 					on_remove_item_activate (NULL, NULL);
 					return TRUE;
@@ -659,13 +647,13 @@ on_searchbtn_clicked (GtkButton *button, gpointer user_data)
 void
 on_onlinebtn_clicked (GtkButton *button, gpointer user_data)
 {
-	network_set_online (!network_is_online ());
+	network_monitor_set_online (!network_monitor_is_online ());
 }
 
 static void
 on_work_offline_activate (GtkToggleAction *menuitem, gpointer user_data)
 {
-	network_set_online (!gtk_toggle_action_get_active (menuitem));
+	network_monitor_set_online (!gtk_toggle_action_get_active (menuitem));
 }
 
 static void
@@ -679,28 +667,36 @@ on_about_activate (GtkMenuItem *menuitem, gpointer user_data)
 	gtk_widget_show (dialog);
 }
 
+static void
+liferea_shell_add_html_tab (const gchar *file, const gchar *name)
+{
+	gchar *filepattern = g_strdup_printf (PACKAGE_DATA_DIR "/" PACKAGE "/doc/html/%s", file);
+	gchar *filename = common_get_localized_filename (filepattern);
+	gchar *fileuri = g_strdup_printf ("file://%s", filename);
+
+	browser_tabs_add_new (fileuri, name, TRUE);
+
+	g_free (filepattern);
+	g_free (filename);
+	g_free (fileuri);
+}
+
 static void
 on_topics_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
-	gchar *filename = g_strdup_printf ("file://" PACKAGE_DATA_DIR "/" PACKAGE "/doc/html/%s", _("topics_en.html"));
-	browser_tabs_add_new (filename, _("Help Topics"), TRUE);
-	g_free (filename);
+	liferea_shell_add_html_tab ("topics_%s.html", _("Help Topics"));
 }
 
 static void
 on_quick_reference_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
-	gchar *filename = g_strdup_printf ("file://" PACKAGE_DATA_DIR "/" PACKAGE "/doc/html/%s", _("reference_en.html"));
-	browser_tabs_add_new (filename, _("Quick Reference"), TRUE);
-	g_free (filename);
+	liferea_shell_add_html_tab ("reference_%s.html", _("Quick Reference"));
 }
 
 static void
 on_faq_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
-	gchar *filename = g_strdup_printf ("file://" PACKAGE_DATA_DIR "/" PACKAGE "/doc/html/%s", _("faq_en.html"));
-	browser_tabs_add_new (filename, _("FAQ"), TRUE);
-	g_free (filename);
+	liferea_shell_add_html_tab ("faq_%s.html", _("FAQ"));
 }
 
 static void
@@ -733,37 +729,35 @@ on_menu_export (gpointer callback_data, guint callback_action, GtkWidget *widget
 	export_OPML_file ();
 }
 
-// FIXME: change to signal callback
-void
-liferea_shell_online_status_changed (int online)
+static void
+liferea_shell_set_online_icon (gboolean online)
 {
 	GtkWidget	*widget;
 
-	if (!shell)
-		return;	// FIXME: this check shouldn't be necessary
-
 	widget = liferea_shell_lookup ("onlineimage");
 
 	if (online) {
 		liferea_shell_set_status_bar (_("Liferea is now online"));
-		gtk_image_set_from_pixbuf (GTK_IMAGE (widget), icons[ICON_ONLINE]);
-		atk_object_set_name (gtk_widget_get_accessible (widget), _("Work Offline"));		
+		gtk_image_set_from_pixbuf (GTK_IMAGE (widget), (GdkPixbuf *)icon_get (ICON_ONLINE));
+		atk_object_set_name (gtk_widget_get_accessible (widget), _("Work Offline"));	
 	} else {
 		liferea_shell_set_status_bar (_("Liferea is now offline"));
-		gtk_image_set_from_pixbuf (GTK_IMAGE (widget), icons[ICON_OFFLINE]);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (widget), (GdkPixbuf *)icon_get (ICON_OFFLINE));
 		atk_object_set_name (gtk_widget_get_accessible (widget), _("Work Online"));	
 	}
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (shell->priv->generalActions, "ToggleOfflineMode")), !online);
+}
 
-	/* Propagate new online status to other interested widgets */
-	// FIXME: bad design, HTML view implementation needs to register for network events itself
-	liferea_htmlview_set_online (online);
+static void
+liferea_shell_online_status_changed (NetworkMonitor *nm, gboolean online, gpointer userdata)
+{
+	liferea_shell_set_online_icon (online);
 	ui_tray_update ();
 }
 
 /* methods to receive URLs which were dropped anywhere in the main window */
 static void
-liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time)
+liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time_received)
 {
 	gchar		*tmp1, *tmp2, *freeme;
 	GtkWidget	*mainwindow;
@@ -775,12 +769,15 @@ liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x,
 	nodePtr		node;
 	gint		tx, ty;
 	
-	g_return_if_fail (data->data != NULL);
+	g_return_if_fail (gtk_selection_data_get_data (data) != NULL);
 		
-	mainwindow = GTK_WIDGET (liferea_shell_lookup ("mainwindow"));
-	treeview = GTK_TREE_VIEW (liferea_shell_lookup ("feedlist"));
+	mainwindow = GTK_WIDGET (shell->priv->window);
+	treeview = GTK_TREE_VIEW (shell->priv->feedlistView);
 	model = gtk_tree_view_get_model (treeview);
 
+	/* x and y are relative to the main window, make them relative to the treeview */
+	gtk_widget_translate_coordinates (mainwindow, GTK_WIDGET (treeview), x, y, &tx, &ty);
+
 	/* Allow link drops only over feed list widget. This is to avoid
 	   the frequent accidental text drops in the HTML view. */
 
@@ -788,16 +785,13 @@ liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x,
 
 	if((x > alloc.x+alloc.width) || (x < alloc.x) ||
 	   (y > alloc.y+alloc.height) || (y < alloc.y)) {
-		gtk_drag_finish (context, FALSE, FALSE, time);
+		gtk_drag_finish (context, FALSE, FALSE, time_received);
 		return;
 	}		
 
-	/* x and y are relative to the main window, make them relative to the treeview */
-	g_assert (gtk_widget_translate_coordinates (mainwindow, GTK_WIDGET (treeview), x, y, &tx, &ty));
-
-	if ((data->length >= 0) && (data->format == 8)) {
-		/* extra handling to accept multiple drops */	
-		freeme = tmp1 = g_strdup (data->data);
+	if ((gtk_selection_data_get_length (data) >= 0) && (gtk_selection_data_get_format (data) == 8)) {
+		/* extra handling to accept multiple drops */
+		freeme = tmp1 = g_strdup (gtk_selection_data_get_data (data));
 		while ((tmp2 = strsep (&tmp1, "\n\r"))) {
 			if (strlen (tmp2)) {
 				/* if the drop is over a node, select it so that feedlist_add_subscription()
@@ -805,44 +799,46 @@ liferea_shell_URL_received (GtkWidget *widget, GdkDragContext *context, gint x,
 				if (gtk_tree_view_get_dest_row_at_pos (treeview, tx, ty, &path, NULL)) {
 					if (gtk_tree_model_get_iter (model, &iter, path)) {
 						gtk_tree_model_get (model, &iter, FS_PTR, &node, -1);
-						/* if node is NULL, ui_feedlist_select() will unselect the tv */
-						ui_feedlist_select (node);
+						/* if node is NULL, feed_list_view_select() will unselect the tv */
+						feed_list_view_select (node);
 					}
 					gtk_tree_path_free (path);
 				}
 				feedlist_add_subscription (g_strdup (tmp2), NULL, NULL,
-				                           FEED_REQ_RESET_TITLE |
 				                           FEED_REQ_PRIORITY_HIGH);
 			}
 		}
 		g_free (freeme);
-		gtk_drag_finish (context, TRUE, FALSE, time);		
+		gtk_drag_finish (context, TRUE, FALSE, time_received);
 	} else {
-		gtk_drag_finish (context, FALSE, FALSE, time);
+		gtk_drag_finish (context, FALSE, FALSE, time_received);
 	}
 }
 
-void
-liferea_shell_setup_URL_receiver (GtkWidget *widget)
+static void
+liferea_shell_setup_URL_receiver (void)
 {
+	GtkWidget *mainwindow;
 	GtkTargetEntry target_table[] = {
-		{ "STRING",     		0, 0 },
-		{ "text/plain", 		0, 0 },
-		{ "text/uri-list",		0, 1 },
-		{ "_NETSCAPE_URL",		0, 1 },
-		{ "application/x-rootwin-drop", 0, 2 }
+		{ "STRING",     		GTK_TARGET_OTHER_WIDGET, 0 },
+		{ "text/plain", 		GTK_TARGET_OTHER_WIDGET, 0 },
+		{ "text/uri-list",		GTK_TARGET_OTHER_WIDGET, 1 },
+		{ "_NETSCAPE_URL",		GTK_TARGET_OTHER_APP, 1 },
+		{ "application/x-rootwin-drop", GTK_TARGET_OTHER_APP, 2 }
 	};
 
+	mainwindow = GTK_WIDGET (shell->priv->window);
+
 	/* doesn't work with GTK_DEST_DEFAULT_DROP... */
-	gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_ALL,
+	gtk_drag_dest_set (mainwindow, GTK_DEST_DEFAULT_ALL,
 	                   target_table, sizeof (target_table)/sizeof (target_table[0]),
 	                   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 		       
-	g_signal_connect (G_OBJECT (widget), "drag_data_received",
+	g_signal_connect (G_OBJECT (mainwindow), "drag_data_received",
 	                  G_CALLBACK (liferea_shell_URL_received), NULL);
 }
 
-static const GtkActionEntry ui_mainwindow_action_entries[] = {
+static const GtkActionEntry liferea_shell_action_entries[] = {
 	{"SubscriptionsMenu", NULL, N_("_Subscriptions")},
 	{"UpdateAll", "gtk-refresh", N_("Update _All"), "<control>U", N_("Updates all subscriptions."),
 	 G_CALLBACK(on_menu_update_all)},
@@ -857,7 +853,9 @@ static const GtkActionEntry ui_mainwindow_action_entries[] = {
 	 G_CALLBACK(on_remove_items_activate)},
 
 	{"ItemMenu", NULL, N_("_Item")},
-	{"NextUnreadItem", GTK_STOCK_GO_FORWARD, N_("_Next Unread Item"), "<control>N", N_("Jumps to the next unread item. If necessary selects the next feed with unread items."),
+	/* No tooltip here as it really hinders usability to have it flashing
+	   when skimming through items using "Next Unread"! */
+	{"NextUnreadItem", GTK_STOCK_GO_FORWARD, N_("_Next Unread Item"), "<control>N", NULL,	
 	 G_CALLBACK(on_next_unread_item_activate)},
 
 	{"ViewMenu", NULL, N_("_View")},
@@ -869,14 +867,11 @@ static const GtkActionEntry ui_mainwindow_action_entries[] = {
 	{"ToolsMenu", NULL, N_("_Tools")},
 	{"ShowUpdateMonitor", NULL, N_("_Update Monitor"), NULL, N_("Show a list of all feeds currently in the update queue"),
 	 G_CALLBACK(on_menu_show_update_monitor)},
-	{"ShowScriptManager", NULL, N_("_Script Manager"), NULL, N_("Allows to configure and edit LUA hook scripts"),
-	 G_CALLBACK(on_menu_show_script_manager)},
 	{"ShowPreferences", GTK_STOCK_PREFERENCES, N_("_Preferences"), NULL, N_("Edit Preferences."),
 	 G_CALLBACK(on_prefbtn_clicked)},
 
 	{"SearchMenu", NULL, N_("S_earch")},
 	{"SearchFeeds", "gtk-find", N_("Search All Feeds..."), "<control>F", N_("Show the search dialog."), G_CALLBACK(on_searchbtn_clicked)},
-	{"CreateEngineSearch", NULL, N_("Search With ...")},
 
 	{"HelpMenu", NULL, N_("_Help")},
 	{"ShowHelpContents", "gtk-help", N_("_Contents"), "F1", N_("View help for this application."), G_CALLBACK(on_topics_activate)},
@@ -886,7 +881,7 @@ static const GtkActionEntry ui_mainwindow_action_entries[] = {
 	{"ShowAbout", "gtk-about", N_("_About"), NULL, N_("Shows an about dialog."), G_CALLBACK(on_about_activate)}
 };
 
-static const GtkRadioActionEntry ui_mainwindow_view_radio_entries[] = {
+static const GtkRadioActionEntry liferea_shell_view_radio_entries[] = {
 	{"NormalView", NULL, N_("_Normal View"), NULL, N_("Set view mode to mail client mode."),
 	 0},
 	{"WideView", NULL, N_("_Wide View"), NULL, N_("Set view mode to use three vertical panes."),
@@ -895,12 +890,12 @@ static const GtkRadioActionEntry ui_mainwindow_view_radio_entries[] = {
 	 2}
 };
 
-static GtkToggleActionEntry ui_mainwindow_feedlist_toggle_entries[] = {
+static GtkToggleActionEntry liferea_shell_feedlist_toggle_entries[] = {
 	{"ReducedFeedList", NULL, N_("_Reduced Feed List"), NULL, N_("Hide feeds with no unread items."),
 	G_CALLBACK(on_feedlist_reduced_activate)}
 };
 
-static const GtkActionEntry ui_mainwindow_add_action_entries[] = {
+static const GtkActionEntry liferea_shell_add_action_entries[] = {
 	{"NewSubscription", "gtk-add", N_("_New Subscription..."), NULL, N_("Adds a subscription to the feed list."),
 	 G_CALLBACK(on_menu_feed_new)},
 	{"NewFolder", NULL, N_("New _Folder..."), NULL, N_("Adds a folder to the feed list."), G_CALLBACK(on_menu_folder_new)},
@@ -909,19 +904,19 @@ static const GtkActionEntry ui_mainwindow_add_action_entries[] = {
 	{"NewNewsBin", NULL, N_("New _News Bin..."), NULL, N_("Adds a new news bin."), G_CALLBACK(on_new_newsbin_activate)}
 };
 
-static const GtkActionEntry ui_mainwindow_feed_action_entries[] = {
+static const GtkActionEntry liferea_shell_feed_action_entries[] = {
 	{"MarkFeedAsRead", "gtk-apply", N_("_Mark Items Read"), "<control>R", N_("Marks all items of the selected feed list node / in the item list as read."), 
 	 G_CALLBACK(on_menu_allread)},
 	{"UpdateSelected", "gtk-refresh", N_("_Update"), NULL, N_("Updates the selected subscription or all subscriptions of the selected folder."),
 	 G_CALLBACK(on_menu_update)}
 };
 
-static const GtkActionEntry ui_mainwindow_read_write_action_entries[] = {
+static const GtkActionEntry liferea_shell_read_write_action_entries[] = {
 	{"Properties", "gtk-properties", N_("_Properties"), NULL, N_("Opens the property dialog for the selected subscription."), G_CALLBACK(on_menu_properties)},
 	{"DeleteSelected", "gtk-delete", N_("_Remove"), NULL, N_("Removes the selected subscription."), G_CALLBACK(on_menu_delete)}
 };
 
-static const GtkActionEntry ui_mainwindow_item_action_entries[] = {
+static const GtkActionEntry liferea_shell_item_action_entries[] = {
 	{"ToggleItemReadStatus", "gtk-apply", N_("Toggle _Read Status"), "<control>M", N_("Toggles the read status of the selected item."),
 	 G_CALLBACK(on_toggle_unread_status)},
 	{"ToggleItemFlag", NULL, N_("Toggle Item _Flag"), "<control>T", N_("Toggles the flag status of the selected item."),
@@ -932,12 +927,12 @@ static const GtkActionEntry ui_mainwindow_item_action_entries[] = {
 	 G_CALLBACK(on_popup_launchitem_selected)}
 };
 
-static const GtkToggleActionEntry ui_mainwindow_action_toggle_entries[] = {
+static const GtkToggleActionEntry liferea_shell_action_toggle_entries[] = {
 	{"ToggleOfflineMode", NULL, N_("_Work Offline"), NULL, N_("This option allows you to disable subscription updating."),
 	 G_CALLBACK(on_work_offline_activate)}
 };
 
-static const char *ui_mainwindow_ui_desc =
+static const char *liferea_shell_ui_desc =
 "<ui>"
 "  <menubar name='MainwindowMenubar'>"
 "    <menu action='SubscriptionsMenu'>"
@@ -987,12 +982,10 @@ static const char *ui_mainwindow_ui_desc =
 "    </menu>"
 "    <menu action='ToolsMenu'>"
 "      <menuitem action='ShowUpdateMonitor'/>"
-"      <menuitem action='ShowScriptManager'/>"
 "      <menuitem action='ShowPreferences'/>"
 "    </menu>"
 "    <menu action='SearchMenu'>"
 "      <menuitem action='SearchFeeds'/>"
-"      <menu action='CreateEngineSearch'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='ShowHelpContents'/>"
@@ -1013,28 +1006,11 @@ static const char *ui_mainwindow_ui_desc =
 "  </toolbar>"
 "</ui>";
 
-static GdkPixbuf *
-liferea_shell_get_theme_icon (GtkIconTheme *icon_theme, const gchar *name, gint size)
-{
-	GError *error = NULL;
-	GdkPixbuf *pixbuf;
-
-	pixbuf = gtk_icon_theme_load_icon (icon_theme,
-	                                   name, /* icon name */
-	                                   size, /* size */
-	                                   0,  /* flags */
-	                                   &error);
-	if (!pixbuf) {
-		debug1(DEBUG_GUI, "Couldn't load icon: %s", error->message);
-		g_error_free (error);
-	}
-	return pixbuf;
-}
-
 static void
 liferea_shell_restore_state (void)
 {
 	gchar *toolbar_style, *accels_file;
+	gint last_vpane_pos, last_hpane_pos, last_wpane_pos;
 	
 	debug0 (DEBUG_GUI, "Setting toolbar style");
 	
@@ -1054,12 +1030,15 @@ liferea_shell_restore_state (void)
 
 	debug0 (DEBUG_GUI, "Loading pane proportions");
 		
-	if (0 != conf_get_int_value (LAST_VPANE_POS))
-		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("leftpane")), conf_get_int_value (LAST_VPANE_POS));
-	if (0 != conf_get_int_value (LAST_HPANE_POS))
-		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("normalViewPane")), conf_get_int_value (LAST_HPANE_POS));
-	if (0 != conf_get_int_value (LAST_WPANE_POS))
-		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("wideViewPane")), conf_get_int_value (LAST_WPANE_POS));
+	conf_get_int_value (LAST_VPANE_POS, &last_vpane_pos);
+	if (last_vpane_pos)
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("leftpane")), last_vpane_pos);
+	conf_get_int_value (LAST_HPANE_POS, &last_hpane_pos);
+	if (last_hpane_pos)
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("normalViewPane")), last_hpane_pos);
+	conf_get_int_value (LAST_WPANE_POS, &last_wpane_pos);
+	if (last_wpane_pos)
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("wideViewPane")), last_wpane_pos);
 }
 
 void
@@ -1068,10 +1047,8 @@ liferea_shell_create (int initialState)
 	GtkUIManager	*ui_manager;
 	GtkAccelGroup	*accel_group;
 	GError		*error = NULL;	
-	GtkWidget	*widget;
-	int		i;
 	GString		*buffer;
-	GtkIconTheme	*icon_theme;
+	gboolean	show_tray_icon, toggle;
 	
 	debug_enter ("liferea_shell_create");
 
@@ -1080,45 +1057,45 @@ liferea_shell_create (int initialState)
 	g_object_new (LIFEREA_SHELL_TYPE, NULL);
 
 	shell->priv->window = GTK_WINDOW (liferea_shell_lookup ("mainwindow"));
-
-	// FIXME: do we use this anywhere?
-	gtk_widget_set_name (liferea_shell_lookup ("feedlist"), "feedlist");
 	
 	/* 2.) menu creation */
 	
 	debug0 (DEBUG_GUI, "Setting up menues");
 
+	shell->priv->itemlist = itemlist_create ();
+
 	/* Prepare some toggle button states */	
-	ui_mainwindow_feedlist_toggle_entries[0].is_active = conf_get_bool_value (REDUCED_FEEDLIST);
+	conf_get_bool_value (REDUCED_FEEDLIST, &toggle);
+	liferea_shell_feedlist_toggle_entries[0].is_active = toggle;
 
 	ui_manager = gtk_ui_manager_new ();
 
 	shell->priv->generalActions = gtk_action_group_new ("GeneralActions");
 	gtk_action_group_set_translation_domain (shell->priv->generalActions, PACKAGE);
-	gtk_action_group_add_actions (shell->priv->generalActions, ui_mainwindow_action_entries, G_N_ELEMENTS (ui_mainwindow_action_entries), shell->priv);
-	gtk_action_group_add_toggle_actions (shell->priv->generalActions, ui_mainwindow_action_toggle_entries, G_N_ELEMENTS (ui_mainwindow_action_toggle_entries), shell->priv);
-	gtk_action_group_add_radio_actions (shell->priv->generalActions, ui_mainwindow_view_radio_entries, G_N_ELEMENTS (ui_mainwindow_view_radio_entries), itemlist_get_view_mode (), (GCallback)on_view_activate, (gpointer)TRUE);
-	gtk_action_group_add_toggle_actions (shell->priv->generalActions, ui_mainwindow_feedlist_toggle_entries, G_N_ELEMENTS (ui_mainwindow_feedlist_toggle_entries), shell->priv);
+	gtk_action_group_add_actions (shell->priv->generalActions, liferea_shell_action_entries, G_N_ELEMENTS (liferea_shell_action_entries), shell->priv);
+	gtk_action_group_add_toggle_actions (shell->priv->generalActions, liferea_shell_action_toggle_entries, G_N_ELEMENTS (liferea_shell_action_toggle_entries), shell->priv);
+	gtk_action_group_add_radio_actions (shell->priv->generalActions, liferea_shell_view_radio_entries, G_N_ELEMENTS (liferea_shell_view_radio_entries), itemlist_get_view_mode (), (GCallback)on_view_activate, (gpointer)TRUE);
+	gtk_action_group_add_toggle_actions (shell->priv->generalActions, liferea_shell_feedlist_toggle_entries, G_N_ELEMENTS (liferea_shell_feedlist_toggle_entries), shell->priv);
 	gtk_ui_manager_insert_action_group (ui_manager, shell->priv->generalActions, 0);
 
 	shell->priv->addActions = gtk_action_group_new ("AddActions");
 	gtk_action_group_set_translation_domain (shell->priv->addActions, PACKAGE);
-	gtk_action_group_add_actions (shell->priv->addActions, ui_mainwindow_add_action_entries, G_N_ELEMENTS (ui_mainwindow_add_action_entries), shell->priv);
+	gtk_action_group_add_actions (shell->priv->addActions, liferea_shell_add_action_entries, G_N_ELEMENTS (liferea_shell_add_action_entries), shell->priv);
 	gtk_ui_manager_insert_action_group (ui_manager, shell->priv->addActions, 0);
 
 	shell->priv->feedActions = gtk_action_group_new ("FeedActions");
 	gtk_action_group_set_translation_domain (shell->priv->feedActions, PACKAGE);
-	gtk_action_group_add_actions (shell->priv->feedActions, ui_mainwindow_feed_action_entries, G_N_ELEMENTS (ui_mainwindow_feed_action_entries), shell->priv);
+	gtk_action_group_add_actions (shell->priv->feedActions, liferea_shell_feed_action_entries, G_N_ELEMENTS (liferea_shell_feed_action_entries), shell->priv);
 	gtk_ui_manager_insert_action_group (ui_manager, shell->priv->feedActions, 0);
 
 	shell->priv->readWriteActions = gtk_action_group_new("ReadWriteActions");
 	gtk_action_group_set_translation_domain (shell->priv->readWriteActions, PACKAGE);
-	gtk_action_group_add_actions (shell->priv->readWriteActions, ui_mainwindow_read_write_action_entries, G_N_ELEMENTS (ui_mainwindow_read_write_action_entries), shell->priv);
+	gtk_action_group_add_actions (shell->priv->readWriteActions, liferea_shell_read_write_action_entries, G_N_ELEMENTS (liferea_shell_read_write_action_entries), shell->priv);
 	gtk_ui_manager_insert_action_group (ui_manager, shell->priv->readWriteActions, 0);
 
 	shell->priv->itemActions = gtk_action_group_new ("ItemActions");
 	gtk_action_group_set_translation_domain (shell->priv->itemActions, PACKAGE);
-	gtk_action_group_add_actions (shell->priv->itemActions, ui_mainwindow_item_action_entries, G_N_ELEMENTS (ui_mainwindow_item_action_entries), shell->priv);
+	gtk_action_group_add_actions (shell->priv->itemActions, liferea_shell_item_action_entries, G_N_ELEMENTS (liferea_shell_item_action_entries), shell->priv);
 	gtk_ui_manager_insert_action_group (ui_manager, shell->priv->itemActions, 0);
 
 	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
@@ -1127,11 +1104,9 @@ liferea_shell_create (int initialState)
 
 	g_signal_connect (gtk_accel_map_get (), "changed", G_CALLBACK (on_accel_change), NULL);
 
-	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_mainwindow_ui_desc, -1, &error))
+	if (!gtk_ui_manager_add_ui_from_string (ui_manager, liferea_shell_ui_desc, -1, &error))
 		g_error ("building menus failed: %s", error->message);
 
-	ui_search_engines_setup_menu (ui_manager);
-
 	shell->priv->menubar = gtk_ui_manager_get_widget (ui_manager, "/MainwindowMenubar");
 	shell->priv->toolbar = gtk_ui_manager_get_widget (ui_manager, "/maintoolbar");
 
@@ -1186,7 +1161,7 @@ liferea_shell_create (int initialState)
 
 	debug0 (DEBUG_GUI, "Setting up feed list");
 	shell->priv->feedlistView = GTK_TREE_VIEW (liferea_shell_lookup ("feedlist"));
-	ui_feedlist_init (shell->priv->feedlistView);
+	feed_list_view_init (shell->priv->feedlistView);
 
 	/* 6.) setup menu sensivity */
 	
@@ -1203,80 +1178,25 @@ liferea_shell_create (int initialState)
 	
 	debug0 (DEBUG_GUI, "Setting up item view");
 
-	shell->priv->itemview = itemview_create (ui_itemlist_new (GTK_WIDGET (shell->priv->window)));
+	shell->priv->itemview = itemview_create (GTK_WIDGET (shell->priv->window));
 	
 	/* 8.) load icons as required */
 	
 	debug0 (DEBUG_GUI, "Loading icons");
 	
-	ui_common_add_pixmap_directory (PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "pixmaps");
+	icons_load ();
 	
-	/* first try to load icons from theme */
-	static const gchar *iconThemeNames[] = {
-		NULL,			/* ICON_READ */
-		NULL,			/* ICON_UNREAD */
-		"emblem-important",	/* ICON_FLAG */
-		NULL,			/* ICON_AVAILABLE */
-		NULL,			/* ICON_AVAILABLE_OFFLINE */
-		NULL,			/* ICON_UNAVAILABLE */
-		NULL,			/* ICON_DEFAULT */
-		NULL,			/* ICON_FOLDER_EMPTY */
-		"folder",		/* ICON_FOLDER */
-		"folder-saved-search",	/* ICON_VFOLDER */
-		NULL,			/* ICON_NEWSBIN */
-		NULL,			/* ICON_EMPTY */
-		NULL,			/* ICON_EMPTY_OFFLINE */
-		"gtk-connect",		/* ICON_ONLINE */
-		"gtk-disconnect",	/* ICON_OFFLINE */
-		NULL,			/* ICON_UPDATED */
-		"mail-attachment",	/* ICON_ENCLOSURE */
-		NULL
-	};
-
-	icon_theme = gtk_icon_theme_get_default ();
-	for (i = 0; i < MAX_ICONS; i++)
-		if (iconThemeNames[i])
-			icons[i] = liferea_shell_get_theme_icon (icon_theme, iconThemeNames[i], 16);
-
-	/* and then load own default icons */
-	static const gchar *iconNames[] = {
-		"read.xpm",		/* ICON_READ */
-		"unread.png",		/* ICON_UNREAD */
-		"flag.png",		/* ICON_FLAG */
-		"available.png",	/* ICON_AVAILABLE */
-		"available_offline.png",	/* ICON_AVAILABLE_OFFLINE */
-		NULL,			/* ICON_UNAVAILABLE */
-		"default.png",		/* ICON_DEFAULT */
-		"folder_empty.png",	/* ICON_FOLDER_EMPTY */
-		"directory.png",	/* ICON_FOLDER */
-		"vfolder.png",		/* ICON_VFOLDER */
-		"newsbin.png",		/* ICON_NEWSBIN */
-		"empty.png",		/* ICON_EMPTY */
-		"empty_offline.png",	/* ICON_EMPTY_OFFLINE */
-		"online.png",		/* ICON_ONLINE */
-		"offline.png",		/* ICON_OFFLINE */
-		"edit.png",		/* ICON_UPDATED */
-		"attachment.png",	/* ICON_ENCLOSURE */
-		NULL
-	};
-
-	for (i = 0; i < MAX_ICONS; i++)
-		if (!icons[i])
-			icons[i] = ui_common_create_pixbuf (iconNames[i]);
-
-	/* set up icons that are build from stock */
-	widget = gtk_button_new ();
-	icons[ICON_UNAVAILABLE] = gtk_widget_render_icon (widget, GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_MENU, "");
-	gtk_widget_destroy (widget);
+	/* 9.) update all menu elements */
 	
 	liferea_shell_update_toolbar ();
-	liferea_shell_online_status_changed (network_is_online ());
 	
-	liferea_shell_setup_URL_receiver (GTK_WIDGET (shell->priv->window));	/* setup URL dropping support */
+	liferea_shell_setup_URL_receiver ();	/* setup URL dropping support */
 
 	shell->priv->feedlist = feedlist_create ();
-	
-	ui_tray_enable (conf_get_bool_value (SHOW_TRAY_ICON));		/* init tray icon */
+
+	conf_get_bool_value (SHOW_TRAY_ICON, &show_tray_icon);
+
+	ui_tray_enable (show_tray_icon);		/* init tray icon */
 
 	liferea_shell_restore_state ();
 	
@@ -1290,9 +1210,9 @@ liferea_shell_create (int initialState)
 		/* Needed so that the window structure can be
 		   accessed... otherwise will GTK warning when window is
 		   shown by clicking on notification icon. */
-		// gtk_widget_realize (GTK_WIDGET (shell->priv->window));
-		// Does not work with gtkmozembed...
-		
+		   
+		/* Note: gtk_widget_realize () did not work with GtkMozEmbed
+		   therefore we use show+hide instead. */
 		gtk_widget_show (GTK_WIDGET (shell->priv->window));
 		gtk_widget_hide (GTK_WIDGET (shell->priv->window));
 	}
@@ -1304,7 +1224,7 @@ liferea_shell_create (int initialState)
 	/* force two pane mode */
 	/*   For some reason, this causes the first item to be selected and then
 	     unselected... strange. */
-	ui_feedlist_select (NULL);
+	feed_list_view_select (NULL);
 	
 	itemview_set_layout (NODE_VIEW_MODE_COMBINED);
 	
@@ -1322,32 +1242,44 @@ liferea_shell_create (int initialState)
 	g_string_append (buffer,   "</h3></td></tr><tr><td colspan=\"2\">");
 	g_string_append (buffer,   _("<p>Welcome to <b>Liferea</b>, a desktop news aggregator for online news "
 				   "feeds.</p>"
-				   "<p>The left pane contains the list of your subscriptions. To add a "
-				   "subscription select Feeds -&gt; New Subscription. To browse the headlines "
-				   "of a feed select it in the feed list and the headlines will be loaded "
-				   "into the right pane.</p>"));
+				   "<p>You can add new subscriptions "
+	                           "<ul>"
+	                           "<li>From main menu 'Subscription' -&gt; 'New Subscription'</li>"
+	                           "<li>By dropping feed links into the subscription list</li>"
+	                           "<li>By right clicking links and choosing 'Subscribe' within Liferea</li>"
+	                           "</ul>"
+				   "</p>"));
 	g_string_append (buffer,   "</td></tr></table>");
-
+	
 	g_string_append (buffer,   "</div>");
 
 	htmlview_finish_output (buffer);
 	itemview_display_info (buffer->str);
 	g_string_free (buffer, TRUE);
 	
+	/* 10. Connect network monitoring and set icon*/
+	
+	g_signal_connect (network_monitor_get (), "online-status-changed",
+	                  G_CALLBACK (liferea_shell_online_status_changed), shell);
+
+	liferea_shell_set_online_icon (network_monitor_is_online ());
+
 	debug_exit ("liferea_shell_create");
 }
 
 void
 liferea_shell_destroy (void)
 {
-	ui_feedlist_select (NULL);
+	feed_list_view_select (NULL);
 	liferea_shell_save_position ();
 	ui_tray_enable (FALSE);
-	notification_enable (FALSE);
 	g_object_unref (shell->priv->tabs);
 	g_object_unref (shell->priv->feedlist);
+	g_object_unref (shell->priv->itemview);
 
 	gtk_widget_destroy (GTK_WIDGET (shell->priv->window));
+
+	g_object_unref (shell);
 }
 
 void
@@ -1355,7 +1287,7 @@ liferea_shell_present (void)
 {
 	GtkWidget *mainwindow = GTK_WIDGET (shell->priv->window);
 	
-	if ((gdk_window_get_state (mainwindow->window) & GDK_WINDOW_STATE_ICONIFIED) || !GTK_WIDGET_VISIBLE (mainwindow))
+	if ((gdk_window_get_state (gtk_widget_get_window (mainwindow)) & GDK_WINDOW_STATE_ICONIFIED) || !gtk_widget_get_visible (mainwindow))
 		liferea_shell_restore_position ();
 
 	gtk_window_present (shell->priv->window);
@@ -1366,8 +1298,22 @@ liferea_shell_toggle_visibility (void)
 {
 	GtkWidget *mainwindow = GTK_WIDGET (shell->priv->window);
 	
-	if ((gdk_window_get_state (mainwindow->window) & GDK_WINDOW_STATE_ICONIFIED) ||
-	    !GTK_WIDGET_VISIBLE (mainwindow)) {
+	if (gdk_window_get_state (gtk_widget_get_window (mainwindow)) & GDK_WINDOW_STATE_ICONIFIED) {
+		/* The window is either iconified, or on another workspace */
+		/* Raise it in one click */
+		if (gtk_widget_get_visible (mainwindow)) {
+			liferea_shell_save_position ();
+			gtk_widget_hide (mainwindow);
+		}
+		liferea_shell_restore_position ();
+		/* Note: Without deiconify() desktop moving doesn't work in 
+		   GNOME+metacity. The window would be moved correctly by 
+		   present() but not become visible. */
+		gtk_window_deiconify (GTK_WINDOW (mainwindow));
+		gtk_window_present (GTK_WINDOW (mainwindow));
+	}
+	else if (!gtk_widget_get_visible (mainwindow)) {
+		/* The window is neither iconified nor on another workspace, but is not visible */
 		liferea_shell_restore_position ();
 		gtk_window_present (shell->priv->window);
 	} else {
@@ -1382,7 +1328,8 @@ liferea_shell_get_window (void)
 	return GTK_WIDGET (shell->priv->window);
 }
 
-void liferea_shell_set_view_mode (nodeViewType newMode)
+void
+liferea_shell_set_view_mode (nodeViewType newMode)
 {
 	GtkRadioAction       *action;
 
