@@ -1,7 +1,7 @@
 /**
  * @file vfolder.c  search folder node type
  *
- * Copyright (C) 2003-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2011 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,17 +18,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "vfolder.h"
+
 #include "common.h"
-#include "conf.h"
 #include "db.h"
 #include "debug.h"
 #include "feedlist.h"
 #include "itemset.h"
 #include "itemlist.h"
 #include "node.h"
-#include "vfolder.h"
-#include "ui/ui_common.h"
-#include "ui/ui_feedlist.h"
+#include "rule.h"
+#include "vfolder_loader.h"
+#include "ui/icons.h"
 #include "ui/ui_node.h"
 #include "ui/search_folder_dialog.h"
 
@@ -43,8 +44,11 @@ vfolder_new (nodePtr node)
 	debug_enter ("vfolder_new");
 
 	vfolder = g_new0 (struct vfolder, 1);
+	vfolder->itemset = g_new0 (struct itemSet, 1);
+	vfolder->itemset->nodeId = node->id;
+	vfolder->itemset->ids = NULL;
+	vfolder->itemset->anyMatch = TRUE;
 	vfolder->node = node;
-	vfolder->anyMatch = TRUE;
 	vfolders = g_slist_append (vfolders, vfolder);
 
 	if (!node->title)
@@ -64,11 +68,11 @@ vfolder_import_rules (xmlNodePtr cur,
 	
 	matchType = xmlGetProp (cur, BAD_CAST"matchType");
 	if (matchType) {
-		/* currently we now only OR or AND'ing the rules,
+		/* currently we only OR or AND the rules,
 		   "any" is the value for OR'ing, "all" for AND'ing */
-		vfolder->anyMatch = (0 != xmlStrcmp (matchType, BAD_CAST"all"));
+		vfolder->itemset->anyMatch = (0 != xmlStrcmp (matchType, BAD_CAST"all"));
 	} else {
-		vfolder->anyMatch = TRUE;
+		vfolder->itemset->anyMatch = TRUE;
 	}
 	xmlFree (matchType);
 
@@ -87,9 +91,9 @@ vfolder_import_rules (xmlNodePtr cur,
 					debug2 (DEBUG_CACHE, "loading rule \"%s\" \"%s\"", ruleId, value);
 
 					if (additive && !xmlStrcmp (additive, BAD_CAST"true"))
-						vfolder_add_rule (vfolder, ruleId, value, TRUE);
+						itemset_add_rule (vfolder->itemset, ruleId, value, TRUE);
 					else
-						vfolder_add_rule (vfolder, ruleId, value, FALSE);
+						itemset_add_rule (vfolder->itemset, ruleId, value, FALSE);
 				} else {
 					g_warning ("ignoring invalid rule entry for vfolder \"%s\"...\n", node_get_title (vfolder->node));
 				}
@@ -106,82 +110,83 @@ vfolder_import_rules (xmlNodePtr cur,
 
 static itemSetPtr
 vfolder_load (nodePtr node) 
-{
-	return db_view_load (node->id);
-}
-
-void
-vfolder_update_counters (nodePtr node) 
 {
 	vfolderPtr vfolder = (vfolderPtr)node->data;
-	
-	/* Avoid throwing a "view unread count failed (error code 101, not an error)"
-	   warning in the database when there are no rules yet. */
-	if (vfolder->rules) {
-		node->unreadCount = db_view_get_unread_count (node->id);
-		node->itemCount = db_view_get_item_count (node->id);
-	} else {
-		node->unreadCount = 0;
-		node->itemCount = 0;
-	}
-	ui_node_update (node->id);
-}
 
-void
-vfolder_refresh (vfolderPtr vfolder)
-{
-	g_return_if_fail (NULL != vfolder->node);
-
-	if (0 != g_slist_length (vfolder->rules))
-		rules_to_view (vfolder->rules, vfolder->anyMatch, vfolder->node->id);
-	
-	vfolder_update_counters (vfolder->node);
-}
-
-static gboolean
-vfolder_is_affected (vfolderPtr vfolder, const gchar *ruleName)
-{
-	GSList *iter = vfolder->rules;
-	while (iter) {
-		rulePtr rule = (rulePtr)iter->data;
-		if (g_str_equal (rule->ruleInfo->ruleId, ruleName))
-			return TRUE;
-		iter = g_slist_next(iter);
-	}
-	return FALSE;
+	return vfolder->itemset;
 }
 
 void
 vfolder_foreach (nodeActionFunc func)
-{
-	vfolder_foreach_with_rule (NULL, func);
-}
-
-void
-vfolder_foreach_with_rule (const gchar *ruleName, nodeActionFunc func) 
 {
 	GSList	*iter = vfolders;
 	
 	g_assert (NULL != func);
 	while (iter) {
 		vfolderPtr vfolder = (vfolderPtr)iter->data;
-		if (!ruleName || vfolder_is_affected (vfolder, ruleName))
-			(*func) (vfolder->node);
+		(*func)(vfolder->node);
+		iter = g_slist_next (iter);		
+	}	
+}
+
+void
+vfolder_foreach_data (vfolderActionDataFunc func, itemPtr item)
+{
+	GSList	*iter = vfolders;
+	
+	g_assert (NULL != func);
+	while (iter) {
+		vfolderPtr vfolder = (vfolderPtr)iter->data;
+		(*func) (vfolder, item);
 		iter = g_slist_next (iter);
 	}
 }
 
 void
-vfolder_foreach_with_item (gulong itemId, nodeActionFunc func)
+vfolder_remove_item (vfolderPtr vfolder, itemPtr item)
 {
+	if (!vfolder->itemset->ids)
+		return;
+		
+	vfolder->itemset->ids = g_list_remove (vfolder->itemset->ids, GUINT_TO_POINTER (item->id));
+	vfolder->node->needsUpdate = TRUE;
+}
+
+void
+vfolder_add_item (vfolderPtr vfolder, itemPtr item)
+{
+	vfolder->itemset->ids = g_list_append (vfolder->itemset->ids, GUINT_TO_POINTER (item->id));
+	vfolder->node->needsUpdate = TRUE;
+}
+
+void
+vfolder_merge_item (vfolderPtr vfolder, itemPtr item)
+{
+	gboolean found = itemset_has_item_id (vfolder->itemset, item->id);
+
+	if (itemset_check_item (vfolder->itemset, item)) {
+		if (!found)
+			vfolder_add_item (vfolder, item);
+	} else {
+		if (found)
+			vfolder_remove_item (vfolder, item);
+	}
+}
+
+GSList *
+vfolder_get_all_with_item_id (gulong id)
+{
+	GSList	*result = NULL;
 	GSList	*iter = vfolders;
 	
 	while (iter) {
 		vfolderPtr vfolder = (vfolderPtr)iter->data;
-		if (db_view_contains_item (vfolder->node->id, itemId))
-			(*func) (vfolder->node);	
+		if (g_list_find (vfolder->itemset->ids, GUINT_TO_POINTER (id)))
+			result = g_slist_append (result, vfolder);
 		iter = g_slist_next (iter);
 	}
+
+	return result;
 }
 
 static void
@@ -195,8 +200,9 @@ vfolder_import (nodePtr node,
 	debug1 (DEBUG_CACHE, "import vfolder: title=%s", node_get_title (node));
 
 	vfolder = vfolder_new (node);
+	vfolder->itemset = db_search_folder_load (node->id);
+	
 	vfolder_import_rules (cur, vfolder);
-	vfolder_refresh (vfolder);
 }
 
 static void
@@ -213,9 +219,9 @@ vfolder_export (nodePtr node,
 	
 	g_assert (TRUE == trusted);
 	
-	xmlNewProp (cur, BAD_CAST"matchType", BAD_CAST (vfolder->anyMatch?"any":"all"));
+	xmlNewProp (cur, BAD_CAST"matchType", BAD_CAST (vfolder->itemset->anyMatch?"any":"all"));
 
-	iter = vfolder->rules;
+	iter = vfolder->itemset->rules;
 	while (iter) {
 		rule = iter->data;
 		ruleNode = xmlNewChild (cur, NULL, BAD_CAST"outline", NULL);
@@ -237,51 +243,34 @@ vfolder_export (nodePtr node,
 }
 
 void
-vfolder_add_existing_rule (vfolderPtr vfolder, rulePtr rule)
+vfolder_reset (vfolderPtr vfolder)
 {
-	vfolder->rules = g_slist_append (vfolder->rules, rule);
+	itemlist_unload (FALSE);
+
+	g_list_free (vfolder->itemset->ids);
+	vfolder->itemset->ids = NULL;
+	db_search_folder_reset (vfolder->node->id);
 }
 
 void
-vfolder_add_rule (vfolderPtr vfolder,
-                  const gchar *ruleId,
-                  const gchar *value,
-                  gboolean additive)
+vfolder_rebuild (nodePtr node)
 {
-	rulePtr		rule;
-	
-	rule = rule_new (vfolder, ruleId, value, additive);
-	if (rule)
-		vfolder_add_existing_rule (vfolder, rule);
-	else
-		g_warning ("unknown search folder rule id: \"%s\"", ruleId);
-}
+	vfolderPtr	vfolder = (vfolderPtr)node->data;
 
-void
-vfolder_remove_rule (vfolderPtr vfolder, rulePtr rule) 
-{
-	vfolder->rules = g_slist_remove (vfolder->rules, rule);
+	vfolder_reset (vfolder);
+	itemlist_add_loader (vfolder_loader_new (node));
 }
 
 static void
 vfolder_free (nodePtr node) 
 {
 	vfolderPtr	vfolder = (vfolderPtr) node->data;
-	GSList		*rule;
 
 	debug_enter ("vfolder_free");
 	
 	vfolders = g_slist_remove (vfolders, vfolder);
-	
-	/* free vfolder rules */
-	rule = vfolder->rules;
-	while (rule) {
-		rule_free (rule->data);
-		rule = g_slist_next (rule);
-	}
-	g_slist_free (vfolder->rules);
-	vfolder->rules = NULL;
-	
+	itemset_free (vfolder->itemset);
+		
 	debug_exit ("vfolder_free");
 }
 
@@ -290,15 +279,22 @@ vfolder_free (nodePtr node)
 static void vfolder_save (nodePtr node) { }
 
 static void
-vfolder_update_unread_count (nodePtr node) 
+vfolder_update_counters (nodePtr node) 
 {
-	g_warning("Should never be called!");
+	vfolderPtr vfolder = (vfolderPtr) node->data;
+	
+	/* There is no unread handling for search folders
+	   for performance reasons. So set everything to 0 
+	   here and don't bother with GUI updates... */
+	vfolder->node->needsUpdate = TRUE;
+	vfolder->node->unreadCount = 0;
+	vfolder->node->itemCount = g_list_length (vfolder->itemset->ids);
 }
 
 static void
 vfolder_remove (nodePtr node) 
 {
-	db_view_remove (node->id);
+	vfolder_reset (node->data);
 }
 
 static void
@@ -324,7 +320,6 @@ vfolder_get_node_type (void)
 { 
 	static struct nodeType nti = {
 		NODE_CAPABILITY_SHOW_ITEM_FAVICONS |
-		NODE_CAPABILITY_SHOW_UNREAD_COUNT |
 		NODE_CAPABILITY_SHOW_ITEM_COUNT,
 		"vfolder",
 		NULL,
@@ -332,14 +327,14 @@ vfolder_get_node_type (void)
 		vfolder_export,
 		vfolder_load,
 		vfolder_save,
-		vfolder_update_unread_count,
+		vfolder_update_counters,
 		vfolder_remove,
 		node_default_render,
 		vfolder_add,
 		vfolder_properties,
 		vfolder_free
 	};
-	nti.icon = icons[ICON_VFOLDER];
+	nti.icon = icon_get (ICON_VFOLDER);
 
 	return &nti; 
 }
