@@ -1,8 +1,9 @@
 /**
  * @file webkit.c  WebKit browser module for Liferea
  *
- * Copyright (C) 2007-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2010 Lars Lindner <lars.lindner@gmail.com>
  * Copyright (C) 2008 Lars Strojny <lars@strojny.net>
+ * Copyright (C) 2009 Emilio Pozuelo Monfort <pochu27@gmail.com>
  * Copyright (C) 2009 Adrian Bunk <bunk@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +21,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <gconf/gconf-client.h>
 #include <libsoup/soup.h>
 #include <webkit/webkit.h>
 #include <string.h>
@@ -32,44 +34,155 @@
 
 static WebKitWebSettings *settings = NULL;
 
+/**
+ * Update the settings object if the preferences change.
+ * This will affect all the webviews as they all use the same
+ * settings object.
+ */
+static void
+liferea_webkit_disable_javascript_cb (GConfClient *client,
+				      guint cnxn_id,
+				      GConfEntry *entry,
+				      gpointer user_data)
+{
+	GConfValue *disable_javascript;
+
+	g_return_if_fail (entry != NULL);
+
+	disable_javascript = gconf_entry_get_value (entry);
+	if (!disable_javascript || disable_javascript->type != GCONF_VALUE_BOOL)
+		return;
+
+	g_object_set (
+		settings,
+		"enable-scripts",
+		!gconf_value_get_bool (disable_javascript),
+		NULL
+	);
+}
+
+/**
+ * Update the settings object if the preferences change.
+ * This will affect all the webviews as they all use the same
+ * settings object.
+ */
+static void
+liferea_webkit_enable_plugins_cb (GConfClient *client,
+				  guint cnxn_id,
+				  GConfEntry *entry,
+				  gpointer user_data)
+{
+	GConfValue *enable_plugins;
+
+	g_return_if_fail (entry != NULL);
+
+	enable_plugins = gconf_entry_get_value (entry);
+	if (!enable_plugins || enable_plugins->type != GCONF_VALUE_BOOL)
+		return;
+
+	g_object_set (
+		settings,
+		"enable-plugins",
+		gconf_value_get_bool (enable_plugins),
+		NULL
+	);
+}
+
+static gchar *
+webkit_get_font (guint *size)
+{
+	gchar *font = NULL;
+
+	*size = 11;	/* default fallback */
+
+	/* font configuration support */
+	conf_get_str_value (USER_FONT, &font);
+	if (0 == strlen (font)) {
+		g_free (font);
+		conf_get_str_value (DEFAULT_FONT, &font);
+	}
+
+	if (font) {
+		/* The GTK2/GNOME font name format is "<font name> <size>" */
+		gchar *tmp = strrchr(font, ' ');
+		if (tmp) {
+			*tmp++ = 0;
+			*size = atoi(tmp);
+		}
+	}
+
+	return font;
+}
+
 /**
  * HTML plugin init method
  */
 static void
-webkit_init (void)
+liferea_webkit_init (void)
 {
-	gboolean disable_javascript, enable_plugins;
+	gboolean	disable_javascript, enable_plugins;
+	GConfClient	*client;
+	gchar		*font;
+	guint		fontSize;
 
 	g_assert (!settings);
 
 	settings = webkit_web_settings_new ();
+	font = webkit_get_font (&fontSize);
 
-	g_object_set (
-		settings,
-		"default-font-size",
-		11,
-		NULL
-	);
+	if (font) {
+		g_object_set (
+			settings,
+			"default-font-family",
+			font,
+			NULL
+		);
+		g_object_set (
+			settings,
+			"default-font-size",
+			fontSize,
+			NULL
+		);
+		g_free (font);
+	}
 	g_object_set (
 		settings,
 		"minimum-font-size",
 		7,
 		NULL
 	);
-	disable_javascript = conf_get_bool_value (DISABLE_JAVASCRIPT);
+	conf_get_bool_value (DISABLE_JAVASCRIPT, &disable_javascript);
 	g_object_set (
 		settings,
 		"enable-scripts",
 		!disable_javascript,
 		NULL
 	);
-	enable_plugins = conf_get_bool_value (ENABLE_PLUGINS);
+	conf_get_bool_value (ENABLE_PLUGINS, &enable_plugins);
 	g_object_set (
 		settings,
 		"enable-plugins",
 		enable_plugins,
 		NULL
 	);
+
+
+	client = gconf_client_get_default ();
+
+	gconf_client_notify_add (
+		client,
+		DISABLE_JAVASCRIPT,
+		liferea_webkit_disable_javascript_cb,
+		NULL, NULL, NULL
+	);
+	gconf_client_notify_add (
+		client,
+		ENABLE_PLUGINS,
+		liferea_webkit_enable_plugins_cb,
+		NULL, NULL, NULL
+	);
+
+	g_object_unref (client);
 }
 
 /**
@@ -79,7 +192,7 @@ webkit_init (void)
  * HTML documents created internally.
  */
 static void
-webkit_write_html (
+liferea_webkit_write_html (
 	GtkWidget *scrollpane,
 	const gchar *string,
 	const guint length,
@@ -100,43 +213,46 @@ webkit_write_html (
 }
 
 static void
-webkit_title_changed (
-	WebKitWebView *view,
-	WebKitWebFrame *frame,
-	const gchar *title,
-	gpointer user_data
-)
+liferea_webkit_title_changed (WebKitWebView *view, GParamSpec *pspec, gpointer user_data)
 {
 	LifereaHtmlView	*htmlview;
+	gchar *title;
 
 	htmlview = g_object_get_data (G_OBJECT (view), "htmlview");
+	g_object_get (view, "title", &title, NULL);
+
 	liferea_htmlview_title_changed (htmlview, title);
+	g_free (title);
 }
 
 static void
-webkit_progress_changed (WebKitWebView *view, gint progress, gpointer user_data)
+liferea_webkit_progress_changed (WebKitWebView *view, gint progress, gpointer user_data)
 {
 }
 
 static void
-webkit_location_changed (WebKitWebView *view, const gchar *location)
+liferea_webkit_location_changed (WebKitWebView *view, GParamSpec *pspec, gpointer user_data)
 {
 	LifereaHtmlView	*htmlview;
+	gchar *location;
 
 	htmlview = g_object_get_data (G_OBJECT (view), "htmlview");
+	g_object_get (view, "uri", &location, NULL);
+
 	liferea_htmlview_location_changed (htmlview, location);
+	g_free (location);
 }
 
 /**
  * Action executed when user hovers over a link
  */
 static void
-webkit_on_url (WebKitWebView *view, const gchar *title, const gchar *url, gpointer user_data)
+liferea_webkit_on_url (WebKitWebView *view, const gchar *title, const gchar *url, gpointer user_data)
 {
 	LifereaHtmlView	*htmlview;
 	gchar *selected_url;
 
-	htmlview    = g_object_get_data (G_OBJECT (view), "htmlview");
+	htmlview = g_object_get_data (G_OBJECT (view), "htmlview");
 	selected_url = g_object_get_data (G_OBJECT (view), "selected_url");
 	if (selected_url)
 		g_free (selected_url);
@@ -152,15 +268,15 @@ webkit_on_url (WebKitWebView *view, const gchar *title, const gchar *url, gpoint
 /**
  * A link has been clicked
  *
- * When a link has been clicked the the link management is dispatched to Liferea core in
- * order to manage the different filetypes, remote URLs.
+ * When a link has been clicked the link management is dispatched to Liferea
+ * core in order to manage the different filetypes, remote URLs.
  */
 static gboolean
-webkit_link_clicked (WebKitWebView *view,
-		     WebKitWebFrame *frame,
-		     WebKitNetworkRequest *request,
-		     WebKitWebNavigationAction *navigation_action,
-		     WebKitWebPolicyDecision *policy_decision)
+liferea_webkit_link_clicked (WebKitWebView *view,
+			     WebKitWebFrame *frame,
+			     WebKitNetworkRequest *request,
+			     WebKitWebNavigationAction *navigation_action,
+			     WebKitWebPolicyDecision *policy_decision)
 {
 	const gchar			*uri;
 	WebKitWebNavigationReason	reason;
@@ -173,7 +289,6 @@ webkit_link_clicked (WebKitWebView *view,
 
 	/* iframes in items return WEBKIT_WEB_NAVIGATION_REASON_OTHER
 	   and shouldn't be handled as clicks                          */
-
 	if (reason != WEBKIT_WEB_NAVIGATION_REASON_LINK_CLICKED)
 		return FALSE;
 
@@ -193,17 +308,6 @@ webkit_link_clicked (WebKitWebView *view,
 	return url_handled;
 }
 
-
-/**
- * FIXME: when https://bugs.webkit.org/show_bug.cgi?id=14807 is fixed,
- * do this properly.
- */
-static void
-webkit_load_committed (WebKitWebView *view, WebKitWebFrame *frame)
-{
-	webkit_location_changed (view, webkit_web_frame_get_uri (frame));
-}
-
 /**
  * A new window was requested. This is the case e.g. if the link
  * has target="_blank". In that case, we don't open the link in a new
@@ -261,22 +365,29 @@ webkit_create_web_view (WebKitWebView *view, WebKitWebFrame *frame)
  * Add menu items to #menu to extend the context menu.
  */
 static void
-webkit_on_menu (WebKitWebView *view, GtkMenu *menu)
+liferea_webkit_on_menu (WebKitWebView *view, GtkMenu *menu)
 {
-	LifereaHtmlView	*htmlview;
-	gchar		*selected_url;
+	LifereaHtmlView			*htmlview;
+	gchar				*imageUri = NULL;
+	gchar				*linkUri = NULL;
+	WebKitHitTestResult*		hitResult;
+	WebKitHitTestResultContext	context;
+	GdkEvent			*event;
 
+	event = gtk_get_current_event ();
+	hitResult = webkit_web_view_get_hit_test_result (view, (GdkEventButton *)event);
+	g_object_get (hitResult, "context", &context, NULL);
+
+	if (context & WEBKIT_HIT_TEST_RESULT_CONTEXT_LINK)
+		g_object_get (hitResult, "link-uri", &linkUri, NULL);
+	if (context & WEBKIT_HIT_TEST_RESULT_CONTEXT_IMAGE)
+		g_object_get (hitResult, "image-uri", &imageUri, NULL);
+	if (context & WEBKIT_HIT_TEST_RESULT_CONTEXT_MEDIA)
+		g_object_get (hitResult, "media-uri", &linkUri, NULL);		/* treat media as normal link */
+		
 	htmlview = g_object_get_data (G_OBJECT (view), "htmlview");
 	
-	selected_url = g_object_get_data (G_OBJECT (view), "selected_url");	
-
-	/* don't pass empty URLs */
-	if (selected_url && strlen (selected_url) == 0)
-		selected_url = NULL;
-	else
-		selected_url = g_strdup (selected_url);
-		
-	liferea_htmlview_prepare_context_menu (htmlview, menu, selected_url);
+	liferea_htmlview_prepare_context_menu (htmlview, menu, linkUri, imageUri);
 }
 
 /**
@@ -286,11 +397,10 @@ webkit_on_menu (WebKitWebView *view, GtkMenu *menu)
  * And we ignore them.
  */
 static gboolean
-webkit_javascript_message  (WebKitWebView *view,
-			    const char *message,
-			    int line,
-			    const char *source_id
-			    )
+liferea_webkit_javascript_message  (WebKitWebView *view,
+				    const char *message,
+				    int line,
+				    const char *source_id)
 {
 	return TRUE;
 }
@@ -302,7 +412,7 @@ webkit_javascript_message  (WebKitWebView *view,
  * and embeds WebKitWebView into it.
  */
 static GtkWidget *
-webkit_new (LifereaHtmlView *htmlview)
+liferea_webkit_new (LifereaHtmlView *htmlview)
 {
 	WebKitWebView *view;
 	GtkWidget *scrollpane;
@@ -322,7 +432,7 @@ webkit_new (LifereaHtmlView *htmlview)
 	/** Create HTML widget and pack it into the scrolled window */
 	view = WEBKIT_WEB_VIEW (webkit_web_view_new ());
 
-	webkit_web_view_set_settings (view, WEBKIT_WEB_SETTINGS (settings));
+	webkit_web_view_set_settings (view, settings);
 
 	gtk_container_add (GTK_CONTAINER (scrollpane), GTK_WIDGET (view));
 
@@ -336,26 +446,26 @@ webkit_new (LifereaHtmlView *htmlview)
 	/** Connect signal callbacks */
 	g_signal_connect (
 		view,
-		"title-changed",
-		G_CALLBACK (webkit_title_changed),
+		"notify::title",
+		G_CALLBACK (liferea_webkit_title_changed),
 		view
 	);
 	g_signal_connect (
 		view,
 		"load-progress-changed",
-		G_CALLBACK (webkit_progress_changed),
+		G_CALLBACK (liferea_webkit_progress_changed),
 		view
 	);
 	g_signal_connect (
 		view,
 		"hovering-over-link",
-		G_CALLBACK (webkit_on_url),
+		G_CALLBACK (liferea_webkit_on_url),
 		view
 	);
 	g_signal_connect (
 		view,
 		"navigation-policy-decision-requested",
-		G_CALLBACK (webkit_link_clicked),
+		G_CALLBACK (liferea_webkit_link_clicked),
 		view
 	);
 	g_signal_connect (
@@ -367,19 +477,19 @@ webkit_new (LifereaHtmlView *htmlview)
 	g_signal_connect (
 		view,
 		"populate-popup",
-		G_CALLBACK (webkit_on_menu),
+		G_CALLBACK (liferea_webkit_on_menu),
 		view
 	);
 	g_signal_connect (
 		view,
-		"load-committed",
-		G_CALLBACK (webkit_load_committed),
+		"notify::uri",
+		G_CALLBACK (liferea_webkit_location_changed),
 		view
 	);
 	g_signal_connect (
 		view,
 		"console-message",
-		G_CALLBACK (webkit_javascript_message),
+		G_CALLBACK (liferea_webkit_javascript_message),
 		view
 	);
 	g_signal_connect (
@@ -397,7 +507,7 @@ webkit_new (LifereaHtmlView *htmlview)
  * Launch URL
  */
 static void
-webkit_launch_url (GtkWidget *scrollpane, const gchar *url)
+liferea_webkit_launch_url (GtkWidget *scrollpane, const gchar *url)
 {
 	// FIXME: hack to make URIs like "gnome.org" work
 	// https://bugs.webkit.org/show_bug.cgi?id=24195
@@ -408,12 +518,6 @@ webkit_launch_url (GtkWidget *scrollpane, const gchar *url)
 		http_url = g_strdup (url);
 	}
 
-	// Register the new url for the history
-	webkit_location_changed (
-		WEBKIT_WEB_VIEW (gtk_bin_get_child (GTK_BIN (scrollpane))),
-		http_url
-	);
-
 	webkit_web_view_load_uri (
 		WEBKIT_WEB_VIEW (gtk_bin_get_child (GTK_BIN (scrollpane))),
 		http_url
@@ -426,18 +530,49 @@ webkit_launch_url (GtkWidget *scrollpane, const gchar *url)
  * Change zoom level of the HTML scrollpane
  */
 static void
-webkit_change_zoom_level (GtkWidget *scrollpane, gfloat zoom_level)
+liferea_webkit_change_zoom_level (GtkWidget *scrollpane, gfloat zoom_level)
 {
 	WebKitWebView *view;
 	view = WEBKIT_WEB_VIEW (gtk_bin_get_child (GTK_BIN (scrollpane)));
 	webkit_web_view_set_zoom_level (view, zoom_level);
 }
 
+/**
+ * Return whether text is selected
+ */
+static gboolean
+liferea_webkit_has_selection (GtkWidget *scrollpane)
+{
+	WebKitWebView *view;
+	view = WEBKIT_WEB_VIEW (gtk_bin_get_child (GTK_BIN (scrollpane)));
+
+	/* 
+	   Currently (libwebkit-1.0 1.2.0) this doesn't work:
+
+		return webkit_web_view_has_selection (view);
+
+	   So we use *_can_copy_clipboard() as a workaround.
+	*/
+	
+	return webkit_web_view_can_copy_clipboard (view);
+}
+
+/**
+ * Copy selected text to the clipboard
+ */
+static void
+liferea_webkit_copy_selection (GtkWidget *scrollpane)
+{
+	WebKitWebView *view;
+	view = WEBKIT_WEB_VIEW (gtk_bin_get_child (GTK_BIN (scrollpane)));
+	webkit_web_view_copy_clipboard (view);
+}
+
 /**
  * Return current zoom level as a float
  */
 static gfloat
-webkit_get_zoom_level (GtkWidget *scrollpane)
+liferea_webkit_get_zoom_level (GtkWidget *scrollpane)
 {
 	WebKitWebView *view;
 	view = WEBKIT_WEB_VIEW (gtk_bin_get_child (GTK_BIN (scrollpane)));
@@ -450,7 +585,7 @@ webkit_get_zoom_level (GtkWidget *scrollpane)
  * Copied from gtkhtml/gtkhtml.c
  */
 static gboolean
-webkit_scroll_pagedown (GtkWidget *scrollpane)
+liferea_webkit_scroll_pagedown (GtkWidget *scrollpane)
 {
 	GtkScrolledWindow *itemview;
 	GtkAdjustment *vertical_adjustment;
@@ -462,8 +597,8 @@ webkit_scroll_pagedown (GtkWidget *scrollpane)
 	g_assert (NULL != itemview);
 	vertical_adjustment = gtk_scrolled_window_get_vadjustment (itemview);
 	old_value = gtk_adjustment_get_value (vertical_adjustment);
-	new_value = old_value + vertical_adjustment->page_increment;
-	limit = vertical_adjustment->upper - vertical_adjustment->page_size;
+	new_value = old_value + gtk_adjustment_get_page_increment (vertical_adjustment);
+	limit = gtk_adjustment_get_upper (vertical_adjustment) - gtk_adjustment_get_page_size (vertical_adjustment);
 	if (new_value > limit) {
 		new_value = limit;
 	}
@@ -476,7 +611,7 @@ webkit_scroll_pagedown (GtkWidget *scrollpane)
 }
 
 static void
-webkit_set_proxy (const gchar *host, guint port, const gchar *user, const gchar *pwd)
+liferea_webkit_set_proxy (const gchar *host, guint port, const gchar *user, const gchar *pwd)
 {
 	SoupURI *proxy = NULL;
 
@@ -496,15 +631,17 @@ webkit_set_proxy (const gchar *host, guint port, const gchar *user, const gchar
 
 static struct
 htmlviewImpl webkitImpl = {
-	.init		= webkit_init,
-	.create		= webkit_new,
-	.write		= webkit_write_html,
-	.launch		= webkit_launch_url,
-	.zoomLevelGet	= webkit_get_zoom_level,
-	.zoomLevelSet	= webkit_change_zoom_level,
-	.scrollPagedown	= webkit_scroll_pagedown,
-	.setProxy	= webkit_set_proxy,
-	.setOffLine	= NULL
+	.init		= liferea_webkit_init,
+	.create		= liferea_webkit_new,
+	.write		= liferea_webkit_write_html,
+	.launch		= liferea_webkit_launch_url,
+	.zoomLevelGet	= liferea_webkit_get_zoom_level,
+	.zoomLevelSet	= liferea_webkit_change_zoom_level,
+	.hasSelection	= liferea_webkit_has_selection,
+	.copySelection	= liferea_webkit_copy_selection,
+	.scrollPagedown	= liferea_webkit_scroll_pagedown,
+	.setProxy	= liferea_webkit_set_proxy,
+	.setOffLine	= NULL // FIXME: blocked on https://bugs.webkit.org/show_bug.cgi?id=18893
 };
 
 DECLARE_HTMLVIEW_IMPL (webkitImpl);
