@@ -23,7 +23,6 @@
 
 #include <string.h>
 #include <errno.h>
-#include <unistd.h>	/* fsync */
 #include <glib/gstdio.h>
 
 #include <libxml/xmlerror.h>
@@ -118,7 +117,7 @@ xhtml_extract (xmlNodePtr xml, gint xhtmlMode, const gchar *defaultBase)
 				body = xhtml_find_body (oldDoc);
 	
 				/* Copy namespace from original documents root node. This is
-				   ro determine additional namespaces for item content. For
+				   to determine additional namespaces for item content. For
 				   example to handle RSS 2.0 feeds as provided by LiveJournal:
 
 				   <rss version='2.0' xmlns:lj='http://www.livejournal.org/rss/lj/1.0/'>
@@ -225,7 +224,7 @@ xhtml_is_well_formed (const gchar *data)
 
 	xml = g_strdup_printf ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n<test>%s</test>", data);
 	
-	doc = xml_parse (xml, strlen (xml), FALSE, errors);
+	doc = xml_parse (xml, strlen (xml), errors);
 	if (doc)
 		xmlFreeDoc (doc);
 		
@@ -238,6 +237,7 @@ xhtml_is_well_formed (const gchar *data)
 }
 
 static GSList *dhtml_strippers = NULL;
+static GSList *unsupported_tag_strippers = NULL;
 
 static void
 xhtml_stripper_add (GSList **strippers, const gchar *pattern)
@@ -291,6 +291,16 @@ xhtml_strip_dhtml (const gchar *html)
 	return xhtml_strip (html, dhtml_strippers);
 }
 
+gchar *
+xhtml_strip_unsupported_tags (const gchar *html)
+{
+	if (!unsupported_tag_strippers) {
+		xhtml_stripper_add(&unsupported_tag_strippers, "<\\s*/?wbr[^>]*/?\\s*>");
+	}
+	
+	return xhtml_strip(html, unsupported_tag_strippers);
+}
+
 typedef struct {
 	gchar	*data;
 	gint	length;
@@ -351,8 +361,6 @@ unmarkupize (gchar *string, void(*parse)(gchar *string, result_buffer *buffer))
 	if (!string)
 		return NULL;
 		
-	string = common_utf8_fix (string);
-
 	/* only do something if there are any entities or tags */
 	if(NULL == (strpbrk (string, "&<>")))
 		return string;
@@ -477,7 +485,7 @@ xpath_find (xmlNodePtr node, const gchar *expr)
 }
 
 gboolean
-xpath_foreach_match (xmlNodePtr node, gchar *expr, xpathMatchFunc func, gpointer user_data)
+xpath_foreach_match (xmlNodePtr node, const gchar *expr, xpathMatchFunc func, gpointer user_data)
 {
 	
 	if (node && node->doc) {
@@ -507,25 +515,17 @@ xpath_foreach_match (xmlNodePtr node, gchar *expr, xpathMatchFunc func, gpointer
 gchar *
 xml_get_attribute (xmlNodePtr node, const gchar *name)
 {
-	/* For RSS we use libxml2 in recovery mode and we need to
-	   ensure to never get invalid characters, which is why we
-	   use common_utf8_fix() on each result here... */
-	   
-	return common_utf8_fix (xmlGetProp (node, BAD_CAST name));
+	return xmlGetProp (node, BAD_CAST name);
 }
 
 gchar *
 xml_get_ns_attribute (xmlNodePtr node, const gchar *name, const gchar *namespace)
 {
-	/* For RSS we use libxml2 in recovery mode and we need to
-	   ensure to never get invalid characters, which is why we
-	   use common_utf8_fix() on each result here... */
-	   
-	return common_utf8_fix (xmlGetNsProp (node, BAD_CAST name, BAD_CAST namespace));
+	return xmlGetNsProp (node, BAD_CAST name, BAD_CAST namespace);
 }
 
 xmlDocPtr
-xml_parse (gchar *data, size_t length, gboolean recovery, errorCtxtPtr errCtx)
+xml_parse (gchar *data, size_t length, errorCtxtPtr errCtx)
 {
 	xmlParserCtxtPtr	ctxt;
 	xmlDocPtr		doc;
@@ -538,7 +538,7 @@ xml_parse (gchar *data, size_t length, gboolean recovery, errorCtxtPtr errCtx)
 	if (errCtx)
 		xmlSetGenericErrorFunc (errCtx, (xmlGenericErrorFunc)xml_buffer_parse_error);
 	
-	doc = xmlSAXParseMemory (ctxt->sax, data, length, /* recovery = */ recovery);
+	doc = xmlSAXParseMemory (ctxt->sax, data, length, 0);
 	
 	/* This seems to reset the errorfunc to its default, so that the
 	   GtkHTML2 module is not unhappy because it also tries to call the
@@ -570,7 +570,7 @@ xml_parse_feed (feedParserCtxtPtr fpc)
 	errors = g_new0 (struct errorCtxt, 1);
 	errors->msg = fpc->feed->parseErrors;
 	
-	fpc->doc = xml_parse (fpc->data, (size_t)fpc->dataLength, fpc->recovery, errors);
+	fpc->doc = xml_parse (fpc->data, (size_t)fpc->dataLength, errors);
 	if (!fpc->doc) {
 		debug1 (DEBUG_PARSING, "xml_parse_feed(): could not parse feed \"%s\"!", fpc->subscription->node->title);
 		g_string_prepend (fpc->feed->parseErrors, _("XML Parser: Could not parse document:\n"));
@@ -592,57 +592,3 @@ xml_init (void)
 	/* has to be called for multithreaded programs */
 	xmlInitParser ();
 }
-
-/* The following slightly modified function was originally taken 
- * from GConf 2.16.0 backends/xml-dir.c: Copyright (C) 1999, 2000 Red Hat Inc. */
-
-/* for info on why this is used rather than xmlDocDump or xmlSaveFile
- * and friends, see http://bugzilla.gnome.org/show_bug.cgi?id=108329 */
-gint
-xml_save_to_file (xmlDocPtr doc, gchar *filename)
-{
-	FILE	*fp;
-	char	*xmlbuf;
-	int	fd, n;
-
-	fp = g_fopen (filename, "w");
-	if (NULL == fp)
-		return -1;
-  
-	xmlDocDumpFormatMemory (doc, (xmlChar **)&xmlbuf, &n, TRUE);
-	if (n <= 0) {
-		errno = ENOMEM;
-		return -1;
-	}
-
-	if (fwrite (xmlbuf, sizeof (xmlChar), n, fp) < n) {
-		xmlFree (xmlbuf);
-		return -1;
-	}
-
-	xmlFree (xmlbuf);
-
-	/* From the fflush(3) man page:
-	*
-	* Note that fflush() only flushes the user space buffers provided by the
-	* C library. To ensure that the data is physically stored on disk the
-	* kernel buffers must be flushed too, e.g. with sync(2) or fsync(2).
-	*/
-
-	/* flush user-space buffers */
-	if (fflush (fp) != 0)
-		return -1;
-
-	if ((fd = fileno (fp)) == -1)
-		return -1;
-
-#ifdef HAVE_FSYNC
-	/* sync kernel-space buffers to disk */
-	if (fsync (fd) == -1)
-		return -1;
-#endif
-
-	fclose (fp);
-
-	return 0;
-}
